<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "createdAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "type": "exact",
 *     "name": "ipartial",
 *     "isActive": "exact",
 *     "category": "exact",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "type",
 *     "properties",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "name",
 *     "createdAt",
 *     "type",
 *     "isActive",
 *     "category",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class,
 *  arguments={
 *      "parameterName": "properties",
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "isActive",
 *          "category",
 *          "name",
 *          "type",
 *          "createdAt",
 *      }
 *  }
 * )
 * @ORM\Table(
 *   name="collectable",
 *   indexes={
 *     @ORM\Index(
 *       name="collectable_name",
 *       columns={"name"}
 *     ),
 *     @ORM\Index(
 *       name="collectable_type",
 *       columns={"type"}
 *     ),
 *     @ORM\Index(
 *       name="collectable_is_active",
 *       columns={"is_active"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\CollectableRepository")
 */
class Collectable
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read"})
     */
    private $name;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $parentId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Collectable", inversedBy="collectables")
     * @Groups({"read"})
     */
    private $parent;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Collectable", mappedBy="parent")
     * @Groups({"read"})
     */
    private $collectables;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Groups({"read"})
     */
    private $type;

    /**
     * @ORM\Column(type="json", nullable=true, options={"jsonb"=true})
     * @Groups({"read"})
     */
    private $properties = [];

    /**
     * @ORM\Column(type="boolean", options={"default" : true})
     * @Groups({"read"})
     */
    private $isActive = false;

    /**
     * @ORM\Column(type="string", length=255, nullable=true, options={"default" : ""})
     * @Groups({"read"})
     */
    private $category = '';

    public function __construct()
    {
        $this->collectables = new ArrayCollection();
    }

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getParentId(): ?string
    {
        return $this->parentId;
    }

    public function setParentId(?string $parentId): self
    {
        $this->parentId = $parentId;

        return $this;
    }

    public function getParent(): ?self
    {
        return $this->parent;
    }

    public function setParent(?self $parent): self
    {
        $this->parent = $parent;

        return $this;
    }

    /**
     * @return Collection|self[]
     */
    public function getCollectables(): Collection
    {
        return $this->collectables;
    }

    public function addCollectable(self $collectable): self
    {
        if (!$this->collectables->contains($collectable)) {
            $this->collectables[] = $collectable;
            $collectable->setParent($this);
        }

        return $this;
    }

    public function removeCollectable(self $collectable): self
    {
        if ($this->collectables->contains($collectable)) {
            $this->collectables->removeElement($collectable);
            // set the owning side to null (unless already changed)
            if ($collectable->getParent() === $this) {
                $collectable->setParent(null);
            }
        }

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getProperties(): ?array
    {
        return $this->properties;
    }

    public function setProperties(?array $properties): self
    {
        $this->properties = $properties;

        return $this;
    }

    public function getIsActive(): ?bool
    {
        return $this->isActive;
    }

    public function setIsActive(?bool $isActive): self
    {
        $this->isActive = $isActive;

        return $this;
    }

    public function getCategory(): ?string
    {
        return $this->category;
    }

    public function setCategory(?string $category): self
    {
        $this->category = $category;

        return $this;
    }
}
