<?php declare(strict_types=1);

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\ORM\Mapping as ORM;

use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;

use ApiPlatform\Core\Annotation\ApiProperty;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "registeredAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "managerId": "exact",
 *     "workerId": "exact",
 *     "greenhouseId": "exact",
 *     "chemicalId": "exact",
 *     "type": "exact",
 *     "registeredAt": "exact",
 *     "empasNumber": "ipartial",
 *     "manager.name": "ipartial",
 *     "worker.name": "ipartial",
 *     "chemical.name": "ipartial",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "type",
 *     "empasNumber",
 *     "registeredAt",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class, 
 *  arguments={
 *      "parameterName": "properties", 
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "type",
 *          "empasNumber",
 *          "registeredAt",
 *          "managerId",
 *          "workerId",
 *          "worker",
 *          "chemicalId",
 *      }
 *  }
 * )
 * @ORM\Table(
 *   name="empas",
 *   indexes={
 *     @ORM\Index(
 *       name="empas_manager_id",
 *       columns={"manager_id"}
 *     ),
 *     @ORM\Index(
 *       name="empas_worker_id",
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="empas_chemical_id",
 *       columns={"chemical_id"}
 *     ),
 *     @ORM\Index(
 *       name="empas_registered_at",
 *       columns={"registered_at"}
 *     ),
 *     @ORM\Index(
 *       name="empas_empas_number",
 *       columns={"empas_number"}
 *     ),
 *     @ORM\Index(
 *       name="empas_type",
 *       columns={"type"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\EmpasRepository")
 */
class Empas
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $managerId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Manager")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $manager;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $workerId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $worker;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $greenhouseId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Greenhouse")
     * @ORM\JoinColumn(nullable=false)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $greenhouse;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $chemicalId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Chemical")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $chemical;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read"})
     */
    private $empasNumber;

    /**
     * @ORM\Column(type="string", length=6, nullable=true)
     * @Groups({"read"})
     */
    private $type;

    /**
     * @ORM\Column(type="datetimetz")
     * @Groups({"read"})
     */
    private $registeredAt;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read"})
     */
    private $sprayConcentration;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read"})
     */
    private $sprayPressure;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read"})
     */
    private $robotSpeed;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read"})
     */
    private $sprayVolume;

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getManagerId(): ?string
    {
        return $this->managerId;
    }

    public function setManagerId(?string $managerId): self
    {
        $this->managerId = $managerId;

        return $this;
    }

    public function getManager(): ?Manager
    {
        return $this->manager;
    }

    public function setManager(?Manager $manager): self
    {
        $this->manager = $manager;

        return $this;
    }

    public function getGreenhouseId(): ?string
    {
        return $this->greenhouseId;
    }

    public function setGreenhouseId(?string $greenhouseId): self
    {
        $this->greenhouseId = $greenhouseId;

        return $this;
    }

    public function getGreenhouse(): ?Greenhouse
    {
        return $this->greenhouse;
    }

    public function setGreenhouse(?Greenhouse $greenhouse): self
    {
        $this->greenhouse = $greenhouse;

        return $this;
    }

    public function getChemicalId(): ?string
    {
        return $this->chemicalId;
    }

    public function setChemicalId(?string $chemicalId): self
    {
        $this->chemicalId = $chemicalId;

        return $this;
    }

    public function getChemical(): ?Chemical
    {
        return $this->chemical;
    }

    public function setChemical(?Chemical $chemical): self
    {
        $this->chemical = $chemical;

        return $this;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getEmpasNumber(): ?string
    {
        return $this->empasNumber;
    }

    public function setEmpasNumber(string $empasNumber): self
    {
        $this->empasNumber = $empasNumber;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getRegisteredAt(): ?\DateTimeInterface
    {
        return $this->registeredAt;
    }

    public function setRegisteredAt(\DateTimeInterface $registeredAt): self
    {
        $this->registeredAt = $registeredAt;

        return $this;
    }

    public function getSprayConcentration(): ?string
    {
        return $this->sprayConcentration;
    }

    public function setSprayConcentration(string $sprayConcentration): self
    {
        $this->sprayConcentration = $sprayConcentration;

        return $this;
    }

    public function getSprayPressure(): ?string
    {
        return $this->sprayPressure;
    }

    public function setSprayPressure(string $sprayPressure): self
    {
        $this->sprayPressure = $sprayPressure;

        return $this;
    }

    public function getRobotSpeed(): ?string
    {
        return $this->robotSpeed;
    }

    public function setRobotSpeed(string $robotSpeed): self
    {
        $this->robotSpeed = $robotSpeed;

        return $this;
    }

    public function getSprayVolume(): ?string
    {
        return $this->sprayVolume;
    }

    public function setSprayVolume(string $sprayVolume): self
    {
        $this->sprayVolume = $sprayVolume;

        return $this;
    }
}
