<?php

namespace App\Entity;

use App\Entity\Collectable;
use App\Entity\Worker;
use App\Entity\Task;
use App\Entity\PlantPhenology;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "createdAt",
 *     "registeredAt",
 *     "updatedAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "code": "exact",
 *     "propertyId": "exact",
 *     "parentId": "exact",
 *     "workerId": "exact",
 *     "taskId": "exact",
 *     "plantId": "exact",
 *     "value": "ipartial",
 *     "worker.name": "ipartial",
 *     "plant.name": "ipartial",
 *     "plant.id": "exact",
 *     "task.greenhouseId": "exact",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "parentId",
 *     "workerId",
 *     "taskId",
 *     "plantId",
 *     "value",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "code",
 *     "propertyId",
 *     "parentId",
 *     "workerId",
 *     "taskId",
 *     "plantId",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class, 
 *  arguments={
 *      "parameterName": "properties", 
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "code",
 *          "propertyId",
 *          "parentId",
 *          "workerId",
 *          "taskId",
 *          "plantId",
 *          "value",
 *          "registeredAt",
 *          "createdAt",
 *      }
 *  }
 * )
 * @ORM\Table(
 *   name="metric",
 *   indexes={
 *     @ORM\Index(
 *       name="metric_code",
 *       columns={"code"}
 *     ),
 *     @ORM\Index(
 *       name="metric_property_id",
 *       columns={"property_id"}
 *     ),
 *     @ORM\Index(
 *       name="metric_parent_id",
 *       columns={"parent_id"}
 *     ),
 *     @ORM\Index(
 *       name="metric_worker_id",
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="metric_task_id",
 *       columns={"task_id"}
 *     ),
 *     @ORM\Index(
 *       name="metric_plant_id",
 *       columns={"plant_id"}
 *     ),
 *     @ORM\Index(
 *       name="metric_registered_at",
 *       columns={"registered_at"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\MetricRepository")
 */
class Metric
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="guid")
     * @Groups({"read"})
     */
    private $code;

    /**
     * @ORM\Column(type="guid")
     * @Groups({"read"})
     */
    private $propertyId;

    /**
     * @var Collectable|null
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Collectable")
     * @ORM\JoinColumn(name="property_id", referencedColumnName="id")
     */
    private $property;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $parentId;

    /**
     * @var Collectable|null
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Collectable")
     * @ORM\JoinColumn(name="parent_id", referencedColumnName="id")
     */
    private $parent;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $workerId;

    /**
     * @var Collectable|null
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(name="worker_id", referencedColumnName="id")
     */
    private $worker;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $taskId;

    /**
     * @var Task|null
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Task")
     * @ORM\JoinColumn(name="task_id", referencedColumnName="id")
     */
    private $task;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $plantId;

    /**
     * @var PlantPhenology|null
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\PlantPhenology")
     * @ORM\JoinColumn(name="plant_id", referencedColumnName="id")
     */
    private $plant;

    /**
     * @ORM\Column(type="text", nullable=true)
     * @Groups({"read"})
     */
    private $value;

    /**
     * @ORM\Column(type="datetimetz")
     * @Groups({"read"})
     */
    private $registeredAt;

    /**
     * @ORM\Column(type="json", nullable=true, options={"jsonb"=true})
     * @Groups({"read"})
     */
    private $raw = [];

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getPropertyId(): ?string
    {
        return $this->propertyId;
    }

    public function setPropertyId(string $propertyId): self
    {
        $this->propertyId = $propertyId;

        return $this;
    }

    public function getProperty(): ?Collectable
    {
        return $this->property;
    }

    public function setProperty(?Collectable $property): self
    {
        $this->property = $property;

        return $this;
    }

    public function getParentId(): ?string
    {
        return $this->parentId;
    }

    public function setParentId(?string $parentId): self
    {
        $this->parentId = $parentId;

        return $this;
    }

    public function getParent(): ?Collectable
    {
        return $this->parent;
    }

    public function setParent(?Collectable $parent): self
    {
        $this->parent = $parent;

        return $this;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getTaskId(): ?string
    {
        return $this->taskId;
    }

    public function setTaskId(?string $taskId): self
    {
        $this->taskId = $taskId;

        return $this;
    }

    public function getTask(): ?Task
    {
        return $this->task;
    }

    public function setTask(?Task $task): self
    {
        $this->task = $task;

        return $this;
    }

    public function getPlantId(): ?string
    {
        return $this->plantId;
    }

    public function setPlantId(?string $plantId): self
    {
        $this->plantId = $plantId;

        return $this;
    }

    public function getPlant(): ?PlantPhenology
    {
        return $this->plant;
    }

    public function setPlant(?PlantPhenology $plant): self
    {
        $this->plant = $plant;

        return $this;
    }

    public function getValue(): ?string
    {
        return $this->value;
    }

    public function setValue(?string $value): self
    {
        $this->value = $value;

        return $this;
    }

    public function getRegisteredAt(): ?\DateTimeInterface
    {
        return $this->registeredAt;
    }

    public function setRegisteredAt(\DateTimeInterface $registeredAt): self
    {
        $this->registeredAt = $registeredAt;

        return $this;
    }

    public function getRaw(): ?array
    {
        return $this->raw;
    }

    public function setRaw(?array $raw): self
    {
        $this->raw = $raw;

        return $this;
    }
}
