<?php

declare(strict_types=1);

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\ORM\Mapping as ORM;
use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "startedAt",
 *   }
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "finishedAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "workerId": "exact",
 *     "startedAt": "exact",
 *     "finishedAt": "exact",
 *     "type": "exact",
 *     "worker.name": "ipartial",
 *     "reason": "ipartial",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "startedAt",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "finishedAt",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "startedAt",
 *     "finishedAt",
 *     "type",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class, 
 *  arguments={
 *      "parameterName": "properties", 
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "startedAt",
 *          "finishedAt",
 *          "type",
 *          "workerId",
 *          "workerName",
 *          "reason",
 *      }
 *  }
 * )
 * @ORM\Table(
 *   name="mobile_login",
 *   indexes={
 *     @ORM\Index(
 *       name="mobile_login_worker_id",
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="mobile_login_started_at",
 *       columns={"started_at"}
 *     ),
 *     @ORM\Index(
 *       name="mobile_login_finished_at",
 *       columns={"finished_at"}
 *     ),
 *     @ORM\Index(
 *       name="mobile_login_type",
 *       columns={"type"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\MobileLoginRepository")
 */
class MobileLogin
{
    const SIGN_IN = 'sign_in';
    const SIGN_OUT = 'sign_out';

    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @ORM\GeneratedValue(strategy="UUID")
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read"})
     */
    private $startedAt;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read"})
     */
    private $finishedAt;

    /**
     * @ORM\Column(type="string", length=8, nullable=true)
     * @Groups({"read"})
     */
    private $type;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Groups({"read"})
     */
    private $reason;

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="create")
     * @ORM\Column(type="datetimetz")
     * @Groups({"read"})
     */
    private $createdAt;

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="update")
     * @ORM\Column(type="datetimetz")
     * @Groups({"read"})
     */
    private $updatedAt;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $workerId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(nullable=false)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $worker;

    /**
     * @Groups({"read"})
     */
    private $workerName;

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getStartedAt(): ?\DateTimeInterface
    {
        return $this->startedAt;
    }

    public function setStartedAt(?\DateTimeInterface $startedAt): self
    {
        $this->startedAt = $startedAt;

        return $this;
    }

    public function getFinishedAt(): ?\DateTimeInterface
    {
        return $this->finishedAt;
    }

    public function setFinishedAt(?\DateTimeInterface $finishedAt): self
    {
        $this->finishedAt = $finishedAt;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getReason(): ?string
    {
        return $this->reason;
    }

    public function setReason(?string $reason): self
    {
        $this->reason = $reason;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getWorkerName(): ?string
    {
        return $this->worker ? $this->worker->getName() : null;
    }
}
