<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "createdAt",
 *     "updatedAt",
 *     "registeredAt",
 *     "curedAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "taskId": "exact",
 *     "workerId": "exact",
 *     "pestId": "exact",
 *     "intensityId": "exact",
 *     "value": "ipartial",
 *     "worker.name": "ipartial",
 *     "pest.name": "ipartial",
 *     "task.name": "ipartial",
 *     "intensity.name": "ipartial",
 *     "registeredAt": "exact",
 *     "curedAt": "exact",
 *     "task.greenhouseId": "exact",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "taskId",
 *     "workerId",
 *     "pestId",
 *     "intensityId",
 *     "value",
 *     "curedAt",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "taskId",
 *     "workerId",
 *     "pestId",
 *     "intensityId",
 *     "shelfNumber",
 *     "createdAt",
 *     "updatedAt",
 *     "registeredAt",
 *     "curedAt",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class,
 *  arguments={
 *      "parameterName": "properties",
 *      "overrideDefaultProperties": false,
 *      "whitelist": null
 *  }
 * )
 * @ORM\Table(
 *   name="pest_infestation",
 *   indexes={
 *     @ORM\Index(
 *      name="pest_infestation_task_id",
 *      columns={"task_id"}
 *    ),
 *     @ORM\Index(
 *      name="pest_infestation_worker_id",
 *      columns={"worker_id"}
 *    ),
 *     @ORM\Index(
 *      name="pest_infestation_pest_id",
 *      columns={"pest_id"}
 *    ),
 *     @ORM\Index(
 *      name="pest_infestation_intensity_id",
 *      columns={"intensity_id"}
 *    ),
 *    @ORM\Index(
 *      name="pest_infestation_shelf_number",
 *      columns={"shelf_number"}
 *    ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\PestInfestationRepository")
 */
class PestInfestation
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="UUID")
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $taskId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Task")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $task;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $workerId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $worker;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $pestId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Pest")
     * @ORM\JoinColumn(nullable=true, name="pest_id", referencedColumnName="id", onDelete="CASCADE")
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $pest;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $intensityId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Intensity")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $intensity;

    /**
     * @ORM\Column(type="smallint", options={"default": 0})
     * @Groups({"read"})
     */
    private $shelfNumber = 0;

    /**
     * @ORM\Column(type="text", nullable=true)
     * @Groups({"read"})
     */
    private $value;

    /**
     * @ORM\Column(type="datetimetz", nullable=false)
     * @Groups({"read"})
     */
    private $registeredAt;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read"})
     */
    private $curedAt;

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read"})
     */
    private $raw = [];

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getTaskId(): ?string
    {
        return $this->taskId;
    }

    public function setTaskId(?string $taskId): self
    {
        $this->taskId = $taskId;

        return $this;
    }

    public function getTask(): ?Task
    {
        return $this->task;
    }

    public function setTask(?Task $task): self
    {
        $this->task = $task;

        return $this;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getPestId(): ?string
    {
        return $this->pestId;
    }

    public function setPestId(?string $pestId): self
    {
        $this->pestId = $pestId;

        return $this;
    }

    public function getPest(): ?Pest
    {
        return $this->pest;
    }

    public function setPest(?Pest $pest): self
    {
        $this->pest = $pest;

        return $this;
    }

    public function getIntensityId(): ?string
    {
        return $this->intensityId;
    }

    public function setIntensityId(?string $intensityId): self
    {
        $this->intensityId = $intensityId;

        return $this;
    }

    public function getIntensity(): ?Intensity
    {
        return $this->intensity;
    }

    public function setIntensity(?Intensity $intensity): self
    {
        $this->intensity = $intensity;

        return $this;
    }

    public function getShelfNumber(): ?int
    {
        return $this->shelfNumber;
    }

    public function setShelfNumber(int $shelfNumber): self
    {
        $this->shelfNumber = $shelfNumber;

        return $this;
    }

    public function getValue(): ?string
    {
        return $this->value;
    }

    public function setValue(?string $value): self
    {
        $this->value = $value;

        return $this;
    }

    public function getRegisteredAt(): ?\DateTimeInterface
    {
        return $this->registeredAt;
    }

    public function setRegisteredAt(\DateTimeInterface $registeredAt): self
    {
        $this->registeredAt = $registeredAt;

        return $this;
    }

    public function getCuredAt(): ?\DateTimeInterface
    {
        return $this->curedAt;
    }

    public function setCuredAt(?\DateTimeInterface $curedAt): self

    {
        $this->curedAt = $curedAt;

        return $this;
    }

    public function getRaw(): ?array
    {
        return $this->raw;
    }

    public function setRaw(array $raw): self
    {
        $this->raw = $raw;

        return $this;
    }
}
