<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Annotation\ApiFilter;
use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\BooleanFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;
use App\Uuid;
use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;
use Doctrine\ORM\Mapping\UniqueConstraint;

/**
 * @ApiResource(
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   }
 * )
 * @ApiFilter(
 *  PropertyFilter::class,
 *  arguments={
 *      "parameterName": "properties",
 *      "overrideDefaultProperties": false,
 *      "whitelist": null
 *  }
 * )
 * @ApiFilter(
 *   SearchFilter::class, 
 *   properties={
 *     "id": "exact",
 *     "workerId": "exact", 
 *     "sectionId": "exact", 
 *     "worker.taskCount": "exact",
 *     "worker.departmentId": "exact",
 *     "worker.positionId": "exact",
 *     "worker.isActive": "exact",
 *   }
 * )
 * @ApiFilter(
 *   BooleanFilter::class,
 *   properties={
 *     "isQuality",
 *   }
 * )
 * @ApiFilter(
 *   DateFilter::class, 
 *   properties={
 *     "worker.spreadTill",
 *     "period",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class, 
 *   properties={
 *     "period",
 *     "ratingInt",
 *     "rating",
 *     "sectionId",
 *     "worker.taskCount",
 *   }, 
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *   ExistsFilter::class, 
 *   properties={
 *     "workerId",
 *     "rawWorkerId",
 *     "sectionId",
 *     "rawSectionId",
 *   }
 * )
 * @ApiFilter(
 *   RangeFilter::class, 
 *   properties={
 *     "rating",
 *     "ratingInt",
 *     "worker.taskCount",
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\RatingRepository")
 * @ORM\Table(
 *   name="rating",
 *   indexes={
 *     @ORM\Index(
 *       name="rating_worker", 
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="rating_section", 
 *       columns={"section_id"}
 *     ),
 *     @ORM\Index(
 *       name="rating_raw_worker", 
 *       columns={"raw_worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="rating_raw_section", 
 *       columns={"raw_section_id"}
 *     ),
 *   },
 *   uniqueConstraints={
 *      @UniqueConstraint(name="sectionId_workerId",
 *      columns={
 *          "section_id", "worker_id", "is_quality",
 *      }),
 *   },
 * )
 */
class Rating
{
    /**
     * @var Uuid
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="datetimetz")
     */
    private $period;

    /**
     * @ORM\Column(type="float" , options={"default" : 0})
     * @Groups({"read"})
     */
    private $dynamicRating = 0;

    /**
     * @ORM\Column(type="float")
     * @Groups({"read"})
     */
    private $rating;

    /**
     * @ORM\Column(type="smallint")
     * @Groups({"read"})
     */
    private $ratingInt;

    /**
     * @ORM\Column(type="guid", nullable=true)
     */
    private $workerId;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $sectionId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Section")
     */
    private $section;

    /**
     * @Groups({"read"})
     */
    private $name;

    /**
     * @ORM\Column(type="guid", nullable=true)
     */
    private $rawSectionId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker", inversedBy="ratings")
     */
    private $worker;

    /**
     * @ORM\Column(type="guid", nullable=true)
     */
    private $rawWorkerId;

    /**
     * @ORM\Column(type="json", nullable=true)
     */
    private $raw = [];

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="create")
     * @ORM\Column(type="datetimetz")
     */
    private $createdAt;

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="update")
     * @ORM\Column(type="datetimetz")
     */
    private $updatedAt;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isQuality = false;

    /**
     * @var \DateTime $ratingUpdatedAt
     *
     * @ORM\Column(type="datetimetz", options={"default" : "CURRENT_TIMESTAMP" })
     * @Gedmo\Timestampable(on="change", field={"rating"})
     * @Groups({"read"})
     */
    private $ratingUpdatedAt;

    public function __construct()
    {
        $this->ratingUpdatedAt = new \DateTime('now', new \DateTimeZone("UTC"));
    }

    public function setId(?string $id): void
    {
        $this->id = $id;
    }

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getPeriod(): ?\DateTimeInterface
    {
        return $this->period;
    }

    public function setPeriod(\DateTimeInterface $period): self
    {
        $this->period = $period;

        return $this;
    }

    public function getDynamicRating(): ?float
    {
        return $this->dynamicRating;
    }

    public function setDynamicRating(float $dynamicRating): self
    {
        $this->dynamicRating = $dynamicRating;

        return $this;
    }

    public function getRating(): ?float
    {
        return $this->rating;
    }

    public function setRating(float $rating): self
    {
        $this->rating = $rating;

        return $this;
    }

    public function getRatingInt(): ?int
    {
        return $this->ratingInt;
    }

    public function setRatingInt(int $ratingInt): self
    {
        $this->ratingInt = $ratingInt;

        return $this;
    }

    public function getSection(): ?Section
    {
        return $this->section;
    }

    public function setSection(?Section $section): self
    {
        $this->section = $section;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->section ? $this->section->getName() : "";
    }

    public function getRawSectionId(): ?string
    {
        return $this->rawSectionId;
    }

    public function setRawSectionId(?string $rawSectionId): self
    {
        $this->rawSectionId = $rawSectionId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getRawWorkerId(): ?string
    {
        return $this->rawWorkerId;
    }

    public function setRawWorkerId(?string $rawWorkerId): self
    {
        $this->rawWorkerId = $rawWorkerId;

        return $this;
    }

    public function getRaw(): ?array
    {
        return $this->raw;
    }

    public function setRaw(?array $raw): self
    {
        $this->raw = $raw;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getSectionId(): ?string
    {
        return $this->sectionId;
    }

    public function setSectionId(?string $sectionId): self
    {
        $this->sectionId = $sectionId;

        return $this;
    }

    public function getIsQuality(): bool
    {
        return $this->isQuality;
    }

    public function setIsQuality(bool $isQuality): self
    {
        $this->isQuality = $isQuality;

        return $this;
    }

    public function getRatingUpdatedAt(): ?\DateTimeInterface
    {
        return $this->ratingUpdatedAt;
    }

    public function setRatingUpdatedAt(\DateTimeInterface $ratingUpdatedAt): self
    {
        $this->ratingUpdatedAt = $ratingUpdatedAt;

        return $this;
    }

}
