<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use App\Uuid;
use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\BooleanFilter;
use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use Symfony\Component\Serializer\Annotation\MaxDepth;

/**
 * @ApiResource(
 *  normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\SectionRepository")
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "module": "exact",
 *     "isActive": "exact",
 *     "name": "ipartial",
 *   }
 *
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "parentId",
 *     "rawParentId",
 *     "isActive"
 *   }
 * )
 * )
 * @ApiFilter(
 *   BooleanFilter::class,
 *   properties={
 *     "onPause",
 *     "isRegular",
 *     "hasQualityCheck",
 *     "isHarvesting",
 *     "isPhenology",
 *     "isGuarding",
 *     "isSpraying",
 *     "isQuality",
 *     "isActive",
 *   }
 * )
 * @ORM\Table(
 *   name="section",
 *   indexes={
 *     @ORM\Index(
 *       name="section_parent_id",
 *       columns={"parent_id"}
 *     ),
 *     @ORM\Index(
 *       name="section_is_active",
 *       columns={"is_active"}
 *     ),
 *     @ORM\Index(
 *       name="section_raw_parent_id",
 *       columns={"raw_parent_id"}
 *     ),
 *   }
 * )
 */
class Section
{
    /**
     * @var Uuid
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @Groups({"read"})
     * @ORM\Column(type="string", length=255)
     */
    private $name;

    /**
     * @ORM\Column(type="json", nullable=true)
     */
    private $raw = [];

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="create")
     * @ORM\Column(type="datetimetz")
     */
    private $createdAt;

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="update")
     * @ORM\Column(type="datetimetz")
     */
    private $updatedAt;

    /**
     * @ORM\Column(type="guid", nullable=true)
     */
    private $parentId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Section")
     */
    private $parent;

    /**
     * @Groups({"read"})
     */
    private $structure = [];

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Groups({"read"})
     */
    private $description;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $requireFinish;

    /**
     * @ORM\Column(type="guid", nullable=true)
     */
    private $rawParentId;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Application", mappedBy="section")
     */
    private $applications = [];

    /**
     * @Groups({"read"})
     */
    private $positions = [];

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\GreenhouseSection", mappedBy="section")
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $greenhouseSections;

    /**
     * @ORM\Column(type="string", length=24, nullable=true)
     * @Groups({"read"})
     */
    private $module;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isActive = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isHarvesting = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isRegular = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isPhenology = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isGuarding = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isSpraying = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $isQuality = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $hasQualityCheck = false;

    public function __construct()
    {
        $this->greenhouseSections = new ArrayCollection();
    }

    public function setId(?string $id): void
    {
        $this->id = $id;
    }

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getRaw(): ?array
    {
        return $this->raw;
    }

    public function setRaw(?array $raw): self
    {
        $this->raw = $raw;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function getParentId(): ?string
    {
        return $this->parentId;
    }

    public function setParentId(?string $parentId): self
    {
        $this->parentId = $parentId;

        return $this;
    }

    public function getParent(): ?self
    {
        return $this->parent;
    }

    public function setParent(?self $parent): self
    {
        $this->parent = $parent;

        return $this;
    }

    public function getStructure(): ?array
    {
        $structure = $this->parent && $this->parent->getStructure() ?
            $this->parent->getStructure() :
            [];

        $structure[] = [
          'level' => count($structure),
          'id' => $this->getId(),
          'name' => $this->getName(),
          'parentId' => $this->getParentId(),
        ];

        return $structure;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getRequireFinish(): ?bool
    {
        return $this->requireFinish;
    }

    public function setRequireFinish(?bool $requireFinish): self
    {
        $this->requireFinish = $requireFinish;

        return $this;
    }

    public function getRawParentId(): ?string
    {
        return $this->rawParentId;
    }

    public function setRawParentId(?string $rawParentId): self
    {
        $this->rawParentId = $rawParentId;

        return $this;
    }

    /**
     * @return Collection|Application[]
     */
    public function getApplications(): Collection
    {
        return $this->applications;
    }

    public function getPositions(): ?array
    {
        $positions = [] ;

        foreach($this->getApplications() as $app) {
            if($app->getPosition()) {
                $positions[] = $app->getPosition()->getId();
            }
        }

        return $positions;
    }

    /**
     * @return Collection|GreenhouseSection[]
     */
    public function getGreenhouseSections(): Collection
    {
        return $this->greenhouseSections;
    }

    public function addGreenhouseSection(GreenhouseSection $greenhouseSection): self
    {
        if (!$this->greenhouseSections->contains($greenhouseSection)) {
            $this->greenhouseSections[] = $greenhouseSection;
            $greenhouseSection->setSection($this);
        }

        return $this;
    }

    public function removeGreenhouseSection(GreenhouseSection $greenhouseSection): self
    {
        if ($this->greenhouseSections->contains($greenhouseSection)) {
            $this->greenhouseSections->removeElement($greenhouseSection);
            // set the owning side to null (unless already changed)
            if ($greenhouseSection->getSection() === $this) {
                $greenhouseSection->setSection(null);
            }
        }

        return $this;
    }

    public function getModule(): ?string
    {
        return $this->module;
    }

    public function setModule(?string $module): self
    {
        $this->module = $module;

        return $this;
    }

    public function getIsActive(): ?bool
    {
        return $this->isActive;
    }

    public function setIsActive(bool $isActive): self
    {
        $this->isActive = $isActive;

        return $this;
    }

    public function getIsHarvesting(): bool
    {
        return $this->isHarvesting;
    }

    public function setIsHarvesting(bool $isHarvesting): self
    {
        $this->isHarvesting = $isHarvesting;

        return $this;
    }

    public function getIsRegular(): bool
    {
        return $this->isRegular;
    }

    public function setIsRegular(bool $isRegular): self
    {
        $this->isRegular = $isRegular;

        return $this;
    }

    public function getIsPhenology(): bool
    {
        return $this->isPhenology;
    }

    public function setIsPhenology(bool $isPhenology): self
    {
        $this->isPhenology = $isPhenology;

        return $this;
    }

    public function getIsGuarding(): bool
    {
        return $this->isGuarding;
    }

    public function setIsGuarding(bool $isGuarding): self
    {
        $this->isGuarding = $isGuarding;

        return $this;
    }

    public function getIsSpraying(): bool
    {
        return $this->isSpraying;
    }

    public function setIsSpraying(bool $isSpraying): self
    {
        $this->isSpraying = $isSpraying;

        return $this;
    }

    public function getIsQuality(): bool
    {
        return $this->isQuality;
    }

    public function setIsQuality(bool $isQuality): self
    {
        $this->isQuality = $isQuality;

        return $this;
    }

    public function getHasQualityCheck(): bool
    {
        return $this->hasQualityCheck;
    }

    public function setHasQualityCheck(bool $hasQualityCheck): self
    {
        $this->hasQualityCheck = $hasQualityCheck;

        return $this;
    }

}
