<?php declare(strict_types=1);

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\ORM\Mapping as ORM;
use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;
use App\Filter\NotFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "syncedAt",
 *     "createdAt",
 *     "updatedAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "name": "ipartial",
 *     "taskId": "exact",
 *     "trolleyId": "exact",
 *     "syncedAt": "exact",
 *     "status": "exact",
 *     "toSend": "exact",
 *     "rowNumber": "exact",
 *     "trolleyNumber": "exact",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "name",
 *     "taskId",
 *     "trolleyId",
 *     "status",
 *     "syncedAt",
 *     "trolleyNumber",
 *     "rowNumber",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "id",
 *     "name",
 *     "syncedAt",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class, 
 *  arguments={
 *      "parameterName": "properties", 
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "name",
 *          "taskId",
 *          "trolleyId",
 *          "toSend",
 *          "status",
 *          "syncedAt",
 *          "trolleyNumber",
 *          "rowNumber",
 *      }
 *  }
 * )
 * @ApiFilter(
 *   NotFilter::class,
 *   properties={
 *     "id",
 *   }
 * )
 * @ORM\Table(
 *   name="sync",
 *   indexes={
 *     @ORM\Index(
 *       name="sync_name",
 *       columns={"name"}
 *     ),
 *     @ORM\Index(
 *       name="sync_task_id",
 *       columns={"task_id"}
 *     ),
 *     @ORM\Index(
 *       name="sync_trolley_id",
 *       columns={"trolley_id"}
 *     ),
 *     @ORM\Index(
 *       name="sync_synced_at",
 *       columns={"synced_at"}
 *     ),
 *     @ORM\Index(
 *       name="sync_to_send",
 *       columns={"to_send"}
 *     ),
 *     @ORM\Index(
 *       name="sync_status",
 *       columns={"status"}
 *     ),
 *     @ORM\Index(
 *       name="sync_trolley_number",
 *       columns={"trolley_number"}
 *     ),
 *     @ORM\Index(
 *       name="sync_row_number",
 *       columns={"row_number"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\SyncRepository")
 */
class Sync
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Groups({"read"})
     */
    private $name;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $taskId;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $trolleyId;

    /**
     * @ORM\OneToOne(targetEntity="App\Entity\Task", cascade={"persist", "remove"})
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $task;

    /**
     * @ORM\OneToOne(targetEntity="App\Entity\Trolley", cascade={"persist", "remove"})
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $trolley;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read"})
     */
    private $toSend = false;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Groups({"read"})
     */
    private $status;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read"})
     */
    private $syncedAt;

    /**
     * @ORM\Column(type="json_array", nullable=true, options={"jsonb"=true})
     * @Groups({"read"})
     */
    private $rawRequest = [];

    /**
     * @ORM\Column(type="json_array", nullable=true, options={"jsonb"=true})
     * @Groups({"read"})
     */
    private $rawResponse = [];

    /**
     * @ORM\Column(type="smallint", nullable=true)
     * @Groups({"read"})
     */
    private $rowNumber = null;

    /**
     * @ORM\Column(type="smallint", nullable=true)
     * @Groups({"read"})
     */
    private $trolleyNumber = null;

    /**
     * @Groups({"read"})
     */
    private $trolleyTask = null;

    /**
     * @Groups({"read"})
     */
    private $taskTrolley = null;

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }
   
    public function getTaskId(): ?string
    {
        return $this->taskId;
    }

    public function setTaskId(?string $taskId): self
    {
        $this->taskId = $taskId;

        return $this;
    }

    public function getTrolleyId(): ?string
    {
        return $this->trolleyId;
    }

    public function setTrolleyId(?string $trolleyId): self
    {
        $this->trolleyId = $trolleyId;

        return $this;
    }    

    public function getTask(): ?Task
    {
        return $this->task;
    }

    public function setTask(?Task $task): self
    {
        $this->task = $task;

        return $this;
    }

    public function getTrolley(): ?Trolley
    {
        return $this->trolley;
    }

    public function setTrolley(?Trolley $trolley): self
    {
        $this->trolley = $trolley;

        return $this;
    }

    public function getToSend(): ?bool
    {
        return $this->toSend;
    }

    public function setToSend(bool $toSend): self
    {
        $this->toSend = $toSend;

        return $this;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(?string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getSyncedAt(): ?\DateTimeInterface
    {
        return $this->syncedAt;
    }

    public function setSyncedAt(?\DateTimeInterface $syncedAt): self
    {
        $this->syncedAt = $syncedAt;

        return $this;
    }

    public function getRawRequest(): ?array
    {
        return $this->rawRequest;
    }

    public function setRawRequest(?array $rawRequest): self
    {
        $this->rawRequest = $rawRequest;

        return $this;
    }

    public function getRawResponse(): ?array
    {
        return $this->rawResponse;
    }

    public function setRawResponse(?array $rawResponse): self
    {
        $this->rawResponse = $rawResponse;

        return $this;
    }

    public function getTrolleyTask(): ?Task
    {
        return $this->trolleyTask;
    }

    public function setTrolleyTask(?Task $trolleyTask): self
    {
        $this->trolleyTask = $trolleyTask;

        return $this;
    }

    public function getTaskTrolley(): ?Trolley
    {
        return $this->taskTrolley;
    }

    public function setTaskTrolley(?Trolley $taskTrolley): self
    {
        $this->taskTrolley = $taskTrolley;

        return $this;
    }

    public function getTrolleyNumber(): ?int
    {
        return $this->trolleyNumber;
    }

    public function setTrolleyNumber(?int $trolleyNumber): self
    {
        $this->trolleyNumber = $trolleyNumber;

        return $this;
    }

    public function getRowNumber(): ?int
    {
        return $this->rowNumber;
    }

    public function setRowNumber(?int $rowNumber): self
    {
        $this->rowNumber = $rowNumber;

        return $this;
    }
}
