<?php

namespace App\Entity;

use App\Entity\Greenhouse;
use App\Entity\Structure;
use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use App\Uuid;
use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;
use App\Controller\TaskActivate;
use App\Controller\TaskDeactivate;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\BooleanFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;
use App\Filter\NotFilter;

use ApiPlatform\Core\Serializer\Filter\GroupFilter;

/**
 * @ApiResource(
 *   itemOperations={
 *     "get",
 *     "delete",
 *     "put",
 *     "patch",
 *     "patch_activate"={
 *         "method"="PATCH",
 *         "path"="/tasks/{id}/activate",
 *         "controller"=TaskActivate::class,
 *         "openapi_context" = {
 *            "parameters" = {
 *               {
 *                  "name" = "id",
 *                  "in" = "path",
 *                  "required" = "true",
 *                  "type" = "string"
 *               },
 *            }
 *         },
 *     },
 *     "patch_deactivate"={
 *         "method"="PATCH",
 *         "path"="/tasks/{id}/deactivate",
 *         "controller"=TaskDeactivate::class,
 *         "openapi_context" = {
 *            "parameters" = {
 *               {
 *                  "name" = "id",
 *                  "in" = "path",
 *                  "required" = "true",
 *                  "type" = "string"
 *               },
 *            }
 *         },
 *     },
 *   },
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "startAt",
 *     "finishAt",
 *     "startedAt",
 *     "finishedAt",
 *     "lockedTill",
 *     "worker.spreadTill",
 *   }
 * )
 * @ApiFilter(
 *   BooleanFilter::class,
 *   properties={
 *     "isQuality",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "departmentId": "exact",
 *     "sectionId": "exact",
 *     "workerId": "exact",
 *     "previousWorkerId": "exact",
 *     "positionId": "exact",
 *     "isFinished": "exact",
 *     "rowNumber": "exact",
 *     "toSend": "exact",
 *     "toExchange": "exact",
 *     "worker.name": "ipartial",
 *     "pass": "exact",
 *     "isActive": "exact",
 *     "exclusive": "exact",
 *     "greenhouseId": "exact",
 *     "worker.searchable": "ipartial",
 *     "worker.code": "exact",
 *     "status": "exact",
 *     "section.isSpraying": "exact",
 *     "section.module": "exact",
 *     "qualityTaskId": "exact",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "workerId",
 *     "previousWorkerId",
 *     "rawWorkerId",
 *     "departmentId",
 *     "rawDepartmentId",
 *     "positionId",
 *     "rawPositionId",
 *     "sectionId",
 *     "rawSectionId",
 *     "properties",
 *     "rawProperties",
 *     "rowAmount",
 *     "rowNumber",
 *     "parentId",
 *     "startedAt",
 *     "finishedAt",
 *     "greenhouseId",
 *     "status",
 *   }
 * )
 * @ApiFilter(
 *   RangeFilter::class,
 *   properties={
 *     "rowAmount",
 *     "rowNumber",
 *     "rating",
 *     "dynamicRating",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "rating",
 *     "parentId",
 *     "orderBy",
 *     "rowNumber",
 *     "exclusive",
 *     "finishedAt",
 *     "startedAt",
 *     "startAt",
 *     "finishAt",
 *     "status",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  GroupFilter::class,
 *  arguments={
 *      "parameterName": "groups",
 *      "overrideDefaultGroups": true,
 *      "whitelist": {
 *          "read",
 *          "section",
 *          "mobile",
 *      }
 *  }
 * )
 * @ApiFilter(
 *  PropertyFilter::class,
 *  arguments={
 *      "parameterName": "properties",
 *      "overrideDefaultProperties": false,
 *      "whitelist": null
 *  }
 * )
 * @ApiFilter(
 *   NotFilter::class,
 *   properties={
 *     "workerId",
 *   }
 * )
 * @ORM\Table(
 *   name="task",
 *   indexes={
 *     @ORM\Index(
 *       name="task_department_id",
 *       columns={"department_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_raw_department_id",
 *       columns={"raw_department_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_section_id",
 *       columns={"section_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_raw_section_id",
 *       columns={"raw_section_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_worker_id",
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_greenhouse_id",
 *       columns={"greenhouse_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_previous_worker_id",
 *       columns={"previous_worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_raw_worker_id",
 *       columns={"raw_worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_position_id",
 *       columns={"position_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_raw_position_id",
 *       columns={"raw_position_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_started_at",
 *       columns={"started_at"}
 *     ),
 *     @ORM\Index(
 *       name="task_finished_at",
 *       columns={"finished_at"}
 *     ),
 *     @ORM\Index(
 *       name="task_start_at",
 *       columns={"start_at"}
 *     ),
 *     @ORM\Index(
 *       name="task_finish_at",
 *       columns={"finish_at"}
 *     ),
 *     @ORM\Index(
 *       name="task_locked_till",
 *       columns={"locked_till"}
 *     ),
 *     @ORM\Index(
 *       name="task_row_number",
 *       columns={"row_number"}
 *     ),
 *     @ORM\Index(
 *       name="task_exclusive",
 *       columns={"exclusive"}
 *     ),
 *     @ORM\Index(
 *       name="task_is_finished",
 *       columns={"is_finished"}
 *     ),
 *     @ORM\Index(
 *       name="task_pass",
 *       columns={"pass"}
 *     ),
 *     @ORM\Index(
 *       name="task_is_active",
 *       columns={"is_active"}
 *     ),
 *     @ORM\Index(
 *       name="task_to_exchange",
 *       columns={"to_exchange"}
 *     ),
 *     @ORM\Index(
 *       name="task_parent_id",
 *       columns={"parent_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_status",
 *       columns={"status"}
 *     ),
 *     @ORM\Index(
 *       name="task_to_exchange_index_timed",
 *       columns={"start_at", "finish_at", "to_exchange", "is_active", "parent_id", "row_number"},
 *       options={
 *          "where": "to_exchange=true and is_active=true"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_to_exchange_index",
 *       columns={"finish_at", "to_exchange", "is_active", "parent_id", "row_number"},
 *       options={
 *          "where": "to_exchange=true and is_active=true"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_worker_tasks",
 *       columns={"worker_id", "is_finished"},
 *       options={
 *          "where": "is_active=true"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_worker_tasks_timed",
 *       columns={"start_at", "finish_at", "worker_id", "is_finished"},
 *       options={
 *          "where": "is_active=true"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_worker_tasks_not_finished",
 *       columns={"worker_id", "is_finished"},
 *       options={
 *          "where": "is_finished=false"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_worker_tasks_not_finished_timed",
 *       columns={"start_at", "finish_at", "worker_id", "is_finished"},
 *       options={
 *          "where": "is_finished=false"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_worker_task_in_progress",
 *       columns={"worker_id", "is_finished", "started_at", "finished_at", "finish_at", "start_at", "is_active"},
 *       options={
 *          "where": "is_active=true"
 *       },
 *     ),
 *     @ORM\Index(
 *       name="task_section_query",
 *       columns={"start_at", "section_id", "pass", "greenhouse_id", "row_number"},
 *     ),
 *     @ORM\Index(
 *       name="task_greenhouse_tasks",
 *       columns={"is_active", "greenhouse_id", "parent_id"},
 *     ),
 *     @ORM\Index(
 *       name="task_greenhouse_tasks_timed",
 *       columns={"start_at", "is_active", "greenhouse_id", "parent_id"},
 *     ),
 *     @ORM\Index(
 *       name="task_in_progress",
 *       columns={"id", "is_finished", "started_at", "finished_at", "row_number", "greenhouse_id", "finish_at", "start_at", "is_active"},
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\TaskRepository")
 */
class Task
{

    const PHENOLOGY_ID = 'a7ae8b33-c5a4-11ea-b88d-00505696ff79';

    /**
     * @var Uuid
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $id;

    /**
     * @ORM\Column(type="text", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $name;

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="create")
     * @ORM\Column(type="datetimetz")
     * @Groups({"read", "section", "mobile"})
     */
    private $createdAt;

    /**
     * @var \DateTime
     * @Gedmo\Timestampable(on="update")
     * @ORM\Column(type="datetimetz")
     * @Groups({"read", "section", "mobile"})
     */
    private $updatedAt;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $workerId;

    /**
     * @ORM\Column(type="float" , options={"default" : 0})
     * @Groups({"read", "section", "mobile"})
     */
    private $ratingScore = 0;

    /**
     * @ORM\Column(type="float" , options={"default" : 0})
     * @Groups({"read", "section", "mobile"})
     */
    private $ratingTime = 0;

        /**
     * @ORM\Column(type="float" , options={"default" : 0})
     * @Groups({"read", "section", "mobile"})
     */
    private $dynamicRating = 0;

    private $forceWorkerId;
    private $forceStartedAt;
    private $forceFinishedAt;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $sectionId;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $sectionName;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $rowType;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $isHarvesting = false;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $isPhenology = false;

    /**
     * @ORM\Column(type="text", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $description;

    /**
     * @ORM\Column(type="boolean")
     * @Groups({"read", "section", "mobile"})
     */
    private $requireFinish;

    /**
     * @ORM\Column(type="boolean")
     * @Groups({"read", "section", "mobile"})
     */
    private $isFinished;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $startedAt;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $finishedAt;

    /**
     * @ORM\Column(type="json_array", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $properties = [];

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $departmentId;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $requireOnlineStart;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $color;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $parentId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @MaxDepth(1)
     * @Groups({"read", "section", "mobile"})
     */
    private $worker;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Section")
     * @MaxDepth(1)
     * @Groups({"read", "section", "mobile"})
     */
    private $section;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Department")
     * @MaxDepth(1)
     * @Groups({"read", "section", "mobile"})
     */
    private $department;

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $raw = [];

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $startAt;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $finishAt;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $positionId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Position")
     * @MaxDepth(1)
     * @Groups({"read", "section", "mobile"})
     */
    private $position;

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $intervals = [];

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rawWorkerId;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rawSectionId;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rawDepartmentId;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rawPositionId;

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rawProperties = [];

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $ratingSectionId;

    /**
     * @ORM\Column(type="smallint", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rowAmount;

    /**
     * @ORM\Column(type="smallint", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $rowNumber;

    /**
     * @ORM\Column(type="float")
     * @Groups({"read", "section", "mobile"})
     */
    private $rating;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $positions = [];

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $measurements = [];

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $plants = [];

    /**
     * @ORM\Column(type="boolean")
     * @Groups({"read", "section", "mobile"})
     */
    private $toSend = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read", "section", "mobile"})
     */
    private $toExchange = false;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $workerName;

    /**
     * @Groups({"read", "section", "mobile"})
     * @ORM\Column(type="guid", nullable=true)
     */
    private $greenhouseId;

    /**
     * @var Greenhouse|null
     *
     * @Groups({"read", "section", "mobile"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Greenhouse")
     * @ORM\JoinColumn(name="greenhouse_id", referencedColumnName="id")
     */
    private $greenhouse;

    /**
     * @Groups({"read", "section", "mobile"})
     * @ORM\Column(type="guid", nullable=true)
     */
    private $qualityTaskId;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $qualityTask;

    /**
     * @var Structure|null
     *
     * @ORM\OneToOne(targetEntity="App\Entity\Structure")
     * @MaxDepth(1)
     * @ORM\JoinColumn(name="structure_id", referencedColumnName="id")
     */
    private $structureId;

    /**
     * @ORM\Column(type="integer", options={"default" : 0})
     * @Groups({"read", "section", "mobile"})
     */
    private $orderBy = 0;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $rowOrderBy = 0;

    /**
     * @ORM\Column(type="datetimetz", options={"default" : "CURRENT_TIMESTAMP"})
     * @Groups({"read", "section", "mobile"})
     */
    private $lockedTill;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $inProgress = true;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $todaySectionTasksCount = 0;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $qualifiedWorkersOnline = 0;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $day;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $shift;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $direction;

    /**
     * @ORM\Column(type="smallint", options={"default" : 0})
     * @Groups({"read", "section", "mobile"})
     */
    private $pass = 0;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read", "section", "mobile"})
     */
    private $isActive = false;

    /**
     * @ORM\Column(type="smallint", options={"default" : 0})
     * @Groups({"read", "section", "mobile"})
     */
    private $exclusive = 0;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $previousWorkerId = null;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $sync = null;

    /**
     */
    private $collectables = [];

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $taskLimit = 3;

    /**
     * @ORM\Column(type="smallint", nullable=true)
     * @Groups({"read", "section", "mobile"})
     */
    private $status;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $isGuarding = false;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $isSpraying = false;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     * @Groups({"read", "section", "mobile"})
     */
    private $isQuality = false;

    /**
     * @Groups({"read", "section", "mobile"})
     */
    private $shelfAmount = null;

    public function setId(?string $id): void
    {
        $this->id = $id;
    }

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        if ($this->workerId == null || $workerId == null) {
            $this->workerId = $workerId;
        }

        return $this;
    }

    public function setForceWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function setForceStartedAt(?\DateTimeInterface $startedAt): self
    {
        $this->startedAt = $startedAt;

        return $this;
    }

    public function setForceFinishedAt(?\DateTimeInterface $finishedAt): self
    {
        $this->finishedAt = $finishedAt;

        return $this;
    }

    public function getSectionId(): ?string
    {
        return $this->sectionId;
    }

    public function setSectionId(?string $sectionId): self
    {
        $this->sectionId = $sectionId;

        return $this;
    }

    public function getSectionName(): ?string
    {
        $this->sectionName = $this->section->getName();

        return $this->sectionName;
    }

    public function getRowType(): ?string
    {
        return $this->rowType;
    }

    public function setRowType(?string $rowType): self
    {
        $this->rowType = $rowType;

        return $this;
    }

    public function getIsHarvesting(): ?bool
    {
        $this->isHarvesting = $this->section->getIsHarvesting();

        return $this->isHarvesting;
    }

    public function getIsPhenology(): ?bool
    {
        $this->isPhenology = $this->section->getIsPhenology();

        return $this->isPhenology;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getRequireFinish(): ?bool
    {
        return $this->requireFinish;
    }

    public function setRequireFinish(bool $requireFinish): self
    {
        $this->requireFinish = $requireFinish;

        return $this;
    }

    public function getIsFinished(): ?bool
    {
        return $this->isFinished;
    }

    public function setIsFinished(bool $isFinished): self
    {
        $this->isFinished = $isFinished;

        return $this;
    }

    public function getStartedAt(): ?\DateTimeInterface
    {
        return $this->startedAt;
    }

    public function setStartedAt(?\DateTimeInterface $startedAt): self
    {
        if ($this->startedAt == null || $startedAt == null) {
            $this->startedAt = $startedAt;
        }

        return $this;
    }

    public function getFinishedAt(): ?\DateTimeInterface
    {
        return $this->finishedAt;
    }

    public function setFinishedAt(?\DateTimeInterface $finishedAt): self
    {
        // if ($this->finishedAt == null || ($finishedAt == null && $this->isFinished === false)) {
            $this->finishedAt = $finishedAt;
        // }

        return $this;
    }

    public function getProperties(): ?array
    {

        if($this->sectionId === self::PHENOLOGY_ID && !empty($this->collectables)) {
            foreach($this->collectables as $c) {
                if(
                    array_search($c->getId(), array_column($this->properties, 'id')) === false
                    ) {

                    $this->properties[] = [
                        'id' => $c->getId(),
                        'name' => $c->getName(),
                        'type' => $c->getType(),
                        'stage' => 'start',
                        'value' => '',
                        'regexp' => '',
                        'parentId' => $c->getParent() ? $c->getParent()->getId() : $c->getId(),
                        'required' => true,
                        'description' => $c->getName(),
                    ];
                }
            }
        }

        return $this->properties;
    }

    public function setProperties(?array $properties): self
    {
        $this->properties = $properties;

        return $this;
    }

    public function getDepartmentId(): ?string
    {
        return $this->departmentId;
    }

    public function setDepartmentId(?string $departmentId): self
    {
        $this->departmentId = $departmentId;

        return $this;
    }

    public function getRequireOnlineStart(): ?bool
    {
        return $this->requireOnlineStart;
    }

    public function setRequireOnlineStart(?bool $requireOnlineStart): self
    {
        $this->requireOnlineStart = $requireOnlineStart;

        return $this;
    }

    public function getColor(): ?string
    {
        $s = new \DateTime('NOW'); //$this->getStartAt();
        $f = $this->getFinishAt();

        $d = $s->diff($f, true);

        if ($d->days <= 5) {
            return '#FFC2B3';
        } elseif ($d->days <= 15) {
            return '#D3FF00';
        } elseif ($d->days <= 45) {
            return '#034500';
        } else {
            return '#FFFFFF';
        }
    }

    public function getParentId(): ?string
    {
        return $this->parentId;
    }

    public function setParentId(?string $parentId): self
    {
        $this->parentId = $parentId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getSection(): ?Section
    {
        return $this->section;
    }

    public function setSection(?Section $section): self
    {
        $this->section = $section;

        return $this;
    }

    public function getDepartment(): ?Department
    {
        return $this->department;
    }

    public function setDepartment(?Department $department): self
    {
        $this->department = $department;

        return $this;
    }

    public function getRaw(): ?array
    {
        return $this->raw;
    }

    public function setRaw(?array $raw): self
    {
        $this->raw = $raw;

        return $this;
    }

    public function getStartAt(): ?\DateTimeInterface
    {
        return $this->startAt;
    }

    public function setStartAt(?\DateTimeInterface $startAt): self
    {
        $this->startAt = $startAt;

        return $this;
    }

    public function getFinishAt(): ?\DateTimeInterface
    {
        return $this->finishAt;
    }

    public function setFinishAt(?\DateTimeInterface $finishAt): self
    {
        $this->finishAt = $finishAt;

        return $this;
    }

    public function getPositionId(): ?string
    {
        return $this->positionId;
    }

    public function setPositionId(?string $positionId): self
    {
        $this->positionId = $positionId;

        return $this;
    }

    public function getPosition(): ?Position
    {
        return $this->position;
    }

    public function setPosition(?Position $position): self
    {
        $this->position = $position;

        return $this;
    }

    public function getIntervals(): ?array
    {
        return $this->intervals;
    }

    public function setIntervals(?array $intervals): self
    {
        $this->intervals = $intervals;

        return $this;
    }

    public function getRawWorkerId(): ?string
    {
        return $this->rawWorkerId;
    }

    public function setRawWorkerId(?string $rawWorkerId): self
    {
        $this->rawWorkerId = $rawWorkerId;

        return $this;
    }

    public function getRawSectionId(): ?string
    {
        return $this->rawSectionId;
    }

    public function setRawSectionId(?string $rawSectionId): self
    {
        $this->rawSectionId = $rawSectionId;

        return $this;
    }

    public function getRawDepartmentId(): ?string
    {
        return $this->rawDepartmentId;
    }

    public function setRawDepartmentId(?string $rawDepartmentId): self
    {
        $this->rawDepartmentId = $rawDepartmentId;

        return $this;
    }

    public function getRawPositionId(): ?string
    {
        return $this->rawPositionId;
    }

    public function setRawPositionId(?string $rawPositionId): self
    {
        $this->rawPositionId = $rawPositionId;

        return $this;
    }

    public function getRawProperties(): ?array
    {
        return $this->rawProperties;
    }

    public function setRawProperties(?array $rawProperties): self
    {
        $this->rawProperties = $rawProperties;

        return $this;
    }

    public function getRatingSectionId(): ?string
    {
        return $this->ratingSectionId;
    }

    public function setRatingSectionId(?string $ratingSectionId): self
    {
        $this->ratingSectionId = $ratingSectionId;

        return $this;
    }

    public function getRowAmount(): ?int
    {
        return $this->rowAmount;
    }

    public function setRowAmount(?int $rowAmount): self
    {
        $this->rowAmount = $rowAmount;

        return $this;
    }

    public function getRowNumber(): ?int
    {
        return $this->rowNumber;
    }

    public function setRowNumber(?int $rowNumber): self
    {
        $this->rowNumber = $rowNumber;

        return $this;
    }

    public function getRating(): ?float
    {
        return $this->rating;
    }

    public function setRating(float $rating): self
    {
        $this->rating = $rating;

        return $this;
    }

    public function getPositions(): ?array
    {
        $positions = [] ;

        $section = $this->getSection() ;

        if ($section) {
            $positions = $section->getPositions();
        }

        return $positions;
    }

    public function setMeasurements(?array $measurements): self
    {
        $this->measurements = $measurements;

        return $this;
    }

    public function getMeasurements(): ?array
    {
        // $measurements = [
        //     [
        //         'barcode' => '6573d15e-6723-4bd1-b406-7678e6fdf8e1',
        //         'parentId' => 'caf34cc5-8126-49aa-a2ab-96f3fdad9dc8',
        //     ],
        //     [
        //         'barcode' => 'd2ac2f70-3248-48a1-a814-134fdc333003',
        //         'parentId' => 'bd65305e-ec84-4f9f-bbbe-3d88f65d9cd9',
        //     ]
        // ] ;

        return $this->measurements;
    }

    public function setPlants(?array $plants): self
    {
        $this->plants = $plants;

        return $this;
    }

    public function getPlants(): ?array
    {
        return $this->plants;
    }

    public function getToSend(): ?bool
    {
        return $this->toSend;
    }

    public function setToSend(bool $toSend): self
    {
        $this->toSend = $toSend;

        return $this;
    }

    public function getToExchange(): ?bool
    {
        return $this->toExchange;
    }

    public function setToExchange(bool $toExchange): self
    {
        $this->toExchange = $toExchange;

        return $this;
    }

    public function getWorkerName(): ?string
    {
        return $this->worker ? $this->worker->getName() : null;
    }

    public function getInProgress(): ?bool
    {
        return $this->inProgress;
    }

    public function setInProgress(?bool $inProgress): self
    {
        $this->inProgress = $inProgress;

        return $this;
    }

    public function getGreenhouseId(): ?string
    {
        return $this->greenhouseId;
    }

    public function setGreenhouseId(?string $greenhouseId): self
    {
        $this->greenhouseId = $greenhouseId;

        return $this;
    }

    public function getGreenhouse(): ?Greenhouse
    {
        return $this->greenhouse;
    }

    public function setGreenhouse(?Greenhouse $greenhouse): self
    {
        $this->greenhouse = $greenhouse;

        return $this;
    }

    public function getQualityTaskId(): ?string
    {
        return $this->qualityTaskId;
    }

    public function setQualityTaskId(?string $qualityTaskId): self
    {
        $this->qualityTaskId = $qualityTaskId;

        return $this;
    }

    public function setQualityTask($qualityTask)
    {
        $this->qualityTask = $qualityTask;

        return $this;
    }

    public function getQualityTask()
    {
        return $this->qualityTask;
    }

    public function getRowOrderBy(): ?int
    {
        return $this->rowOrderBy;
    }

    public function setRowOrderBy(?int $rowOrderBy): self
    {
        $this->rowOrderBy = $rowOrderBy;

        return $this;
    }

    public function getStructure(): ?Structure
    {
        return $this->structureId;
    }

    public function setStructure(?Structure $structureId): Task
    {
        $this->structureId = $structureId;

        return $this;
    }

    public function getStructureId(): ?string
    {
        return $this->structureId;
    }

    public function setStructureId(?string $structureId): self
    {
        $this->structureId = $structureId;

        return $this;
    }

    public function getOrderBy(): ?int
    {
        return $this->orderBy;
    }

    public function setOrderBy(int $orderBy): self
    {
        $this->orderBy = $orderBy;

        return $this;
    }

    public function getRatingScore(): ?float
    {
        return $this->ratingScore;
    }

    public function setRatingScore(float $ratingScore): self
    {
        $this->ratingScore = $ratingScore;

        return $this;
    }

    public function getRatingTime(): ?float
    {
        return $this->ratingTime;
    }

    public function setRatingTime(float $ratingTime): self
    {
        $this->ratingTime = $ratingTime;

        return $this;
    }

    public function getDynamicRating(): ?float
    {
        return $this->dynamicRating;
    }

    public function setDynamicRating(float $dynamicRating): self
    {
        $this->dynamicRating = $dynamicRating;

        return $this;
    }

    public function getLockedTill(): ?\DateTimeInterface
    {
        return $this->lockedTill;
    }

    public function setLockedTill(\DateTimeInterface $lockedTill): self
    {
        $this->lockedTill = $lockedTill;

        return $this;
    }

    public function getTodaySectionTasksCount(): ?int
    {
        return $this->todaySectionTasksCount;
    }

    public function setTodaySectionTasksCount(?int $todaySectionTasksCount): self
    {
        $this->todaySectionTasksCount = $todaySectionTasksCount;

        return $this;
    }

    public function getQualifiedWorkersOnline(): ?int
    {
        return $this->qualifiedWorkersOnline;
    }

    public function setQualifiedWorkersOnline(?int $qualifiedWorkersOnline): self
    {
        $this->qualifiedWorkersOnline = $qualifiedWorkersOnline;

        return $this;
    }

    public function getDay(): ?int
    {
        $orderBy = str_split(
            str_pad($this->orderBy, 5, '0', STR_PAD_LEFT)
        );

        return $orderBy[0];
    }

    public function getShift(): ?int
    {
        $orderBy = str_split(
            str_pad($this->orderBy, 5, '0', STR_PAD_LEFT)
        );

        return $orderBy[1];
    }

    public function getDirection(): ?bool
    {
        return (int) ltrim(
            substr(
                str_pad($this->orderBy, 5, '0', STR_PAD_LEFT),
                2
            ),
            '0'
        ) === $this->rowNumber;
    }

    public function getPass(): ?int
    {
        return $this->pass;
    }

    public function setPass(?int $pass): self
    {
        $this->pass = $pass;

        return $this;
    }

    public function getIsActive(): ?bool
    {
        return $this->isActive;
    }

    public function setIsActive(?bool $isActive): self
    {
        $this->isActive = $isActive;

        return $this;
    }

    public function activate(): self
    {
        $this->finishAt = clone $this->startAt;
        $this->finishAt->modify('+2 weeks')->modify('Monday this week');

        return $this;
    }

    public function deactivate(): self
    {
        $this->finishAt = clone $this->startAt;

        if(!$this->isFinished) {
            $this->previousWorkerId = $this->workerId ;
            $this->workerId = null ;
        }

        return $this;
    }

    public function getExclusive(): ?int
    {
        return $this->exclusive;
    }

    public function setExclusive(int $exclusive): self
    {
        $this->exclusive = $exclusive;

        return $this;
    }

    public function getPreviousWorkerId(): ?string
    {
        return $this->previousWorkerId;
    }

    public function setPreviousWorkerId(?string $previousWorkerId): self
    {
        $this->previousWorkerId = $previousWorkerId;

        return $this;
    }

    public function getSync(): ?Sync
    {
        return $this->sync;
    }

    public function setSync(?Sync $sync): self
    {
        $this->sync = $sync;

        return $this;
    }

    public function getCollectables(): ?array
    {
        return $this->collectables;
    }

    public function setCollectables(?array $collectables): self
    {
        $this->collectables = $collectables;

        return $this;
    }

    public function getTaskLimit(): int
    {
        return $this->taskLimit;
    }

    public function setTaskLimit(int $taskLimit): self
    {
        $this->taskLimit = $taskLimit;

        return $this;
    }

    public function getStatus(): ?int
    {
        return $this->status;
    }

    public function setStatus(?int $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getIsGuarding(): ?bool
    {
        $this->isGuarding = $this->section->getIsGuarding();

        return $this->isGuarding;
    }

    public function getIsSpraying(): ?bool
    {
        $this->isSpraying = $this->section->getIsSpraying();

        return $this->isSpraying;
    }

    public function getIsQuality(): bool
    {
        return $this->isQuality;
    }

    public function setIsQuality(bool $isQuality): self
    {
        $this->isQuality = $isQuality;

        return $this;
    }

    public function getShelfAmount(): ?int
    {
        return $this->section->getIsGuarding() ? 30 : $this->shelfAmount;
    }
}
