<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Annotation\ApiProperty;
use Symfony\Component\Serializer\Annotation\MaxDepth;

use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use Doctrine\ORM\Mapping\UniqueConstraint;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "createdAt",
 *     "registeredAt",
 *     "task.startAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "taskId": "exact",
 *     "controllerTaskId": "exact",
 *     "greenhouseSectionId": "exact",
 *     "checkMetricId": "exact",
 *     "task.rowNumber": "exact",
 *     "task.sectionId": "exact",
 *     "task.greenhouseId": "exact",
 *     "task.worker.id": "exact",
 *     "task.worker.name": "ipartial",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *      "createdAt",
 *      "registeredAt",
 *      "taskId",
 *      "controllerTaskId",
 *      "controllerId",
 *      "greenhouseSectionId",
 *      "checkMetricId",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class,
 *  arguments={
 *      "parameterName": "properties",
 *      "overrideDefaultProperties": false,
 *      "whitelist": null
 *  }
 * )
 * @ORM\Table(
 *   name="task_check",
 *   indexes={
 *     @ORM\Index(
 *      name="check_task_id",
 *      columns={"task_id"}
 *    ),
 *     @ORM\Index(
 *      name="check_controller_task_id",
 *      columns={"controller_task_id"}
 *    ),
 *     @ORM\Index(
 *      name="check_controller_id",
 *      columns={"controller_id"}
 *    ),
 *     @ORM\Index(
 *      name="check_greenhouse_section_id",
 *      columns={"greenhouse_section_id"}
 *    ),
 *     @ORM\Index(
 *      name="check_check_metric_id",
 *      columns={"check_metric_id"}
 *    ),
 *   },
 *   uniqueConstraints={
 *      @UniqueConstraint(name="controllerId_controllerTaskId_checkMetricId_taskId",
 *      columns={
 *          "controller_id", "controller_task_id", "check_metric_id", "task_id",
 *      }),
 *   },
 * )
 * @ORM\Entity(repositoryClass="App\Repository\TaskCheckRepository")
 */
class TaskCheck
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="datetimetz")
     * @Groups({"read"})
     */
    private $registeredAt;

    /**
     * @ORM\Column(type="guid")
     * @Groups({"read"})
     */
    private $taskId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Task")
     * @ORM\JoinColumn(nullable=false)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $task;

    /**
     * @ORM\Column(type="guid")
     * @Groups({"read"})
     */
    private $controllerId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(nullable=false)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $controller;

    /**
     * @ORM\Column(type="guid")
     * @Groups({"read"})
     */
    private $controllerTaskId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Task")
     * @ORM\JoinColumn(nullable=false)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $controllerTask;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $greenhouseSectionId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\GreenhouseSection")
     * @ORM\JoinColumn(nullable=true)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $greenhouseSection;

    /**
     * @ORM\Column(type="guid")
     * @Groups({"read"})
     */
    private $checkMetricId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\CheckMetric")
     * @ORM\JoinColumn(nullable=false, name="check_metric_id", referencedColumnName="id", onDelete="CASCADE")
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $checkMetric;

    /**
     * @ORM\Column(type="string", length=32, nullable=false)
     * @Groups({"read"})
     */
    private $value;

    /**
     * @ORM\Column(type="float" , options={"default" : 0})
     * @Groups({"read"})
     */
    private $ratingScore = 0;

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getRegisteredAt(): ?\DateTimeInterface
    {
        return $this->registeredAt;
    }

    public function setRegisteredAt(\DateTimeInterface $registeredAt): self
    {
        $this->registeredAt = $registeredAt;

        return $this;
    }

    public function getTaskId(): ?string
    {
        return $this->taskId;
    }

    public function setTaskId(?string $taskId): self
    {
        $this->taskId = $taskId;

        return $this;
    }

    public function getControllerId(): ?string
    {
        return $this->controllerId;
    }

    public function setControllerId(?string $controllerId): self
    {
        if ($this->controllerId == null || $controllerId == null) {
            $this->controllerId = $controllerId;
        }

        return $this;
    }

    public function getController(): ?Worker
    {
        return $this->controller;
    }

    public function setController(?Worker $worker): self
    {
        $this->controller = $worker;

        return $this;
    }

    public function getTask(): ?Task
    {
        return $this->task;
    }

    public function setTask(?Task $task): self
    {
        $this->task = $task;

        return $this;
    }

    public function getControllerTaskId(): ?string
    {
        return $this->controllerTaskId;
    }

    public function setControllerTaskId(?string $controllerTaskId): self
    {
        $this->controllerTaskId = $controllerTaskId;

        return $this;
    }

    public function getControllerTask(): ?Task
    {
        return $this->controllerTask;
    }

    public function setControllerTask(?Task $controllerTask): self
    {
        $this->controllerTask = $controllerTask;

        return $this;
    }

    public function getRatingScore(): ?float
    {
        return $this->ratingScore;
    }

    public function setRatingScore(float $ratingScore): self
    {
        $this->ratingScore = $ratingScore;

        return $this;
    }
    
    public function getGreenhouseSectionId(): ?string
    {
        return $this->greenhouseSectionId;
    }

    public function setGreenhouseSectionId(?string $greenhouseSectionId): self
    {
        $this->greenhouseSectionId = $greenhouseSectionId;

        return $this;
    }

    public function getGreenhouseSection(): ?GreenhouseSection
    {
        return $this->greenhouseSection;
    }

    public function setGreenhouseSection(?GreenhouseSection $greenhouseSection): self
    {
        $this->greenhouseSection = $greenhouseSection;

        return $this;
    }

    public function getCheckMetricId(): ?string
    {
        return $this->checkMetricId;
    }

    public function setCheckMetricId(?string $checkMetricId): self
    {
        $this->checkMetricId = $checkMetricId;

        return $this;
    }

    public function getCheckMetric(): ?CheckMetric
    {
        return $this->checkMetric;
    }

    public function setCheckMetric(?CheckMetric $checkMetric): self
    {
        $this->checkMetric = $checkMetric;

        return $this;
    }

    public function getValue(): ?string
    {
        return $this->value;
    }

    public function setValue(?string $value): self
    {
        $this->value = $value;

        return $this;
    }
}
