<?php

namespace App\Entity;

use App\Entity\Worker;
use App\Entity\Task;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\ORM\Mapping as ORM;

use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "createdAt",
 *     "registeredAt",
 *     "updatedAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "workerId": "exact",
 *     "taskId": "exact",
 *     "worker.name": "ipartial",
 *     "worker.code": "ipartial",
 *     "task.rowNumber": "exact",
 *     "task.greenhouseId": "exact",
 *     "task.sectionId": "exact",
 *     "worker.greenhouseId": "exact",
 *     "type": "exact",
 *   }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "workerId",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "workerId",
 *     "taskId",
 *     "registeredAt",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class, 
 *  arguments={
 *      "parameterName": "properties", 
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "workerId",
 *          "taskId",
 *          "registeredAt",
 *          "createdAt",
 *          "updatedAt",
 *      }
 *  }
 * )
 * @ORM\Table(
 *   name="task_metric",
 *   indexes={
 *     @ORM\Index(
 *       name="task_metric_worker_id",
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_metric_task_id",
 *       columns={"task_id"}
 *     ),
 *     @ORM\Index(
 *       name="task_metric_registered_at",
 *       columns={"registered_at"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\TaskMetricRepository")
 */
class TaskMetric
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="UUID")
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $taskId;

    /**
     * @var Task
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Task")
     * @ORM\JoinColumn(name="task_id", referencedColumnName="id")
     */
    private $task;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read"})
     */
    private $workerId;

    /**
     * @var Worker|null
     *
     * @Groups({"read"})
     * @MaxDepth(1)
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(name="worker_id", referencedColumnName="id")
     */
    private $worker;

    /**
     * @ORM\Column(type="string", length=32)
     * @Groups({"read"})
     */
    private $type;

    /**
     * @ORM\Column(type="datetimetz")
     * @Groups({"read"})
     */
    private $registeredAt;

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read"})
     */
    private $data = [];

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read"})
     */
    private $raw = [];

    /**
     * @ORM\Column(type="json", nullable=true)
     * @Groups({"read"})
     */
    private $meta = [];

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getTaskId(): ?string
    {
        return $this->taskId;
    }

    public function setTaskId(string $taskId): self
    {
        $this->taskId = $taskId;

        return $this;
    }

    public function getTask(): ?Task
    {
        return $this->task;
    }

    public function setTask(?Task $task): self
    {
        $this->task = $task;

        return $this;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getRegisteredAt(): ?\DateTimeInterface
    {
        return $this->registeredAt;
    }

    public function setRegisteredAt(\DateTimeInterface $registeredAt): self
    {
        $this->registeredAt = $registeredAt;

        return $this;
    }

    public function getData(): ?array
    {
        return $this->data;
    }

    public function setData(?array $data): self
    {
        $this->data = $data;

        return $this;
    }

    public function getRaw(): ?array
    {
        return $this->raw;
    }

    public function setRaw(?array $raw): self
    {
        $this->raw = $raw;

        return $this;
    }

    public function getMeta(): ?array
    {
        return $this->meta;
    }

    public function setMeta(?array $meta): self
    {
        $this->meta = $meta;

        return $this;
    }
}
