<?php declare(strict_types=1);

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Annotation\ApiProperty;
use Doctrine\ORM\Mapping as ORM;
use App\Uuid;
use App\Traits\DateTimeTrait as dateTimeTrait;
use Symfony\Component\Serializer\Annotation\MaxDepth;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;
use ApiPlatform\Core\Annotation\ApiFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\ExistsFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Serializer\Filter\PropertyFilter;

use ApiPlatform\Core\Serializer\Filter\GroupFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read"}, "enable_max_depth"=true},
 *   attributes={
 *     "pagination_client_items_per_page"=true
 *   },
 * )
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={
 *     "registeredAt",
 *   }
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={
 *     "id": "exact",
 *     "workerId": "exact",
 *     "registeredAt": "exact",
 *     "type": "exact",
 *     "trolley": "exact",
 *     "worker.name": "ipartial",
 *     "worker.searchable": "ipartial",
 *   }
 * )
 * @ApiFilter(
 *  GroupFilter::class,
 *  arguments={
 *      "parameterName": "groups",
 *      "overrideDefaultGroups": true,
 *      "whitelist": {
 *          "read",
 *          "quick",
 *      }
 *  }
 * )
 * @ApiFilter(
 *   ExistsFilter::class,
 *   properties={
 *     "type",
 *     "trolley",
 *     "registeredAt",
 *   }
 * )
 * @ApiFilter(
 *   RangeFilter::class,
 *   properties={
 *     "trolley",
 *   }
 * )
 * @ApiFilter(
 *   OrderFilter::class,
 *   properties={
 *     "trolley",
 *     "registeredAt",
 *     "type",
 *   },
 *   arguments={"orderParameterName"="order"}
 * )
 * @ApiFilter(
 *  PropertyFilter::class, 
 *  arguments={
 *      "parameterName": "properties", 
 *      "overrideDefaultProperties": false,
 *      "whitelist": {
 *          "id",
 *          "trolley",
 *          "type",
 *          "registeredAt",
 *          "workerId",
 *          "workerName",
 *      }
 *  }
 * )
 * @ORM\Table(
 *   name="trolley",
 *   indexes={
 *     @ORM\Index(
 *       name="trolley_worker_id",
 *       columns={"worker_id"}
 *     ),
 *     @ORM\Index(
 *       name="trolley_registered_at",
 *       columns={"registered_at"}
 *     ),
 *     @ORM\Index(
 *       name="trolley_trolley",
 *       columns={"trolley"}
 *     ),
 *     @ORM\Index(
 *       name="trolley_type",
 *       columns={"type"}
 *     ),
 *     @ORM\Index(
 *       name="trolley_history",
 *       columns={"registered_at", "worker_id", "type"}
 *     ),
 *   }
 * )
 * @ORM\Entity(repositoryClass="App\Repository\TrolleyRepository")
 */
class Trolley
{
    use dateTimeTrait;

    /**
     * @ApiProperty(identifier=true)
     * @ORM\Id
     * @ORM\Column(type="guid", unique=true)
     * @Assert\Uuid
     * @Groups({"read", "quick"})
     */
    private $id;

    /**
     * @ORM\Column(type="smallint", nullable=true)
     * @Groups({"read", "quick"})
     */
    private $trolley;

    /**
     * @ORM\Column(type="datetimetz", nullable=true)
     * @Groups({"read", "quick"})
     */
    private $registeredAt;

    /**
     * @ORM\Column(type="string", length=6, nullable=true)
     * @Groups({"read", "quick"})
     */
    private $type;

    /**
     * @ORM\Column(type="guid", nullable=true)
     * @Groups({"read", "quick"})
     */
    private $workerId;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Worker")
     * @ORM\JoinColumn(nullable=false)
     * @MaxDepth(1)
     * @Groups({"read"})
     */
    private $worker;

    /**
     * @Groups({"read", "quick"})
     */
    private $workerName;

    /**
     * @Groups({"read"})
     */
    private $previousId = null;

    /**
     * @Groups({"read"})
     */
    private $previous = null;

    /**
     * @Groups({"read"})
     */
    private $sync = null;

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getTrolley(): ?int
    {
        return $this->trolley;
    }

    public function setTrolley(?int $trolley): self
    {
        $this->trolley = $trolley;

        return $this;
    }

    public function getRegisteredAt(): ?\DateTimeInterface
    {
        return $this->registeredAt;
    }

    public function setRegisteredAt(?\DateTimeInterface $registeredAt): self
    {
        $this->registeredAt = $registeredAt;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getWorkerId(): ?string
    {
        return $this->workerId;
    }

    public function setWorkerId(?string $workerId): self
    {
        $this->workerId = $workerId;

        return $this;
    }

    public function getWorker(): ?Worker
    {
        return $this->worker;
    }

    public function setWorker(?Worker $worker): self
    {
        $this->worker = $worker;

        return $this;
    }

    public function getWorkerName(): ?string
    {
        return $this->worker ? $this->worker->getName() : null;
    }

    public function getPreviousId(): ?string
    {
        return $this->previousId;
    }

    public function setPreviousId(?string $previousId): self
    {
        $this->previousId = $previousId;

        return $this;
    }    

    public function getPrevious(): ?Trolley
    {
        return $this->previous;
    }

    public function setPrevious(?Trolley $previous): self
    {
        $this->previous = $previous;

        return $this;
    }

    public function getSync(): ?Sync
    {
        return $this->sync;
    }

    public function setSync(?Sync $sync): self
    {
        $this->sync = $sync;

        return $this;
    }
}
