<?php

namespace App\Repository;

use App\Entity\Task;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Common\Persistence\ManagerRegistry;

/**
 * @method Task|null find($id, $lockMode = null, $lockVersion = null)
 * @method Task|null findOneBy(array $criteria, array $orderBy = null)
 * @method Task[]    findAll()
 * @method Task[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class TaskRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Task::class);
    }

    public function getCountTasksDatePeriod(
        string $workerId,
        ?\DateTimeInterface $taskCountDateFrom = null,
        ?\DateTimeInterface $taskCountDateTo = null
    ) {
        $q = $this->createQueryBuilder('t')
            ->select('count(t.id)')
            ->where('t.workerId = :workerId')
            ->andWhere('t.isFinished = true')
            ->setParameter('workerId', $workerId);

        if($taskCountDateFrom != null) {
            $q
                ->andWhere('t.finishedAt >= :taskCountDateFrom')
                ->setParameter('taskCountDateFrom', $taskCountDateFrom);
        }

        if($taskCountDateTo != null) {
            $q
                ->andWhere('t.finishedAt <= :taskCountDateTo')
                ->setParameter('taskCountDateTo', $taskCountDateTo);
        }

        return $q
            ->getQuery()
            ->getSingleScalarResult();
        ;
    }

    public function getCountTasksDatePeriodActive(
        string $workerId,
        ?\DateTimeInterface $taskCountDateFrom = null,
        ?\DateTimeInterface $taskCountDateTo = null
    ) {
        $q = $this->createQueryBuilder('t')
            ->select('count(t.id)')
            ->where('t.workerId = :workerId')
            ->andWhere('t.isFinished = false')
            ->andWhere('t.isActive = true')
            ->setParameter('workerId', $workerId);

        if($taskCountDateFrom != null) {
            $q
                ->andWhere('t.finishAt >= :taskCountDateFrom')
                ->setParameter('taskCountDateFrom', $taskCountDateFrom);
        }

        if($taskCountDateTo != null) {
            $q
                ->andWhere('t.finishAt <= :taskCountDateTo')
                ->setParameter('taskCountDateTo', $taskCountDateTo);
        }

        return $q
            ->getQuery()
            ->getSingleScalarResult();
        ;
    }

    public function getCountTasksInProgress(string $taskId, string $greenhouseId, ?int $rowNumber = null)
    {
        $now = new \DateTime('NOW');

        $q = $this->createQueryBuilder('t')
            ->select('count(t.id)')
            ->where('t.id != :taskId')
            ->andWhere('t.isFinished = false')
            ->andWhere('t.startedAt IS NOT NULL')
            ->andWhere('t.finishedAt IS NULL')
            ->andWhere('t.rowNumber = :rowNumber')
            ->andWhere('t.greenhouseId = :greenhouseId')
            ->andWhere('t.finishAt >= :now')
            ->andWhere('t.startAt <= :now')
            ->andWhere('t.isActive = true')
            ->setParameter('now', $now)
            ->setParameter('rowNumber', $rowNumber)
            ->setParameter('greenhouseId', $greenhouseId)
            ->setParameter('taskId', $taskId);

        return $q
            ->getQuery()
            ->getSingleScalarResult();
        ;
    }

    public function getTaskInProgress(string $workerId)
    {
        $now = new \DateTime('NOW');

        $q = $this->createQueryBuilder('t')
            ->select('t.rowNumber')
            ->where('t.workerId = :workerId')
            ->andWhere('t.isFinished = false')
            ->andWhere('t.startedAt IS NOT NULL')
            ->andWhere('t.finishedAt IS NULL')
            ->andWhere('t.finishAt >= :now')
            ->andWhere('t.startAt <= :now')
            ->andWhere('t.isActive = true')
            ->setMaxResults(1)
            ->setParameter('now', $now)
            ->setParameter('workerId', $workerId);

        $r = $q
            ->getQuery()
            ->getOneOrNullResult()
        ;

        return $r ? $r['rowNumber'] : null;
    }

    public function getTasksCountBySection(
        ?string $sectionId,
        ?\DateTimeInterface $taskCountDateFrom = null,
        ?\DateTimeInterface $taskCountDateTo = null
    ) {
        $q = $this->createQueryBuilder('t')
            ->select('count(t.id)')
            ->where('t.sectionId = :sectionId')
            ->andWhere('t.parentId IS NOT NULL')
            ->andWhere('t.isActive = true')
            ->setParameter('sectionId', $sectionId);

        if($taskCountDateFrom != null) {
            $q
                ->andWhere('(t.startAt >= :taskCountDateFrom OR t.startedAt >= :taskCountDateFrom)')
                ->setParameter('taskCountDateFrom', $taskCountDateFrom);
        }

        if($taskCountDateTo != null) {
            $q
                ->andWhere('(t.startAt < :taskCountDateTo OR t.startedAt < :taskCountDateTo)')
                ->setParameter('taskCountDateTo', $taskCountDateTo);
        }

        return $q
            ->getQuery()
            ->getSingleScalarResult();
        ;
    }

    public function getSyncTask($trolley): ?Task
    {
        $dateFrom = clone $trolley->getRegisteredAt();
        $dateTo = clone $trolley->getRegisteredAt();

        $dateFrom->sub(new \DateInterval('PT15H'));

        return $this->createQueryBuilder('t')
            ->where('t.workerId = :workerId')
            ->andWhere('t.rowNumber IS NOT NULL')
            ->andWhere('t.parentId IS NOT NULL')
            ->andWhere('t.startedAt IS NOT NULL')
            ->andWhere('(t.startedAt >= :dateFrom OR t.finishedAt >= :dateFrom)')
            ->andWhere('(t.startedAt <= :dateTo OR t.finishedAt <= :dateTo)')
            ->leftJoin('t.section', 's')
            ->andWhere('s.isHarvesting = true')
            ->orderBy('t.startedAt', 'DESC')
            ->setMaxResults(1)
            ->setParameter('dateFrom', $dateFrom)
            ->setParameter('dateTo', $dateTo)
            ->setParameter('workerId', $trolley->getWorkerId())
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    public function findByWorkerIdDate($workerId, $dateFrom, $dateTo)
    {
        return $this->createQueryBuilder('t')
            ->where('t.workerId = :workerId')
            ->andWhere('t.rowNumber IS NOT NULL')
            ->andWhere('t.parentId IS NOT NULL')
            ->andWhere('t.startedAt IS NOT NULL')
            ->andWhere('t.startedAt >= :dateFrom')
            ->andWhere('t.startedAt <= :dateTo')
            ->orderBy('t.startedAt', 'ASC')
            ->setParameter('dateFrom', $dateFrom)
            ->setParameter('dateTo', $dateTo)
            ->setParameter('workerId', $workerId)
            ->getQuery()
            ->getResult()
        ;
    }

    public function getPhenologyTasksIds($taskId, $rowNumber, $greenhouseId, $sectionId)
    {
        return $this->createQueryBuilder('t')
            ->select('t.id')
            ->where('t.rowNumber = :rowNumber')
            ->andWhere('t.greenhouseId = :greenhouseId')
            ->andWhere('t.sectionId = :sectionId')
            ->andWhere('t.id != :taskId')
            ->andWhere('t.isFinished = true')
            ->orderBy('t.finishedAt', 'DESC')
            ->setMaxResults(30)
            ->setParameter('rowNumber', $rowNumber)
            ->setParameter('greenhouseId', $greenhouseId)
            ->setParameter('sectionId', $sectionId)
            ->setParameter('taskId', $taskId)
            ->getQuery()
            ->getResult()
        ;
    }

    public function getWorkerTasksNotStarted($workerId, $greenhouseId)
    {
        return $this->createQueryBuilder('t')
            ->where('t.workerId = :workerId')
            ->andWhere('t.isFinished = false')
            ->andWhere('t.startedAt IS NULL')
            ->andWhere('t.finishedAt IS NULL')
            ->andWhere('t.workerId IS NOT NULL')
            ->andWhere('t.greenhouseId IS NOT NULL')
            ->andWhere('t.greenhouseId = :greenhouseId')
            ->orderBy('t.startedAt', 'ASC')
            ->setParameter('workerId', $workerId)
            ->setParameter('greenhouseId', $greenhouseId)
            ->getQuery()
            ->getResult()
        ;
    }

    public function getQualityTaskById($qualityTaskId)
    {
        return $this->createQueryBuilder('t')
            ->leftJoin('t.worker', 'w')
            ->leftJoin('t.section', 's')
            ->select('t.id, t.startedAt, t.finishedAt, t.startAt, t.finishAt, t.sectionId, t.workerId, w.name as workerName, s.name as sectionName')
            ->where('t.id = :id')
            ->setParameter('id', $qualityTaskId)
            ->getQuery()
            ->getResult()
        ;
    }

    /*
    public function findOneBySomeField($value): ?Task
    {
        return $this->createQueryBuilder('t')
            ->andWhere('t.exampleField = :val')
            ->setParameter('val', $value)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }
    */
}
