import React from "react";
import { useDispatch, useSelector } from "react-redux";
import auditActions from "@iso/redux/audit/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./Audit.styles";

import moment from "../../library/helpers/moment";

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = auditActions;

export default function Audit() {
  const {
    audits,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    audit,
    filters,
  } = useSelector((state) => state.Audit);

  const {
    blocksPermissions: { ["audit"]: auditPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = auditPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchAuditName: "",
    filterAuditNameDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchAuditName } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchAuditName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(audits).map((audit, index) => {
    return dataSource.push({
      ...audits[audit],
      key: audit,
    });
  });

  const handleRecord = (actionName, audit) => {
    dispatch(saveIntoAgroInvest(audit, actionName));
  };

  const handleModal = (audit = null) => {
    dispatch(toggleModal(audit));
  };

  const auditUpdate = (key, value) => {
    if (key) audit[key] = value;
    dispatch(update(audit));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.name;

    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "Должность",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchAuditName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterAuditNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterAuditNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.position;
      },
    },
    {
      title: "Вид Работ",
      dataIndex: "parentId",
      key: "parentId",
      sorter: (a, b) => {
        if (a.parentId < b.parentId) return -1;
        if (a.parentId > b.parentId) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.section;
      },
    },
    {
      title: "Дата",
      dataIndex: "createdAt",
      key: "createdAt",
      sorter: (a, b) => {
        if (a.createdAt < b.createdAt) return -1;
        if (a.createdAt > b.createdAt) return 1;
        return 0;
      },
      render: (text, row) => {
        return moment(row.createdAt).calendar();
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            {hasAccessToEdit ? (
              <a onClick={() => handleModal(row)} href="# ">
                <i className="ion-android-create" />
              </a>
            ) : (
              <a disabled style={{ opacity: 0.3 }} href="# ">
                <i className="ion-android-create" />
              </a>
            )}
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Аудит доступа <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                disabled={!hasAccessToEdit}
                loading={isLoading}
                icon="plus"
                onClick={() => dispatch(toggleModal(null))}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {audit.id ? "Изменить параметр " + audit.name : "Добавить"}
                <br />
                <small>{audit.id ? audit.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", audit)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Название" hasFeedback>
                  <Input
                    placeholder="Имя объекта"
                    value={audit.name}
                    onChange={({ target: { value } }) =>
                      auditUpdate("name", value)
                    }
                  />
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
