import React, { useEffect, useState, useCallback, useMemo } from "react";
import { useDispatch, useSelector } from "react-redux";
import { useRouteMatch } from "react-router-dom";

import _ from "lodash";
import ReactJson from "react-json-view";
import moment from "../../library/helpers/moment";

import auditHistoryActions from "@iso/redux/auditHistory/actions";

import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";

import {
  TableWrapper,
  ComponentTitle,
  TitleWrapper,
} from "./AuditHistory.styles";

export default function AuditHistory() {
  const match = useRouteMatch();
  const dispatch = useDispatch();

  const { audit, totalItems, itemsPerPage, page } = useSelector((state) =>
    _.get(state, "AuditHistory", null)
  );

  const taskId = useMemo(() => _.get(match, "params.id", null), [match]);

  const [jsonData, setJsonData] = useState(null);
  const [queryData, setQueryData] = useState(null);
  const [modalJson, setModalJson] = useState(false);
  const [modalQuery, setModalQuery] = useState(false);

  useEffect(() => {
    dispatch(auditHistoryActions.setFilter("page", 1));
  }, [dispatch])

  useEffect(() => {
    if (taskId) {
      dispatch(auditHistoryActions.getTableId(taskId));
    }
    return () => {
      setJsonData(null);
      setQueryData(null);
      setModalJson(false);
      setModalQuery(false);
    };
  }, [dispatch, taskId, page]);

  const dataSource = [];
  Object.keys(audit).map((task, index) => {
    return dataSource.push({
      ...audit[task],
      key: task,
    });
  });

  const onModalViewJson = useCallback(() => setModalJson((prev) => !prev), []);

  const onModalViewQuery = useCallback(
    () => setModalQuery((prev) => !prev),
    []
  );

  const getViewColumn = useCallback(
    (value) =>
      value ? (
        <button
          onClick={() => {
            onModalViewJson();
            setJsonData(value);
          }}
        >
          Показать
        </button>
      ) : (
        ""
      ),
    [onModalViewJson, setJsonData]
  );

  const handleTableChange = useCallback((pagination, newFilters, sorter) => {
    dispatch(auditHistoryActions.setFilter("page", pagination.current));
  }, [dispatch]);

  const columns = [
    {
      title: "id",
      dataINdex: "id",
      key: "id",
      render: (text, row) => {
        return row.id;
      },
    },
    {
      title: "Table id",
      dataIndex: "tableId",
      key: "tableId",
      render: (text, row) => {
        return row.tableId;
      },
    },
    {
      title: "Table Name",
      dataIndex: "tableName",
      key: "tableName",
      render: (text, row) => {
        return row.tableName;
      },
    },
    {
      title: "User Name",
      dataIndex: "userName",
      key: "userName",
      render: (text, row) => {
        return row.userName;
      },
    },
    {
      title: "Action timestamp",
      dataIndex: "actionTimestamp",
      key: "actionTimestamp",
      render: (text, row) => {
        return moment(row.actionTimestamp).format("LLLL");
      },
    },
    {
      title: "Action",
      dataIndex: "action",
      key: "action",
      render: (text, row) => {
        return row.action;
      },
    },
    {
      title: "Old values",
      dataIndex: "oldValues",
      key: "oldValues",
      render: (text, row) => {
        const { oldValues } = row;
        return getViewColumn(oldValues);
      },
    },
    {
      title: "New values",
      dataIndex: "newValues",
      key: "newValues",
      render: (text, row) => {
        const { newValues } = row;
        return getViewColumn(newValues);
      },
    },
    {
      title: "Updated cols",
      dataIndex: "updatedCols",
      key: "updatedCols",
      render: (text, row) => {
        const { updatedCols } = row;
        return getViewColumn(updatedCols);
      },
    },
    {
      title: "query",
      dataIndex: "query",
      key: "query",
      render: (text, row) => {
        const { query } = row;
        return query ? (
          <button
            onClick={() => {
              onModalViewQuery();
              setQueryData(query);
            }}
          >
            Показать
          </button>
        ) : (
          ""
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              История задач <small>всего найдено: {totalItems}</small>
            </ComponentTitle>
          </TitleWrapper>
          <Modal
            visible={modalJson}
            title="Данные из JSON"
            onCancel={onModalViewJson}
            onOk={onModalViewJson}
          >
            {jsonData && (
              <ReactJson
                src={jsonData}
                theme="monokai"
                collapsed={false}
                enableClipboard={true}
              />
            )}
          </Modal>

          <Modal
            visible={modalQuery}
            title="Данные из query"
            onCancel={onModalViewQuery}
            onOk={onModalViewQuery}
          >
            <div>{queryData}</div>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
