import React, { useEffect, useRef, useState } from "react";
import html2pdf from "html2pdf.js";
import { useDispatch, useSelector } from "react-redux";
import Modal from "@iso/components/Feedback/Modal";
import DatePicker from "@iso/components/uielements/datePicker";
import Button from "@iso/components/uielements/button";
import {
  TableWrapper,
  ComponentTitle,
  TitleWrapper,
  DescriptionsWrapper,
  DescriptionsWrapperTitle,
  ActionsTitle,
  CheckMetricWrapper,
} from "./CheckLists.styles";
import { Select } from "antd";
import { Grid } from "react-flexbox-grid";
import actions from "@iso/redux/checkLists/actions";
import CsvDownloader from "react-csv-downloader";

import _ from "lodash";
import moment from "moment";
import "moment/locale/ru";
moment.locale("ru");

const { Option } = Select;

const columns = [
  {
    title: "Имя проверки",
    dataIndex: "name",
    align: "center",
    width: "400px",
  },
  {
    title: "Результат",
    dataIndex: "value",
    align: "center",
    width: "100px",
  },
];
const columnsTasks = [
  {
    title: "Ответственный овощевод",
    dataIndex: "task.worker",
    render: (worker) => {
      return `${worker?.name || '-'} / ${worker?.code || '-'}`;
    },
    align: "center",
    width: "400px",
  },
  {
    title: "Вид работ",
    dataIndex: "task.sectionName",
    align: "center",
    width: "400px",
  },
  {
    title: "Ряд",
    dataIndex: "task.rowNumber",
    align: "center",
    width: "100px",
  },
  {
    title: "Сумма баллов",
    dataIndex: "sumCheckMetrics",
    align: "center",
    width: "100px",
  },
];
const {
  setFilters,
  loadFromAgroInvest,
  rowsLoadFromAgroInvest,
  sectionsLoadFromAgroInvest,
  resetFilters,
} = actions;

const CheckMetric = ({ checkMetric }) => {
  const { controllerTask, task, checkMetricsData, controller } = checkMetric;
  const { greenhouseId, greenhouses } = useSelector((state) => state.App);
  const greenhouseDescription = _.get(
    _.find(greenhouses, (g) => g.id === greenhouseId),
    "description",
    null
  );

  const options = {
    margin: [15, 15],
    filename: task.sectionName,
    image: {
      type: "jpeg",
      quality: 0.95,
    },
    html2canvas: {
      useCORS: true,
      scale: 2,
    },
    jsPDF: {
      unit: "mm",
      format: "a4",
      orientation: "landscape",
      compress: true,
    },
  };

  const modalContentRef = useRef(null);

  const handleDownloadPdfClick = () => {
    if (!modalContentRef?.current) {
      return;
    }

    const pdf = html2pdf()
      .set(options)
      .from(modalContentRef.current)
      .toPdf();

    pdf.save();
  };

  return (
    <CheckMetricWrapper key={controllerTask.id}>
      <div ref={modalContentRef}>
        <DescriptionsWrapper
          title={(
            <DescriptionsWrapperTitle>
              <span>Чек лист: {task.sectionName}</span>
              <Button
                type="secondary"
                onClick={handleDownloadPdfClick}
                data-html2canvas-ignore="true"
              >
                Скачать
              </Button>
            </DescriptionsWrapperTitle>
          )}
          bordered
        >
          <DescriptionsWrapper.Item label="Номер чек-листа">
            {task.id}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Теплица">
            {greenhouseDescription}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Заказ наряд">
            {_.get(controllerTask, "rowNumber", "n/a")}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Дата">
            {moment(controllerTask.createdAt).format("DD MMMM YYYY")}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Время поставленной оценки" span={2}>
            {moment(controllerTask.finishedAt)
              .format("DD MMMM YYYY, HH:mm")}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Комментарий" span={3}>
            {_.get(controllerTask, "comment", "")}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Ответственный овощевод" span={3}>
            {_.get(task, "worker.name", "n/a")}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Автор оценки (контролер)" span={3}>
            {_.get(controller, "name", "n/a")}
          </DescriptionsWrapper.Item>
        </DescriptionsWrapper>

        <TableWrapper
          columns={columns}
          dataSource={checkMetricsData}
          pagination={false}
          bordered={true}
          rowKey={(record) => record.rowNumber}
        />
      </div>
    </CheckMetricWrapper>
  );
};
const CheckLists = () => {
  const dispatch = useDispatch();

  const {
    isLoading,
    isLoadingRows,
    isLoadingSections,
    sections,
    rows,
    filters,
    checkMetrics,
    taskChecks,
  } = useSelector((state) => state.CheckLists);

  const downloadButtonRef = useRef(null);
  const [modalActive, setModalActive] = useState(false);
  const [selectedCheckMetric, setSelectedCheckMetric] = useState(null);
  const { greenhouseId } = useSelector((state) => state.App);

  const requiredFields = ["registeredAt"];
  const searchDisabled =
    _.reduce(
      requiredFields,
      (result, field) => {
        if (filters[field]) return result + 1;
        else return result;
      },
      0
    ) < requiredFields.length;

  useEffect(() => {
    dispatch(rowsLoadFromAgroInvest());
    dispatch(sectionsLoadFromAgroInvest());
  }, [dispatch, greenhouseId]);

  const handleSetFilters = (key, value) => {
    dispatch(
      setFilters({
        ...filters,
        [key]: value,
      })
    );
  };

  const dropFilters = () => {
    dispatch(resetFilters());
  };
  const csvDownloader = (
    <CsvDownloader
      filename={`Чек-лист-${moment(new Date()).format("MMMM-Do-YYYY-h:mm:ss")}`}
      separator=";"
      wrapColumnChar={'"'}
      columns={[]}
      datas={[]}
      style={{ display: "none" }}
      ref={downloadButtonRef}
    />
  );

  const controllerTasks = [];

  const uniqControllerTaskIds = _.uniq(
    _.map(taskChecks, (taskCheck) => taskCheck.controllerTaskId)
  );

  _.forEach(uniqControllerTaskIds, (controllerTaskId) => {
    const currentTaskCheck = _.find(
      taskChecks,
      (taskCheck) => taskCheck.controllerTaskId === controllerTaskId
    );

    controllerTasks.push(currentTaskCheck);
  });

  const controllerTasksWithMetrics = controllerTasks.map((uniqTaskCheck) => {
    const checkMetricsData = _.map(checkMetrics, (checkMetric, index) => {
      return {
        key: checkMetric.id + index,
        rowKey: checkMetric.id,
        name: checkMetric.name,
        value: _.get(
          _.find(
            taskChecks,
            (taskCheck) =>
              taskCheck.checkMetricId === checkMetric.id &&
              taskCheck.controllerTaskId === uniqTaskCheck.controllerTask.id
          ),
          "value",
          "n/a"
        ),
      };
    }).filter((el) => el.value !== "n/a");
    const sumCheckMetrics = _.sumBy(checkMetricsData, ({ value }) =>
      Number(value)
    );
    return { ...uniqTaskCheck, checkMetricsData, sumCheckMetrics };
  });
  const sortedControllerTasks = _.sortBy(
    controllerTasksWithMetrics,
    "controllerTask.finishedAt"
  );

  return (
    <Grid fluid>
      <TitleWrapper>
        <ComponentTitle>
          Вид работ: &nbsp;
          <Select
            disabled={isLoadingSections}
            dropdownMatchSelectWidth={false}
            loading={isLoadingSections}
            style={{ width: 400 }}
            value={_.get(
              _.find(
                sections,
                (section) => section.sectionId === filters["task.sectionId"]
              ),
              "name",
              ""
            )}
            onChange={(value) => handleSetFilters("task.sectionId", value)}
          >
            {_.compact(
              _.map(_.sortBy(sections, ["name"]), (section) => {
                if (section.isQuality) return null;

                return (
                  <Option key={section.sectionId} value={section.sectionId}>
                    {section.name}
                  </Option>
                );
              })
            )}
          </Select>
          &nbsp; Дата: &nbsp;
          <DatePicker
            value={moment(filters.registeredAt)}
            onChange={(value) => {
              if (moment(value)._isValid) {
                handleSetFilters('registeredAt', moment(value));
              }
            }}
            allowClear={false}
          />
          &nbsp; Ряд: &nbsp;
          <Select
            disabled={isLoadingRows}
            loading={isLoadingRows}
            style={{ width: 80 }}
            value={filters["task.rowNumber"]}
            onChange={(value) => handleSetFilters("task.rowNumber", value)}
          >
            {_.map(rows, (row) => (
              <Option key={row.rowNumber} value={row.rowNumber}>
                {row.rowNumber}
              </Option>
            ))}
          </Select>
        </ComponentTitle>
        <ActionsTitle>
          {csvDownloader}
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="cloud-download"
            onClick={() => {
              downloadButtonRef.current.handleClick();
            }}
          />
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="filter"
            onClick={dropFilters}
            style={{ marginLeft: "10px" }}
          />
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="reload"
            disabled={searchDisabled}
            onClick={() => dispatch(loadFromAgroInvest())}
            style={{ marginLeft: "10px" }}
          />
        </ActionsTitle>
      </TitleWrapper>
      <Modal
        visible={modalActive}
        style={{ top: 20 }}
        onCancel={() => setModalActive(false)}
        footer={[
          <Button type="primary" onClick={() => setModalActive(false)}>
            Закрыть
          </Button>,
        ]}
        width={1000}
      >
        {selectedCheckMetric && (
          <CheckMetric checkMetric={selectedCheckMetric} />
        )}
      </Modal>
      <TableWrapper
        onRow={(record, rowIndex) => {
          return {
            onClick: (event) => {
              setSelectedCheckMetric(record);
              setModalActive(true);
            },
          };
        }}
        loading={isLoading}
        columns={columnsTasks}
        dataSource={sortedControllerTasks}
        pagination={false}
        bordered={true}
        rowKey={(record) => record.task.rowNumber}
      />
    </Grid>
  );
};
export default CheckLists;
