import React from "react";
import { useDispatch, useSelector } from "react-redux";
import chemicalActions from "@iso/redux/chemical/actions";
import workerActions from "@iso/redux/workers/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";
import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./Chemical.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = chemicalActions;

const { loadAutocomplete } = workerActions;

export default function Chemical() {
  const {
    chemicals,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    chemical,
    filters,
  } = useSelector((state) => state.Chemical);

  const {
    blocksPermissions: { ["chemical"]: chemicalPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = chemicalPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchChemical: "",
    filterChemicalDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchChemical } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters["name"] = searchChemical;
    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
    dispatch(loadAutocomplete());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(chemicals).map((chemical, index) => {
    return dataSource.push({
      ...chemicals[chemical],
      key: chemical,
    });
  });

  const handleRecord = (actionName, chemical) => {
    dispatch(saveIntoAgroInvest(chemical, actionName));
  };

  const handleModal = (chemical = null) => {
    dispatch(toggleModal(chemical));
  };

  const chemicalUpdate = (key, value) => {
    if (key) chemical[key] = value;
    dispatch(update(chemical));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters = _.assign(filters, newFilters);
    dispatch(setFilters(newFilters, pagination.current));
  };

  // const propertiesUpdate = (id, value) => {
  //   const p = _.find(chemical.properties, { id });

  //   if(p) {
  //     p.value = value;

  //     dispatch(update(chemical));
  //   }
  // }

  const columns = [
    {
      title: "Препарат",
      dataIndex: "chemical",
      key: "chemical",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchChemical", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterChemicalDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterChemicalDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Действия",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            {hasAccessToEdit ? (
              <>
                <a onClick={() => handleModal(row)} href="# ">
                  <i className="ion-android-create" />
                </a>
                <a onClick={() => handleRecord("delete", row)} href="# ">
                  <i className="ion-android-delete" />
                </a>
              </>
            ) : (
              <>
                <a disabled href="#">
                  <i className="ion-android-create" style={{ color: "#aaa" }} />
                </a>
                <a disabled href="#">
                  <i className="ion-android-delete" style={{ color: "#aaa" }} />
                </a>
              </>
            )}
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Препараты <small>всего найдено: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                disabled={!hasAccessToEdit}
                icon="plus"
                onClick={() => handleModal()}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {chemical.id
                  ? `Изменить параметр "${chemical.name}"`
                  : "Добавить"}
                <br />
                <small>{chemical.id ? chemical.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", chemical)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
          >
            <Form>
              <Fieldset>
                <Label>Препарат</Label>
                <FormItem label="Название" hasFeedback>
                  <Input
                    placeholder="Имя объекта"
                    value={chemical.name}
                    onChange={({ target: { value } }) =>
                      chemicalUpdate("name", value)
                    }
                  />
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
