import React, { useEffect, useRef, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import DatePicker from '@iso/components/uielements/datePicker';
import Button from '@iso/components/uielements/button';
import {
  TableWrapper,
  ComponentTitle,
  TitleWrapper,
  DescriptionsWrapper,
  ActionsTitle,
} from './HistoricalCheckLists.styles';
import { Select } from 'antd';
import { Grid } from 'react-flexbox-grid';
import actions from '@iso/redux/historicalCheckLists/actions';
import CsvDownloader from 'react-csv-downloader';

import _ from 'lodash';
import moment from 'moment';
import 'moment/locale/ru';
import { request } from '@iso/lib/helpers/utility';
moment.locale('ru');

const { Option } = Select;

const {
  setFilters,
  loadFromAgroInvest,
  sectionsLoadFromAgroInvest,
  resetFilters,
} = actions;

const HistoricalCheckLists = () => {
  const dispatch = useDispatch();

  const {
    isLoading,
    isLoadingSections,
    sections,
    filters,
    checkLists,
    columns,
    rating,
    dynamicRating,
    averageSum,
    averageDuration,
    averageDynamicRating,
  } = useSelector((state) => state.HistoricalCheckLists);

  const downloadButtonRef = useRef(null);
  const { greenhouseId } = useSelector((state) => state.App);
  const [worker, setWorker] = useState('');
  const [autoComplete, setAutoComplete] = useState([]);
  const [query, setQuery] = useState('');

  const requiredFields = ['registeredAt', 'task.worker.id', 'task.sectionId'];
  const searchDisabled =
    _.reduce(
      requiredFields,
      (result, field) => {
        if (filters[field]) return result + 1;
        else return result;
      },
      0
    ) < requiredFields.length;

  useEffect(() => {
    const fetchAutocomplite = async (query) => {
      const response = await request('workers', {
        exists: {
          spreadTill: true,
        },
        greenhouseId,
        properties: ['id', 'name', 'code'],
        searchable: query,
        isActive: true,
        itemsPerPage: 5,
        page: 1,
      });
      setAutoComplete(response['hydra:member']);
    };

    fetchAutocomplite(query);
  }, [greenhouseId, query]);

  useEffect(() => {
    dispatch(sectionsLoadFromAgroInvest());
  }, [dispatch, greenhouseId]);

  const handleSetFilters = (key, value) => {
    dispatch(
      setFilters({
        ...filters,
        [key]: value,
      })
    );
  };

  const dropFilters = () => {
    setQuery('');
    setWorker('');
    dispatch(resetFilters());
  };
  const csvDownloader = (
    <CsvDownloader
      filename={`Исторические-чек-листы-${moment(new Date()).format(
        'MMMM-Do-YYYY-h:mm:ss'
      )}`}
      separator=";"
      wrapColumnChar={'"'}
      columns={[]}
      datas={[]}
      style={{ display: 'none' }}
      ref={downloadButtonRef}
    />
  );

  const generatedColumns = _.sortBy(columns, ['registeredAt', 'rowNumber']).map(
    ({ name, registeredAt, rowNumber }) => ({
      title: `${moment(registeredAt).format('DD-MM-YYYY')} - ряд: ${rowNumber}`,
      dataIndex: `${name}`,
      align: 'center',
      width: 'auto',
      render: (item, row) => {
        const tag =
          item && (row.type === 'custom' ? (
            <b>{item.value}</b>
          ) : (
            `Баллы:${item.value} | Оценка:${item.ratingScore}`
          )) || 'нет данных';
        return tag;
      },
    })
  );
  const columnsTasks = [
    {
      title: 'Имя проверки',
      dataIndex: 'checkMetricName',
      align: 'center',
      width: '400px',
      fixed: 'left',
      render: (text, row) => {
        const tag =
          row.type === 'custom' ? (
            <b>{row.checkMetricName}</b>
          ) : (
            row.checkMetricName
          );
        return tag;
      },
    },
    {
      title: 'Дата регистрации чек листа - ряд',
      align: 'center',
      children: [...generatedColumns],
    },
  ];

  return (
    <Grid fluid>
      <TitleWrapper>
        <ComponentTitle>
          Имя Сотрудника: &nbsp;
          <Select
            showSearch
            value={worker}
            style={{ width: 400 }}
            defaultActiveFirstOption={false}
            showArrow={false}
            filterOption={false}
            onSearch={setQuery}
            onChange={(workerId) => {
              setWorker(workerId);
              handleSetFilters('task.worker.id', workerId);
            }}
            notFoundContent={null}
          >
            {autoComplete.map(({ id, name, code }) => {
              return <Option key={id}>{`${name} [${code}]`}</Option>;
            })}
          </Select>
          &nbsp; Вид работ: &nbsp;
          <Select
            disabled={isLoadingSections}
            dropdownMatchSelectWidth={false}
            loading={isLoadingSections}
            style={{ width: 400 }}
            value={_.get(
              _.find(
                sections,
                (section) => section.sectionId === filters['task.sectionId']
              ),
              'name',
              ''
            )}
            onChange={(value) => handleSetFilters('task.sectionId', value)}
          >
            {_.compact(
              _.map(_.sortBy(sections, ['name']), (section) => {
                if (section.isQuality) return null;

                return (
                  <Option key={section.sectionId} value={section.sectionId}>
                    {section.name}
                  </Option>
                );
              })
            )}
          </Select>
          &nbsp; Дата c: &nbsp;
          <DatePicker
            value={moment(filters.registeredAt.after)}
            onChange={(value) => {
              if (moment(value)._isValid) {
                handleSetFilters('registeredAt', {
                  ...filters.registeredAt,
                  after: moment(value).format(),
                });
              }
            }}
            allowClear={false}
          />
          &nbsp; по: &nbsp;
          <DatePicker
            value={moment(filters.registeredAt.before)}
            onChange={(value) => {
              if (moment(value)._isValid) {
                handleSetFilters('registeredAt', {
                  ...filters.registeredAt,
                  before: moment(value).format(),
                });
              }
            }}
            allowClear={false}
          />
        </ComponentTitle>
        <ActionsTitle>
          {csvDownloader}
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="cloud-download"
            onClick={() => {
              downloadButtonRef.current.handleClick();
            }}
          />
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="filter"
            onClick={dropFilters}
            style={{ marginLeft: '10px' }}
          />
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="reload"
            disabled={searchDisabled}
            onClick={() => dispatch(loadFromAgroInvest())}
            style={{ marginLeft: '10px' }}
          />
        </ActionsTitle>
      </TitleWrapper>
      <TableWrapper
        loading={isLoading}
        columns={columnsTasks}
        dataSource={checkLists}
        pagination={false}
        bordered={true}
        scroll={{ x: "max-content" }}
        rowKey={(record) => record.checkMetricId}
      />
      {!isLoading && columns.length > 0 && (
        <DescriptionsWrapper title={`Итоги:`} bordered>
          <DescriptionsWrapper.Item label="Средняя оценка качества за период">
            {averageSum}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item
            label="Средняя оценка времени за период"
            span={2}
          >
            {averageDuration}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Итоговая оценка">
            {averageDynamicRating}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Текущий рейтинг" span={2}>
            {rating}
          </DescriptionsWrapper.Item>
          <DescriptionsWrapper.Item label="Динамический рейтинг">
            {dynamicRating}
          </DescriptionsWrapper.Item>
        </DescriptionsWrapper>
      )}
    </Grid>
  );
};
export default HistoricalCheckLists;
