import React, { useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import plantProtectionActions from "@iso/redux/plantProtection/actions";
import Button from "@iso/components/uielements/button";
import { Grid } from "react-flexbox-grid";
import Modal from "@iso/components/Feedback/Modal";
import Form from "@iso/components/uielements/form";

import {
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  Fieldset,
} from "./IntensityDescriptions.styles";
import _ from "lodash";
import TextArea from "antd/lib/input/TextArea";
import Select, { SelectOption } from "@iso/components/uielements/select";
const Option = SelectOption;

const FormItem = Form.Item;

const getDescription = (intensities, id) => {
  const result = _.find(intensities, ({ intensityId }) => intensityId === id);
  return result ? result.description : "---";
};
const getId = (intensities, id) => {
  const result = _.find(intensities, ({ intensityId }) => intensityId === id);
  return result ? result.id : "d01b6eb8-5cc6-4cb4-bb7e-4f2e48fbbba8";
};

const pestType = {
  cucumber: "Огурец",
  tomato: "Томат",
};
export default function IntensityDescriptions() {
  const {
    isLoadingIntensityDescriptions,
    intensityDescriptions,
    intensities,
    pests,
  } = useSelector((state) => state.PlantProtection);

  const {
    blocksPermissions: { ["plant-protection"]: plantProtectionPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const {
    ["intensity-by-diseases"]: { edit: hasAccessToEdit },
  } = plantProtectionPermissions.blocks;

  const [modal, setModal] = useState({ active: false });
  const [selectedPestId, setSelectedPestId] = useState(null);
  const [modalAddPest, setModalAddPest] = useState({ active: false });
  const [
    currentIntensityDescription,
    setCurrentIntensityDescription,
  ] = useState(null);

  const closeModal = () => {
    setModal((prevState) => ({ ...prevState, active: false }));
  };

  const sortedIntensities = _.sortBy(intensities, "orderBy");
  const wrapperStyles = {
    display: "flex",
    alignItems: "center",
  };
  const intensitiesColumn = {
    title:
      "Степени поражённости и их цветовая кодировка и буквенное обозначение",
    children: sortedIntensities.map((el) => {
      return {
        title: (
          <div style={wrapperStyles}>
            {el.name}
            <div
              style={{
                background: `${el.color}`,
                width: "15px",
                height: "15px",
                borderRadius: "50%",
                border: "1px solid #ccc",
                marginLeft: "10px",
              }}
            />
          </div>
        ),
        align: "center",
        key: el.id,
        width: "250px",
        onCell: (record) => {
          return {
            onClick: () => {
              const intensityDescription = {
                pest: record.pest,
                intensity: el,
                description: getDescription(record.intensities, el.id),
                id: getId(record.intensities, el.id),
              };
              setCurrentIntensityDescription(intensityDescription);
              setModal({ active: true });
            },
          };
        },
        render: (text, row) => {
          return getDescription(row.intensities, el.id);
        },
      };
    }),
  };

  const {
    intensityDesriptionsLoadFromAgroInvest,
    saveIntoAgroInvestIntensityDescriptions,
    addIntensityDescriptions,
  } = plantProtectionActions;

  const dispatch = useDispatch();

  useEffect(() => {
    dispatch(intensityDesriptionsLoadFromAgroInvest());
  }, [dispatch, intensityDesriptionsLoadFromAgroInvest]);

  const columns = [
    {
      title: "Болезнь",
      key: "pest",
      width: "250px",
      align: "center",
      sorter: (a, b) => {
        if (a.pest.name < b.pest.name) return -1;
        if (a.pest.name > b.pest.name) return 1;
        return 0;
      },
      render: (text, row) => {
        return `${pestType[row.pest.type]}: ${row.pest.name}`;
      },
    },
    intensitiesColumn,
  ];

  return (
    <>
      <TitleWrapper>
        <ComponentTitle>
          Список описаний болезней по интенсивности
        </ComponentTitle>
        <div>
          <Button
            type="primary"
            shape="circle"
            loading={isLoadingIntensityDescriptions}
            disabled={!hasAccessToEdit}
            icon="plus"
            onClick={() => setModalAddPest({ active: true })}
            style={{ marginLeft: "10px" }}
          />
        </div>
      </TitleWrapper>
      {modal.active && (
        <Modal
          visible={modal.active}
          onClose={closeModal}
          title={`${currentIntensityDescription.pest.name}  --- ${currentIntensityDescription.intensity.name}`}
          okText={"Сохранить"}
          okButtonProps={{ disabled: !hasAccessToEdit }}
          onOk={() => {
            dispatch(
              saveIntoAgroInvestIntensityDescriptions(
                currentIntensityDescription
              )
            );
            setModal({ active: false });
          }}
          onCancel={closeModal}
        >
          <TextArea
            placeholder="Введите новое имя"
            autoSize
            value={currentIntensityDescription.description}
            onChange={(e) => {
              e.persist();
              setCurrentIntensityDescription((prevState) => ({
                ...prevState,
                description: e.target.value,
              }));
            }}
          />
        </Modal>
      )}
      {intensityDescriptions && (
        <>
          <Grid fluid>
            <TableWrapper
              loading={isLoadingIntensityDescriptions}
              rowKey={(row) => row.pest.id}
              columns={columns}
              bordered={true}
              dataSource={intensityDescriptions}
              className="isoSimpleTable"
              pagination={true}
            />
          </Grid>
        </>
      )}
      {modalAddPest.active && (
        <Modal
          visible={modalAddPest.active}
          onClose={() => setModalAddPest({ active: false })}
          title={"Добавить новую болезнь в список"}
          okText={"Добавить"}
          okButtonProps={{ disabled: !selectedPestId || !hasAccessToEdit }}
          onOk={() => dispatch(addIntensityDescriptions(selectedPestId))}
          onCancel={() => setModalAddPest({ active: false })}
        >
          <Form>
            <Fieldset>
              <FormItem label="Название" hasFeedback>
                <Select
                  dropdownMatchSelectWidth={false}
                  value={selectedPestId || "Ничего не выбрано"}
                  onChange={(e) => setSelectedPestId(e)}
                >
                  {_.map(
                    _.sortBy(pests, ["type", "name"]),
                    ({ id, name, type }) => (
                      <Option value={id} key={id}>
                        {`${pestType[type]}: ${name}`}
                      </Option>
                    )
                  )}
                </Select>
              </FormItem>
            </Fieldset>
          </Form>
        </Modal>
      )}
    </>
  );
}
