import React, { useRef, useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import CsvDownloader from "react-csv-downloader";
import metricActions from "@iso/redux/metric/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./Metric.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  loadCurrentPlantsFromAgroInvest,
  toggleModal,
  update,
  setFilters,
  prepareDownload,
} = metricActions;

export default function Metric() {
  const {
    metrics,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    metric,
    currentPlants: plants,
    filters,
    csvColumns,
    csvRows,
  } = useSelector((state) => state.Metric);

  const {
    phenologyGroups: groups,
    greenhouseId,
    allowedGreenhouses,
  } = useSelector((state) => state.App);

  const {
    blocksPermissions: { ["metrics"]: metricsPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = metricsPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchMetricName: "",
    filterMetricNameDropdownVisible: false,
    searchWorkerName: "",
    filterWorkerNameDropdownVisible: false,
  });

  const downloadButtonRef = useRef(null);

  useEffect(() => {
    if (csvColumns && csvRows) {
      downloadButtonRef.current.handleClick();
    }
  }, [csvColumns, csvRows]);

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchMetricName, searchWorkerName } = state;

    setState({
      ...state,
      filterMetricNameDropdownVisible: false,
      filterWorkerNameDropdownVisible: false,
    });

    const newFilters = _.cloneDeep(filters);
    newFilters.code = searchMetricName;
    newFilters["worker.name"] = searchWorkerName;

    dispatch(setFilters(newFilters, 1));
  };

  useEffect(() => {
    dispatch(loadFromAgroInvest());
    dispatch(loadCurrentPlantsFromAgroInvest(greenhouseId));
  }, [dispatch, greenhouseId]);

  const dataSource = [];
  Object.keys(metrics).map((metric, index) => {
    return dataSource.push({
      ...metrics[metric],
      key: metric,
    });
  });

  const handleRecord = (actionName, metric) => {
    dispatch(saveIntoAgroInvest(metric, actionName));
  };

  const handleModal = (metric = null) => {
    dispatch(toggleModal(metric));
  };

  const metricUpdate = (key, value) => {
    if (key) metric[key] = value;
    dispatch(update(metric));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const downloadMetric = () => {
    dispatch(prepareDownload());
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.code = filters.code;
    newFilters["worker.name"] = filters["worker.name"];

    dispatch(setFilters(newFilters, pagination.current));
  };

  // const notBase64 = /[^A-Z0-9+\/=]/i;

  // const isBase64 = (str) => {
  //   const len = str.length;

  //   return len > 1000;

  //   if (!len || len % 4 !== 0 || notBase64.test(str)) {
  //     return false;
  //   }
  //   const firstPaddingChar = str.indexOf('=');
  //   return firstPaddingChar === -1 ||
  //     firstPaddingChar === len - 1 ||
  //     (firstPaddingChar === len - 2 && str[len - 1] === '=');
  // }

  const generateActionButtons = (row) => {
    const allowedGreenhouseIds = _.map(allowedGreenhouses, (g) => g.id);
    const isAttachedUser = _.includes(
      allowedGreenhouseIds,
      _.get(row, "plant.row.greenhouseId")
    );

    const style =
      isAttachedUser && hasAccessToEdit
        ? {}
        : {
            opacity: "0.3",
          };

    return (
      <ActionWrapper>
        <a
          onClick={() => handleModal(row)}
          disabled={!isAttachedUser || !hasAccessToEdit}
          href="# "
        >
          <i className="ion-android-create" style={{ ...style }} />
        </a>
      </ActionWrapper>
    );
  };

  const columns = [
    {
      title: "КОД",
      dataIndex: "code",
      key: "code",
      sorter: (a, b) => {
        if (a.code < b.code) return -1;
        if (a.code > b.code) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchMetricName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.code ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterMetricNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterMetricNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      csvText: (row) => `${row.code} \n`,
      render: (text, row) => {
        return row.code;
      },
    },
    {
      title: "Растение",
      dataIndex: "plantId",
      key: "plantId",
      sorter: (a, b) => {
        if (_.get(a, "plant.name") < _.get(b, "plant.name")) return -1;
        if (_.get(a, "plant.name") > _.get(b, "plant.name")) return 1;
        return 0;
      },
      filters: _.map(plants, (p) => {
        return {
          text: `Ряд ${_.get(p, "row.rowNumber", "-")}, ${p.name}`,
          value: p.id,
        };
      }),
      filteredValue: filters["plantId"] || [],
      filterMultiple: false,
      csvText: (row) => `
        Ряд: ${_.get(row, "plant.row.rowNumber", "-")}, ${_.get(
        row,
        "plant.name",
        "n/a"
      )}
        id: ${_.get(row, "plantId", "n/a")}
      `,
      render: (text, row) => {
        return (
          <>
            Ряд:{" "}
            <b>{`${_.get(row, "plant.row.rowNumber", "-")}, ${_.get(
              row,
              "plant.name",
              "n/a"
            )}`}</b>
            <br />
            id: <b>{_.get(row, "plantId", "n/a")}</b>
          </>
        );
      },
    },
    {
      title: "Метрика",
      dataIndex: "propertyId",
      key: "parentId",
      sorter: (a, b) => {
        if (_.get(a, "parent.name") < _.get(b, "parent.name")) return -1;
        if (_.get(a, "parent.name") > _.get(b, "parent.name")) return 1;
        return 0;
      },
      filters: _.map(groups, (g) => {
        return {
          text: g.name,
          value: g.id,
        };
      }),
      filteredValue: filters["parentId"] || [],
      filterMultiple: false,
      csvText: (row) => `
        Группа: ${_.get(row, "parent.name", "n/a")} \n
        Метрика: ${_.get(row, "property.name", "n/a")}
      `,
      render: (text, row) => {
        return (
          <>
            Группа: <b>{_.get(row, "parent.name", "n/a")}</b>
            <br />
            Метрика: <b>{_.get(row, "property.name", "n/a")}</b>
          </>
        );
      },
    },
    {
      title: "Детали",
      dataIndex: "taskId",
      key: "taskId",
      sorter: (a, b) => {
        if (_.get(a, "worker.name") < _.get(b, "worker.name")) return -1;
        if (_.get(a, "worker.name") > _.get(b, "worker.name")) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchWorkerName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters["worker.name"] ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterWorkerNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterWorkerNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      csvText: (row) => `
        Сотрудник: ${_.get(row, "worker.name", "n/a")} \n
        Задача: ${_.get(row, "task.name", "n/a")}
      `,
      render: (text, row) => {
        return (
          <>
            Сотрудник: <b>{_.get(row, "worker.name", "n/a")}</b>
            <br />
            Задача: <b>{_.get(row, "task.name", "n/a")}</b>
          </>
        );
      },
    },
    {
      title: "Дата",
      dataIndex: "registeredAt",
      key: "registeredAt",
      defaultSortOrder: "descend",
      filters: [
        {
          text: "Сегодня",
          value: JSON.stringify({
            before: moment().utc().endOf("day").format(),
            after: moment().utc().startOf("day").format(),
          }),
        },
        {
          text: "Вчера",
          value: JSON.stringify({
            before: moment().utc().subtract(1, "days").endOf("day").format(),
            after: moment().utc().subtract(1, "days").startOf("day").format(),
          }),
        },
        {
          text: "Эта неделя",
          value: JSON.stringify({
            before: moment().utc().endOf("week").format(),
            after: moment().utc().startOf("week").format(),
          }),
        },
        {
          text: "Прошлая неделя",
          value: JSON.stringify({
            before: moment().utc().startOf("week").subtract(1, "days").format(),
            after: moment().utc().startOf("week").subtract(7, "days").format(),
          }),
        },
        {
          text: "Этот месяц",
          value: JSON.stringify({
            before: moment().utc().endOf("month").format(),
            after: moment().utc().startOf("month").format(),
          }),
        },
      ],
      filteredValue: filters["registeredAt"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (a.registeredAt < b.registeredAt) return -1;
        if (a.registeredAt > b.registeredAt) return 1;
        return 0;
      },
      csvText: (row) => moment(row.registeredAt).calendar(),
      render: (text, row) => {
        return moment(row.registeredAt).calendar();
      },
    },
    {
      title: "Значение",
      dataIndex: "value",
      key: "value",
      sorter: (a, b) => {
        if (+a.value < +b.value) return -1;
        if (+a.value > +b.value) return 1;
        return 0;
      },
      csvText: (row) =>
        row.property.type === "binary" && !!row.value
          ? "Red dot"
          : _.get(row, "value", "n/a"),
      render: (text, row) => {
        return row.property.type === "binary" && !!row.value ? (
          <img
            src={process.env.REACT_APP_MEDIA_URL + "/uploads/" + row.value}
            alt="Red dot"
            style={{ width: 100 }}
          />
        ) : (
          _.get(row, "value", "n/a")
        );
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return generateActionButtons(row);
      },
    },
  ];

  const csvDownloader = (
    <CsvDownloader
      filename={`Метрики-${moment(new Date()).format("MMMM-Do-YYYY-h:mm:ss")}`}
      separator=";"
      wrapColumnChar={'"'}
      columns={csvColumns}
      datas={csvRows}
      style={{ display: "none" }}
      ref={downloadButtonRef}
    />
  );

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Метрики <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              {csvDownloader}
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="cloud-download"
                onClick={downloadMetric}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {metric.id
                  ? `Изменить параметр "${metric.task.name}"`
                  : "Добавить"}
                <br />
                <small>{metric.id ? metric.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", metric)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Значение" hasFeedback>
                  <Input
                    placeholder="Значение замера"
                    value={metric.value}
                    onChange={({ target: { value } }) =>
                      metricUpdate("value", value)
                    }
                  />
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
