import React, { useEffect } from 'react';
import { useSelector, useDispatch } from "react-redux";
import _get from 'lodash/get';
import { InputNumber } from "antd";
import moment from 'moment';

import actions from "@iso/redux/phenology/actions";

import {
  TableWrapper,
  InputNumberContainer,
  ComponentTitle,
  TitleWrapper,
} from './Phenology.styles';

const {
  getHistoricalValves,
  setHistoricalValves,
  updateHistoricalValves,
  setHistoricalValvesFilters,
} = actions;

const HistoricalValves = () => {
  const { greenhouseId } = useSelector((state) => state.App);

  const {
    blocksPermissions: { ["phenology"]: phenologyPermissions },
  } = useSelector((state) => _get(state, "Auth.userPermissions"));

  const {
    ['historicalValves']: { edit: hasAccessToEditHistoricalValves },
  } = phenologyPermissions.blocks;

  const {
    historicalValves,
    isLoading,
  } = useSelector((state) => state.Phenology);

  const {
    items,
    pagination,
    filters,
  } = historicalValves;

  const {
    itemsPerPage,
    page,
    totalItems,
  } = pagination;

  const dispatch = useDispatch();

  useEffect(() => {
    dispatch(getHistoricalValves());
  }, [greenhouseId, dispatch]);

  const handleHistoricalValveChange = (id, name) => (
    (value) => {
      dispatch(setHistoricalValves({
        id,
        [name]: value,
      }))
    }
  );

  const handleHistoricalValveBlur = (id) => (
    ({ target }) => {
      if (target.value.trim()) {
        dispatch(updateHistoricalValves({
          id,
        }))
      }
    }
  );

  const handleTableChange = (pagination, filters) => {
    if (!filters.registeredAt.length) {
      filters.registeredAt = [
        JSON.stringify({
          before: moment().utc().endOf("week").format(),
          after: moment().utc().startOf("week").format()
        })
      ];
    }

    dispatch(setHistoricalValvesFilters({
      page: pagination.current,
      filters,
    }));
  };

  const columns = [
    {
      title: 'Ряд',
      dataIndex: 'row',
    },
    {
      title: 'Исполнитель',
      dataIndex: 'worker',
    },
    {
      title: 'Клапан',
      dataIndex: 'valveNumber',
    },
    {
      title: 'Вид работ',
      dataIndex: 'type',
    },
    {
      title: 'Объём',
      dataIndex: 'volume',
      render: (value, { id }) => {
        return (
          <InputNumberContainer>
            <InputNumber
              value={value}
              onChange={handleHistoricalValveChange(id, 'volume')}
              onBlur={handleHistoricalValveBlur(id)}
              precision={2}
              disabled={!hasAccessToEditHistoricalValves}
            />
          </InputNumberContainer>
        );
      },
    },
    {
      title: 'Ec',
      dataIndex: 'electricConduction',
      render: (value, { id }) => {
        return (
          <InputNumberContainer>
            <InputNumber
              value={value}
              onChange={handleHistoricalValveChange(id, 'electricConduction')}
              onBlur={handleHistoricalValveBlur(id)}
              precision={2}
              disabled={!hasAccessToEditHistoricalValves}
            />
          </InputNumberContainer>
        );
      },
    },
    {
      title: 'pH',
      dataIndex: 'potentialHydrogen',
      render: (value, { id }) => {
        return (
          <InputNumberContainer>
            <InputNumber
              value={value}
              onChange={handleHistoricalValveChange(id, 'potentialHydrogen')}
              onBlur={handleHistoricalValveBlur(id)}
              precision={2}
              disabled={!hasAccessToEditHistoricalValves}
            />
          </InputNumberContainer>
        );
      },
    },
    {
      title: 'Дата замера',
      dataIndex: 'registeredAt',
      sorter: (a, b) => {
        if (a.registeredAt < b.registeredAt) return -1;
        if (a.registeredAt > b.registeredAt) return 1;

        return 0;
      },
      filters: [
        {
          text: "Понедельник",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(1).endOf("day").format(),
            after: moment().utc().isoWeekday(1).startOf("day").format(),
          }),
        },
        {
          text: "Вторник",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(2).endOf("day").format(),
            after: moment().utc().isoWeekday(2).startOf("day").format(),
          }),
        },
        {
          text: "Среда",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(3).endOf("day").format(),
            after: moment().utc().isoWeekday(3).startOf("day").format(),
          }),
        },
        {
          text: "Четверг",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(4).endOf("day").format(),
            after: moment().utc().isoWeekday(4).startOf("day").format(),
          }),
        },
        {
          text: "Пятница",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(5).endOf("day").format(),
            after: moment().utc().isoWeekday(5).startOf("day").format(),
          }),
        },
        {
          text: "Суббота",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(6).endOf("day").format(),
            after: moment().utc().isoWeekday(6).startOf("day").format(),
          }),
        },
        {
          text: "Воскресенье",
          value: JSON.stringify({
            before: moment().utc().isoWeekday(7).endOf("day").format(),
            after: moment().utc().isoWeekday(7).startOf("day").format(),
          }),
        },
      ],
      filteredValue: filters["registeredAt"] || [],
      filterMultiple: false,
      render: (_, row) => moment(row.registeredAt).calendar()
    }
  ];

  const dataSource = items.map(({ id, task, valve, volume, electricConduction, potentialHydrogen, registeredAt }) => {
    return {
      key: id,
      id,
      row: task.rowNumber,
      worker: `${task.worker.name} / ${task.worker.code}`,
      valveNumber: valve.number,
      type: valve.workDescription,
      volume,
      electricConduction,
      potentialHydrogen,
      registeredAt,
    };
  });

  return (
    <>
      <TitleWrapper>
        <ComponentTitle>
          Исторические показания <small>всего найдено: {totalItems}</small>
        </ComponentTitle>
      </TitleWrapper>
      <TableWrapper
        rowKey="key"
        columns={columns}
        bordered={true}
        dataSource={dataSource}
        loading={isLoading}
        onChange={handleTableChange}
        className="isoSimpleTable"
        pagination={{
          defaultPageSize: itemsPerPage,
          hideOnSinglePage: true,
          total: totalItems,
          current: page,
          showTotal: (total, range) => {
            return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
          },
        }}
      />
    </>
  );
}

export default HistoricalValves;
