import React, { useEffect } from "react";
import { useSelector, useDispatch } from "react-redux";
import _ from "lodash";

import cn from "classnames";
import { Icon } from "antd";

import Input from "@iso/components/uielements/input";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import Modal from "@iso/components/Feedback/Modal";
import Switch from "@iso/components/uielements/switch";
import Button from "@iso/components/uielements/button";
import Form from "@iso/components/uielements/form";
import Select, { SelectOption } from "@iso/components/uielements/select";

import phenologyActions from "@iso/redux/phenology/actions";
import workerActions from "@iso/redux/workers/actions";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
  StatusTag,
} from "./Phenology.styles";

const FormItem = Form.Item;
const Option = SelectOption;

const Status = ({ isActive }) => (
  <StatusTag className={cn({ active: isActive, inactive: !isActive })}>
    {isActive ? "активный" : "не активный"}
  </StatusTag>
);

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = phenologyActions;

const {
  loadAutocomplete,
  // setAutocomplete,
} = workerActions;

const PhenologyContent = () => {
  const {
    collectables,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    collectable,
    filters,
  } = useSelector((state) => state.Phenology);

  const {
    blocksPermissions: { ["phenology"]: phenologyPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = phenologyPermissions;

  const dispatch = useDispatch();

  useEffect(() => {
    dispatch(loadFromAgroInvest());
    dispatch(loadAutocomplete());
  }, [dispatch]);

  const { phenologyGroups } = useSelector((state) => state.App);

  const [state, setState] = React.useState({
    searchCollectableName: "",
    filterCollectableNameDropdownVisible: false,
  });

  const dataSource = [];
  Object.keys(collectables).map((collectable, index) => {
    return dataSource.push({
      ...collectables[collectable],
      key: collectable,
    });
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchCollectableName } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchCollectableName;

    dispatch(setFilters(newFilters, 1));
  };

  const handleRecord = (actionName, collectable) => {
    if (collectable.type === "group") collectable.parentId = null;

    dispatch(saveIntoAgroInvest(collectable, actionName));
  };

  const handleModal = (collectable = null) => {
    dispatch(toggleModal(collectable));
  };

  const collectableUpdate = (key, value) => {
    if (key) collectable[key] = value;
    dispatch(update(collectable));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.name;

    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "Название",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchCollectableName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterCollectableNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterCollectableNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Тип",
      dataIndex: "type",
      className: "noWrapCell",
      key: "type",
      filters: [
        {
          text: "Группа",
          value: "group",
        },
        {
          text: "Фото",
          value: "binary",
        },
        {
          text: "Число",
          value: "number",
        },
        {
          text: "Строка",
          value: "string",
        },
      ],
      filteredValue: filters["type"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (a.type < b.type) return -1;
        if (a.type > b.type) return 1;
        return 0;
      },

      render: (text, row) => {
        return (
          <StatusTag className={row.type}>{row.type || "group"}</StatusTag>
        );
      },
    },
    {
      title: "Группа",
      dataIndex: "parentId",
      key: "parentId",
      sorter: (a, b) => {
        if (a.parentId < b.parentId) return -1;
        if (a.parentId > b.parentId) return 1;
        return 0;
      },
      render: (text, row) => {
        return _.get(row, "parent.name", "n/a");
      },
    },
    {
      title: "Культура",
      dataIndex: "category",
      key: "category",
      filters: [
        {
          text: "Общая",
          value: "common",
        },        
        {
          text: "Огурец",
          value: "cucumber",
        },
        {
          text: "Томат",
          value: "tomato",
        },
      ],
      filteredValue: filters["category"] || [],
      filterMultiple: false,
      render: (text, row) => {
        return (
          <StatusTag className={row.category}>
            {row.category === "cucumber" && "Огурец" || row.category === "tomato" && "Томат" || "Общая"}
          </StatusTag>
        );
      },
    },
    {
      title: "Статус",
      dataIndex: "isActive",
      className: "noWrapCell",
      key: "isActive",
      align: "center",
      filters: [
        {
          text: "Активно",
          value: true,
        },
        {
          text: "Не активно",
          value: false,
        },
      ],
      filteredValue: filters["isActive"] || [],
      filterMultiple: false,
      render: (text, row) => {
        const isActive = _.get(row, "isActive", false);

        return (
          <>
            <div>
              <StatusTag className={isActive ? "active" : "non-active"}>
                {isActive ? "Активно" : "Не активно"}
              </StatusTag>
            </div>
          </>
        );
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            {hasAccessToEdit ? (
              <a onClick={() => handleModal(row)} href="# ">
                <i className="ion-android-create" />
              </a>
            ) : (
              <a href="# " disabled>
                <i className="ion-android-create" style={{ color: "#aaa" }} />
              </a>
            )}
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <>
      <TitleWrapper>
        <ComponentTitle>
          Фенология <small>всего найдено: {totalItems}</small>
        </ComponentTitle>
        <div>
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            disabled={!hasAccessToEdit}
            icon="plus"
            onClick={() => dispatch(toggleModal(null))}
            style={{ marginLeft: "10px" }}
          />
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="filter"
            onClick={dropFilters}
            style={{ marginLeft: "10px" }}
          />
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="reload"
            onClick={reload}
            style={{ marginLeft: "10px" }}
          />
        </div>
      </TitleWrapper>

      <Modal
        visible={modalActive}
        onClose={() => dispatch(toggleModal(null))}
        title={
          <>
            {collectable.id
              ? "Изменить параметр " + collectable.name
              : "Добавить"}
            <br />
            <small>{collectable.id ? collectable.id : "n/a"}</small>
          </>
        }
        okText={"Сохранить"}
        onOk={() => handleRecord("update", collectable)}
        onCancel={() => dispatch(toggleModal(null))}
        okButtonProps={{ disabled: !hasAccessToEdit }}
        cancelButtonProps={{ disabled: false }}
      >
        <Form>
          <Fieldset>
            <Label>Основные свойства</Label>
            <FormItem label="Название" hasFeedback>
              <Input
                placeholder="Название собираемого параметра или группы"
                value={collectable.name}
                onChange={({ target: { value } }) =>
                  collectableUpdate("name", value)
                }
              />
            </FormItem>
            <FormItem label="Культура" hasFeedback>
              <Select
                dropdownMatchSelectWidth={false}
                value={collectable.category || "Общая"}
                onChange={(value) => collectableUpdate("category", value)}
              >
                <Option value="">Общая</Option>
                <Option value="cucumber">Огурец</Option>
                <Option value="tomato">Томат</Option>
              </Select>
            </FormItem >
            <FormItem
              label={
                <>
                  Статус &nbsp;
                  <Status isActive={collectable.isActive} />
                </>
              }
              hasFeedback
            >
              <Switch
                checked={collectable.isActive}
                onChange={(isValue) => {
                  collectableUpdate("isActive", isValue);
                  console.log(
                    "🚀 ~ file: Phenology.js ~ line 357 ~ Phenology ~ isValue",
                    isValue
                  );
                }}
              />
            </FormItem>
            <FormItem label="Тип" hasFeedback>
              <Select
                size="large"
                value={collectable.type}
                onChange={(value) => collectableUpdate("type", value)}
              >
                <Option value="group">Группа</Option>
                <Option value="binary">Фото</Option>
                <Option value="number">Число</Option>
                <Option value="string">Строка</Option>
              </Select>
            </FormItem>
            {collectable.type !== "group" && (
              <FormItem label="Группа параметров" hasFeedback>
                <Select
                  size="large"
                  value={collectable.parentId || undefined}
                  placeholder="Выберите группу"
                  onChange={(value) => collectableUpdate("parentId", value)}
                >
                  {_.map(phenologyGroups, (g) => (
                    <Option value={g.id} key={g.id}>
                      {g.name}
                    </Option>
                  ))}
                </Select>
              </FormItem>
            )}
          </Fieldset>
        </Form>
      </Modal>

      <TableWrapper
        rowKey="key"
        columns={columns}
        bordered={true}
        dataSource={dataSource}
        loading={isLoading}
        className="isoSimpleTable"
        onChange={handleTableChange}
        pagination={{
          defaultPageSize: itemsPerPage,
          hideOnSinglePage: true,
          total: totalItems,
          current: page,
          showTotal: (total, range) => {
            return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
          },
        }}
      />
    </>
  );
}

export default PhenologyContent;
