import React from "react";
import { useDispatch, useSelector } from "react-redux";
import plantPhenologyActions from "@iso/redux/plantPhenology/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";
import Select, { SelectOption } from "@iso/components/uielements/select";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./PlantPhenology.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const Option = SelectOption;

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  rowsLoadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
  // resetPlant,
} = plantPhenologyActions;

export default function PlantPhenology() {
  const {
    plants,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    plant,
    isLoadingRows,
    rows,
    filters,
  } = useSelector((state) => state.PlantPhenology);

  const dispatch = useDispatch();

  const [state, setState] = React.useState({});

  const { allowedGreenhouses, greenhouses, greenhouseId } = useSelector(
    (state) => state.App
  );

  const {
    blocksPermissions: { ["plant-phenology"]: plantPhenologyPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = plantPhenologyPermissions;

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = (e) => {
    const { searchPlantName, searchRowNumber } = state;

    setState({
      ...state,
      filterPlantNameDropdownVisible: false,
      filterRowNumberDropdownVisible: false,
    });

    const newFilters = _.cloneDeep(filters);

    newFilters["name"] = searchPlantName;
    newFilters["row.rowNumber"] = searchRowNumber;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    if (greenhouseId) filters["row.greenhouseId"] = [greenhouseId];

    dispatch(setFilters(filters));
  }, [dispatch, filters, greenhouseId]);

  const dataSource = [];
  Object.keys(plants).map((plant, index) => {
    return dataSource.push({
      ...plants[plant],
      key: plant,
    });
  });

  const handleRecord = (actionName, plant) => {
    dispatch(saveIntoAgroInvest(plant, actionName));
  };

  const handleModal = async (plant = null) => {
    await dispatch(toggleModal(plant));
    dispatch(rowsLoadFromAgroInvest());
  };

  const plantUpdate = (key, value) => {
    if (key) plant[key] = value;
    dispatch(update(plant));
  };

  const plantUpdateGreenhouse = async (greenhouseId) => {
    await plantUpdate("greenhouseId", greenhouseId);

    dispatch(rowsLoadFromAgroInvest());
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters = _.assign(filters, newFilters);

    dispatch(setFilters(newFilters, pagination.current));
  };

  const generateActionButtons = (row) => {
    const allowedGreenhouseIds = _.map(allowedGreenhouses, (g) => g.id);
    const isAttachedUser = _.includes(
      allowedGreenhouseIds,
      _.get(row, "row.greenhouseId")
    );

    const style =
      isAttachedUser && hasAccessToEdit
        ? {}
        : {
            opacity: "0.3",
          };

    return (
      <ActionWrapper>
        <a
          onClick={() => handleModal(row)}
          disabled={!isAttachedUser || !hasAccessToEdit}
          href="# "
        >
          <i className="ion-android-create" style={{ ...style }} />
        </a>
      </ActionWrapper>
    );
  };

  const columns = [
    {
      title: "Имя",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchPlantName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterPlantNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterPlantNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Теплица",
      dataIndex: "greenhouse",
      key: "row.greenhouseId",
      filters: _.map(greenhouses, (g) => {
        return {
          text: g.description,
          value: g.id,
        };
      }),
      filteredValue: filters["row.greenhouseId"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        const aGreenhouseName = _.get(
          _.find(greenhouses, (g) => g.id === a.row.greenhouseId),
          "description",
          null
        );
        const bGreenhouseName = _.get(
          _.find(greenhouses, (g) => g.id === b.row.greenhouseId),
          "description",
          null
        );

        if (aGreenhouseName < bGreenhouseName) return -1;
        if (aGreenhouseName > bGreenhouseName) return 1;
        return 0;
      },
      render: (text, row) => {
        return (
          <>
            {_.get(
              _.find(greenhouses, (g) => g.id === row.row.greenhouseId),
              "description",
              row.row.greenhouseId
            )}
          </>
        );
      },
    },
    {
      title: "Ряд",
      dataIndex: "row",
      key: "row",
      sorter: (a, b) => {
        if (+_.get(a, "row.rowNumber") < +_.get(b, "row.rowNumber")) return -1;
        if (+_.get(a, "row.rowNumber") > +_.get(b, "row.rowNumber")) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchRowNumber", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters["row.rowNumber"] ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterRowNumberDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterRowNumberDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return <>{_.get(row, "row.rowNumber", "n/a")}</>;
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return generateActionButtons(row);
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Растения <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                disabled={!hasAccessToEdit}
                icon="plus"
                onClick={() => handleModal()}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {plant.id ? `Изменить параметр "${plant.name}"` : "Добавить"}
                <br />
                <small>{plant.id ? plant.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("PATCH", plant)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Значение" hasFeedback>
                  <Input
                    placeholder="Имя"
                    value={plant.name}
                    onChange={({ target: { value } }) =>
                      plantUpdate("name", value)
                    }
                  />
                </FormItem>
                <FormItem label="Теплица" hasFeedback>
                  <Select
                    size="large"
                    value={_.get(plant, "greenhouseId", undefined)}
                    defaultValue={greenhouseId || undefined}
                    placeholder="Выберите теплицу"
                    onChange={(value) => plantUpdateGreenhouse(value)}
                  >
                    {_.map(allowedGreenhouses, (g) => (
                      <Option value={g.id} key={g.id}>
                        {g.description}
                      </Option>
                    ))}
                  </Select>
                </FormItem>
                {_.get(plant, "greenhouseId") && (
                  <FormItem label="Ряд" hasFeedback>
                    <Select
                      size="large"
                      value={
                        isLoadingRows || _.isEmpty(rows)
                          ? undefined
                          : plant.rowId || undefined
                      }
                      placeholder={
                        _.isEmpty(rows) && !isLoadingRows
                          ? "Грядки не найдены"
                          : isLoadingRows
                          ? "Загрузка"
                          : "Выберите Грядку"
                      }
                      onChange={(value) => plantUpdate("rowId", value)}
                      loading={isLoadingRows}
                    >
                      {_.map(rows, (r) => (
                        <Option value={r.id} key={r.id}>
                          {r.rowNumber}
                        </Option>
                      ))}
                    </Select>
                  </FormItem>
                )}
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
