import React, { useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import plantProtectionActions from "@iso/redux/plantProtection/actions";
import Button from "@iso/components/uielements/button";
import { Grid } from "react-flexbox-grid";
import Modal from "@iso/components/Feedback/Modal";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";

import _ from "lodash";

import {
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  Fieldset,
  ActionWrapper,
} from "./PlantProtection.styles";

const FormItem = Form.Item;

export default function IntensityInterface() {
  const {
    intensities,
    intensity,
    modalActiveIntensity,
    isLoading,
  } = useSelector((state) => state.PlantProtection);

  const {
    blocksPermissions: { ["plant-protection"]: plantProtectionPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const {
    ["intensity"]: { edit: hasAccessToEdit },
  } = plantProtectionPermissions.blocks;

  const {
    toggleModalIntensity,
    updateIntensity,
    saveIntoAgroInvestIntensity,
    deleteIntensity,
  } = plantProtectionActions;

  const [data, setData] = useState([]);
  const [modalName, setModalName] = useState("");
  const [modalColor, setModalColor] = useState("");
  const [modalOrderby, setModalOrderBy] = useState(null);

  const dispatch = useDispatch();

  const compare = (a, b) => {
    if (a.orderBy < b.orderBy) {
      return -1;
    }
    if (a.orderBy > b.orderBy) {
      return 1;
    }
    return 0;
  };

  useEffect(() => {
    setData(intensities.sort(compare));
  }, [intensities]);

  const handleModal = async (intensity = null) => {
    if (intensity) {
      setModalName(intensity.name);
      setModalColor(intensity.color);
      setModalOrderBy(intensity.orderBy);
    } else {
      setModalName("");
      setModalColor("");
      setModalOrderBy("");
    }
    await dispatch(toggleModalIntensity(intensity));
    // dispatch(rowsLoadFromAgroInvest());
  };

  const wrapperStyles = {
    display: "flex",
    alignItems: "center",
  };

  const intensityUpdate = (key, value) => {
    if (key) intensity[key] = value;
    dispatch(updateIntensity(intensity));
  };

  const handleRecord = (actionName, intensity) => {
    intensityUpdate("name", modalName);
    intensityUpdate("color", modalColor);
    intensityUpdate("orderBy", modalOrderby);
    dispatch(saveIntoAgroInvestIntensity(intensity, actionName));
  };

  const generateActionButtons = (row) => {
    return (
      <ActionWrapper>
        {hasAccessToEdit ? (
          <a onClick={() => handleModal(row)} href="# ">
            <i className="ion-android-create" />
          </a>
        ) : (
          <a disabled href="# " style={{ opacity: 0.3 }}>
            <i className="ion-android-create" />
          </a>
        )}
      </ActionWrapper>
    );
  };

  const removeIntensity = (pest) => {
    dispatch(deleteIntensity(pest));
  };

  const columns = [
    {
      title: "Интенсивность",
      key: "name",
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Значение цвета",
      key: "color",
      render: (text, row) => {
        return (
          <div style={wrapperStyles}>
            <div
              style={{
                background: `${row.color}`,
                width: "30px",
                height: "30px",
                borderRadius: "50%",
                border: "1px solid #ccc",
                marginRight: "20px",
              }}
            />
            {row.color}
          </div>
        );
      },
    },
    {
      title: "Баллы",
      key: "orderBy",
      sorter: (a, b) => {
        if (a.orderBy < b.orderBy) return -1;
        if (a.orderBy > b.orderBy) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.orderBy;
      },
    },
    {
      title: "Изменить",
      key: "change",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return generateActionButtons(row);
      },
    },
    {
      title: "Удалить",
      key: "delete",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return hasAccessToEdit ? (
          <a onClick={() => removeIntensity(row)} href="# ">
            X
          </a>
        ) : (
          <a disabled style={{ opacity: 0.3 }} href="# ">
            X
          </a>
        );
      },
    },
  ];

  return (
    <>
      <TitleWrapper>
        <ComponentTitle>Интенсивность поражения</ComponentTitle>
        <div>
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            disabled={!hasAccessToEdit}
            icon="plus"
            onClick={() => handleModal()}
            style={{ marginLeft: "10px" }}
          />
        </div>
      </TitleWrapper>
      {intensities && (
        <>
          <Grid fluid>
            <TableWrapper
              rowKey="name"
              columns={columns}
              bordered={true}
              dataSource={data || []}
              className="isoSimpleTable"
              pagination={{
                hideOnSinglePage: true,
                total: intensities.length,
              }}
            />
          </Grid>
        </>
      )}
      <Modal
        visible={modalActiveIntensity}
        onClose={() => dispatch(toggleModalIntensity(null))}
        title={
          <>
            {intensity.id
              ? `Изменить параметр "${intensity.name}"`
              : "Добавить"}
            <br />
            <small>{intensity.id ? intensity.id : "n/a"}</small>
          </>
        }
        okText={"Сохранить"}
        onOk={() => handleRecord("PATCH", intensity)}
        onCancel={() => dispatch(toggleModalIntensity(null))}
        okButtonProps={{ disabled: false }}
        cancelButtonProps={{ disabled: false }}
      >
        <Form>
          <Fieldset>
            <FormItem label="Название" hasFeedback>
              <Input
                placeholder="Название"
                value={modalName}
                onChange={(e) => setModalName(e.target.value)}
              />
            </FormItem>
          </Fieldset>
          <Fieldset>
            <FormItem label="Цвет" hasFeedback>
              <div style={wrapperStyles}>
                <input
                  type="color"
                  value={modalColor}
                  onChange={(e) => setModalColor(e.target.value)}
                  style={{ marginRight: "20px", cursor: "pointer" }}
                />
                <div
                  style={{
                    width: 40,
                    height: 40,
                    borderRadius: "50%",
                    border: "1px solid #ccc",
                    marginRight: 20,
                    backgroundColor: modalColor,
                  }}
                />
                <div>{modalColor}</div>
              </div>
            </FormItem>
          </Fieldset>
          <Fieldset>
            <FormItem label="Баллы" hasFeedback>
              <Input
                placeholder="Баллы"
                value={modalOrderby}
                onChange={(e) => setModalOrderBy(e.target.value)}
              />
            </FormItem>
          </Fieldset>
        </Form>
      </Modal>
    </>
  );
}
