import React from "react";
import { useDispatch, useSelector } from "react-redux";
import positionActions from "@iso/redux/position/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./Position.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = positionActions;

export default function Position() {
  const {
    positions,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    position,
    filters,
  } = useSelector((state) => state.Position);

  const {
    blocksPermissions: { positions: positionsPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = positionsPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchPositionName: "",
    filterPositionNameDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchPositionName } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchPositionName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(positions).map((position, index) => {
    return dataSource.push({
      ...positions[position],
      key: position,
    });
  });

  const handleRecord = (actionName, position) => {
    dispatch(saveIntoAgroInvest(position, actionName));
  };

  const handleModal = (position = null) => {
    dispatch(toggleModal(position));
  };

  const positionUpdate = (key, value) => {
    if (key) position[key] = value;
    dispatch(update(position));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.name;

    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "Название",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchPositionName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterPositionNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterPositionNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Дата ввода",
      dataIndex: "createdAt",
      key: "createdAt",
      sorter: (a, b) => {
        if (a.createdAt < b.createdAt) return -1;
        if (a.createdAt > b.createdAt) return 1;
        return 0;
      },
      render: (text, row) => {
        return moment(row.createdAt).calendar();
      },
    },
    {
      title: "Дата обновления",
      dataIndex: "updatedAt",
      key: "updatedAt",
      sorter: (a, b) => {
        if (a.updatedAt < b.updatedAt) return -1;
        if (a.updatedAt > b.updatedAt) return 1;
        return 0;
      },
      render: (text, row) => {
        return moment(row.updatedAt).calendar();
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            <a
              onClick={() => handleModal(row)}
              disabled={!hasAccessToEdit}
              href="#"
            >
              <i className="ion-android-create" />
            </a>
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Должности <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                disabled={!hasAccessToEdit}
                loading={isLoading}
                icon="plus"
                onClick={() => dispatch(toggleModal(null))}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {position.id
                  ? "Изменить параметр " + position.name
                  : "Добавить"}
                <br />
                <small>{position.id ? position.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", position)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Название" hasFeedback>
                  <Input
                    placeholder="Имя объекта"
                    value={position.name}
                    onChange={({ target: { value } }) =>
                      positionUpdate("name", value)
                    }
                  />
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
