import React, { useState, useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import _ from "lodash";

import QualityControlActions from "@iso/redux/qualityControl/actions";

import Button from "@iso/components/uielements/button";
import { Grid } from "react-flexbox-grid";
import Modal from "@iso/components/Feedback/Modal";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon, Select, notification } from "antd";

import {
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  Fieldset,
  ActionWrapper,
} from "./QualityControl.styles";

const FormItem = Form.Item;
const { Option } = Select;

const requiredFields = [
  { field: "name", name: "Имя" },
  { field: "sectionId", name: "Вид работ" },
];

export default function CheckSheet() {
  const [state, setState] = React.useState({});

  const {
    isLoading,
    isLoadingSections,
    checkMetrics,
    checkMetric,
    modalActive,
    filters,
    sections,
  } = useSelector((state) => state.QualityControl);

  const { greenhouseId } = useSelector((state) => state.App);

  const {
    blocksPermissions: { ["quality-control"]: qualityControlPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const {
    ["check-metrics"]: { edit: hasAccessToEdit },
  } = qualityControlPermissions.blocks;

  const dispatch = useDispatch();

  const {
    loadFromAgroInvest,
    sectionsLoadFromAgroInvest,
    toggleModal,
    update,
    deleteCheck,
    saveIntoAgroInvest,
    setFilters,
  } = QualityControlActions;

  useEffect(() => {
    dispatch(loadFromAgroInvest());
    dispatch(sectionsLoadFromAgroInvest());
  }, [dispatch, loadFromAgroInvest, greenhouseId]);

  const [dataSource, setDataSource] = useState([]);
  const [modalName, setModalName] = useState([]);
  const [modalSectionId, setModalSectionId] = useState([]);

  useEffect(() => {
    setDataSource(
      checkMetrics
        .filter(({ section: { greenhouseSections } }) => (
          greenhouseSections.find((greenhouse) => (
            greenhouse.greenhouseId === greenhouseId
          ))
        ))
        .map((check, index) => {
          return {
            ...check,
            key: `${check.id}-${index}`,
          };
        })
    );
  }, [checkMetrics]);

  const handleModal = async (checkMetric = null) => {
    if (checkMetric) {
      setModalName(checkMetric.name);
      setModalSectionId(checkMetric.sectionId);
    } else {
      setModalName("");
      setModalSectionId("");
    }
    await dispatch(toggleModal(checkMetric));
  };

  const checkMetricUpdate = (key, value) => {
    if (key) checkMetric[key] = value;
    dispatch(update(checkMetric));
  };

  const handleRecord = (checkMetric) => {
    checkMetricUpdate("name", modalName);
    checkMetricUpdate("sectionId", modalSectionId);

    checkMetric.name = _.trim(checkMetric.name);

    const errorFields = _.compact(
      _.map(requiredFields, (requiredField) => {
        if (checkMetric[requiredField.field]) return null;
        else return requiredField.name;
      })
    );

    if (!_.isEmpty(errorFields)) {
      notification.error({
        message: "Заполните поля: " + _.join(errorFields, ", "),
        duration: 8,
      });
    } else {
      dispatch(saveIntoAgroInvest(checkMetric));
    }
  };

  const removeCheck = (checkMetric) => {
    dispatch(deleteCheck(checkMetric));
  };

  const generateActionButtons = (row) => {
    return (
      <ActionWrapper>
        {hasAccessToEdit ? (
          <a onClick={() => handleModal(row)} href="# ">
            <i className="ion-android-create" />
          </a>
        ) : (
          <a disabled style={{ opacity: 0.3 }} href="# ">
            <i className="ion-android-create" />
          </a>
        )}
      </ActionWrapper>
    );
  };

  const dropFilters = () => {
    setState({});
    dispatch(setFilters({}, 1));
  };

  function onInputChange(event, type) {
    setState({ ...state, [type]: event.target.value });
  }

  function onSearch() {
    const { searchText } = state;

    setState({
      ...state,
      filterDropdownNameVisible: false,
      filterDropdownCodeVisible: false,
      filterDropdownVersionVisible: false,
    });

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchText;

    dispatch(setFilters(newFilters, 1));
  }

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.searchText;
    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "",
      key: "image",
      width: 100,
      render: (text, row) => {
        return row.image ? (
          <img
            src={`${process.env.REACT_APP_MEDIA_URL}/download/${row.image}`}
            style={{ width: 100, height: 100 }}
          />
        ) : null;
      },
    },
    {
      title: "Название",
      key: "name",
      sorter: (a, b) => {
        if (_.get(a, "name", "") < _.get(b, "name", "")) return -1;
        if (_.get(a, "name", "") > _.get(b, "name", "")) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText={state.searchText}
          onInputChange={(e) => onInputChange(e, "searchText")}
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Вид работы",
      key: "sectionId",
      sorter: (a, b) => {
        if (_.get(a, "section.name", "") < _.get(b, "section.name", ""))
          return -1;
        if (_.get(a, "section.name", "") > _.get(b, "section.name", ""))
          return 1;
        return 0;
      },
      filters: _.sortBy(
        _.map(sections, (section) => {
          return {
            text: _.get(section, "name", "n/a"),
            value: _.get(section, "sectionId", "n/a"),
          };
        }),
        "text"
      ),
      filteredValue: filters["sectionId"] || [],
      filterMultiple: false,
      render: (text, row) => {
        return _.get(row.section, "name", "n/a");
      },
    },
    {
      title: "Тип",
      dataIndex: "category",
      key: "category",
      width: "100px",
      filters: [
        {
          text: "Огурцы",
          value: "Огурцы",
        },
        {
          text: "Томаты",
          value: "Томаты",
        },
      ],
      filteredValue: filters["category"] || [],
      filterMultiple: false,
      render: (text, row) => {
        return row.category;
      },
    },
    {
      title: "Изменить",
      key: "edit",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return generateActionButtons(row);
      },
    },
    {
      title: "Удалить",
      key: "delete",
      width: "60px",
      align: "center",
      className: "noWrapCell",
      render: (text, row) => {
        return hasAccessToEdit ? (
          <a onClick={() => removeCheck(row)} href="# ">
            X
          </a>
        ) : (
          <a disabled style={{ opacity: 0.3 }} href="# ">
            X
          </a>
        );
      },
    },
  ];

  return (
    <>
      <Grid fluid>
        <TitleWrapper>
          <ComponentTitle>Списки проверок</ComponentTitle>
          <div>
            <Button
              type="primary"
              shape="circle"
              loading={isLoading}
              disabled={!hasAccessToEdit}
              icon="plus"
              onClick={() => handleModal()}
              style={{ marginLeft: "10px" }}
            />
            <Button
              type="primary"
              shape="circle"
              loading={isLoading}
              icon="filter"
              onClick={dropFilters}
              style={{ marginLeft: "10px" }}
            />
          </div>
        </TitleWrapper>
        {checkMetrics && (
          <>
            <Grid fluid>
              <TableWrapper
                rowKey="key"
                columns={columns}
                bordered={true}
                loading={isLoading}
                dataSource={dataSource || []}
                className="isoSimpleTable"
                onChange={handleTableChange}
                pagination={{
                  hideOnSinglePage: true,
                  total: dataSource.length,
                }}
              />
            </Grid>
          </>
        )}
      </Grid>
      <Modal
        visible={modalActive}
        onClose={() => dispatch(toggleModal(null))}
        title={
          <>
            {checkMetric.id
              ? `Изменить параметр "${checkMetric.name}"`
              : "Добавить"}
            <br />
            <small>{checkMetric.id ? checkMetric.id : "n/a"}</small>
          </>
        }
        okText={"Сохранить"}
        onOk={() => handleRecord(checkMetric)}
        onCancel={() => dispatch(toggleModal(null))}
        okButtonProps={{ disabled: !hasAccessToEdit }}
        cancelButtonProps={{ disabled: false }}
      >
        <Form>
          <Fieldset>
            <FormItem label="Название" hasFeedback>
              <Input
                placeholder="Название"
                value={modalName}
                onChange={(e) => setModalName(e.target.value)}
              />
            </FormItem>
          </Fieldset>
          <Fieldset>
            <FormItem label="Вид работ" hasFeedback>
              <Select
                disabled={isLoadingSections}
                dropdownMatchSelectWidth={false}
                loading={isLoadingSections}
                value={_.get(
                  _.find(
                    sections,
                    (section) => section.sectionId === modalSectionId
                  ),
                  "name",
                  ""
                )}
                onChange={(value) => setModalSectionId(value)}
              >
                {_.map(_.sortBy(sections, ["name"]), (section) => {
                  return (
                    <Option key={section.sectionId} value={section.sectionId}>
                      {section.name}
                    </Option>
                  );
                })}
              </Select>
            </FormItem>
          </Fieldset>
        </Form>
      </Modal>
    </>
  );
}
