import React from "react";
import { useDispatch, useSelector } from "react-redux";
import { Link } from "react-router-dom";
import ratingActions from "@iso/redux/rating/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./Rating.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = ratingActions;

export default function Rating() {
  const {
    ratings,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    rating,
    filters,
  } = useSelector((state) => state.Rating);

  const {
    blocksPermissions: { ratings: ratingsPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = ratingsPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchRatingName: "",
    filterRatingNameDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchRatingName } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchRatingName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(ratings).map((rating, index) => {
    return dataSource.push({
      ...ratings[rating],
      key: rating,
    });
  });

  const handleRecord = (actionName, rating) => {
    dispatch(saveIntoAgroInvest(rating, actionName));
  };

  const handleModal = (rating = null) => {
    dispatch(toggleModal(rating));
  };

  const ratingUpdate = (key, value) => {
    if (key) rating[key] = value;
    dispatch(update(rating));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.name;

    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "Вид работ",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchRatingName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterRatingNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterRatingNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Оценка",
      dataIndex: "description",
      key: "description",
      sorter: (a, b) => {
        if (a.description < b.description) return -1;
        if (a.description > b.description) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.rating + "/" + row.ratingInt;
      },
    },
    {
      title: "Код работника",
      dataIndex: "parentId",
      key: "parentId",
      sorter: (a, b) => {
        if (a.parentId < b.parentId) return -1;
        if (a.parentId > b.parentId) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.workerId;
      },
    },
    {
      title: "Дата",
      dataIndex: "period",
      key: "period",
      sorter: (a, b) => {
        if (a.period < b.period) return -1;
        if (a.period > b.period) return 1;
        return 0;
      },
      render: (text, row) => {
        return moment(row.period).calendar();
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            {hasAccessToEdit ? (
              <a onClick={() => handleModal(row)} href="# ">
                <i className="ion-android-create" />
              </a>
            ) : (
              <a disabled href="# ">
                <i className="ion-android-create" style={{ color: "gray" }} />
              </a>
            )}

            <Link to={`/dashboard/audit/${row.workerId}`}>
              <i className="ion-android-time" />
            </Link>
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Рейтинги <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                disabled={!hasAccessToEdit}
                loading={isLoading}
                icon="plus"
                onClick={() => dispatch(toggleModal(null))}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {rating.id ? "Изменить параметр " + rating.name : "Добавить"}
                <br />
                <small>{rating.id ? rating.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", rating)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Название" hasFeedback>
                  <Input
                    placeholder="Имя объекта"
                    value={rating.name}
                    onChange={({ target: { value } }) =>
                      ratingUpdate("name", value)
                    }
                  />
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
