import React from "react";
import { useDispatch, useSelector } from "react-redux";
import _ from "lodash";
import { Checkbox, Icon } from "antd";
import moment from "moment-timezone";
import "moment/locale/ru";

import Button from "@iso/components/uielements/button";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import Form from "@iso/components/uielements/form";
import Input, { Textarea } from "@iso/components/uielements/input";
import Radio, { RadioGroup } from "@iso/components/uielements/radio";

import sectionsActions from "@iso/redux/sections/actions";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
  StatusTag,
} from "./Sections.styles";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = sectionsActions;

export default function Section() {
  const {
    sections,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    section,
    filters,
  } = useSelector((state) => state.Sections);

  const {
    blocksPermissions: { sections: sectionsPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = sectionsPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchSectionName: "",
    filterSectionNameDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchSectionName } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchSectionName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(sections).map((section, index) => {
    return dataSource.push({
      ...sections[section],
      key: section,
    });
  });

  const handleRecord = (actionName, section) => {
    dispatch(saveIntoAgroInvest(section, actionName));
  };

  const handleModal = (section = null) => {
    dispatch(toggleModal(section));
  };

  const sectionUpdate = (key, value) => {
    if (key && key === "sectionType") {
      section.isHarvesting = value && value === "isHarvesting";
      section.isPhenology = value && value === "isPhenology";
      section.isGuarding = value && value === "isGuarding";
      section.isSpraying = value && value === "isSpraying";
      section.isRegular = value && value === "isRegular";
      section.isQuality = value && value === "isQuality";
    } else if (key) {
      section[key] = value;
    }
    dispatch(update(section));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.name;

    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "Название",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchSectionName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterSectionNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterSectionNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Описание",
      dataIndex: "description",
      key: "description",
      sorter: (a, b) => {
        if (a.description < b.description) return -1;
        if (a.description > b.description) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.description !== "undefined" && _.get(row, "description", "");
      },
    },
    {
      title: "Структура",
      dataIndex: "parentId",
      key: "parentId",
      sorter: (a, b) => {
        if (a.structure[0].name < b.structure[0].name) return -1;
        if (a.structure[0].name > b.structure[0].name) return 1;
        return 0;
      },
      render: (text, row) => {
        return (
          <ul style={{ padding: 0 }}>
            {_.map(row.structure, (s) => {
              return <li key={s.id}>{s.name}</li>;
            })}
          </ul>
        );
      },
    },
    {
      title: "Cбор",
      dataIndex: "isHarvesting",
      key: "isHarvesting",
      sorter: (a, b) => {
        if (a.isHarvesting < b.isHarvesting) return -1;
        if (a.isHarvesting > b.isHarvesting) return 1;
        return 0;
      },
      render: (text, row) => {
        const className = row.isHarvesting ? "active" : "inactive";

        return (
          <>
            <StatusTag className={className}>
              {row.isHarvesting ? "Да" : "Нет"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Фенология",
      dataIndex: "isPhenology",
      key: "isPhenology",
      sorter: (a, b) => {
        if (a.isPhenology < b.isPhenology) return -1;
        if (a.isPhenology > b.isPhenology) return 1;
        return 0;
      },
      render: (text, row) => {
        const className = row.isPhenology ? "active" : "inactive";

        return (
          <>
            <StatusTag className={className}>
              {row.isPhenology ? "Да" : "Нет"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Мониторинг",
      dataIndex: "isGuarding",
      key: "isGuarding",
      sorter: (a, b) => {
        if (a.isGuarding < b.isGuarding) return -1;
        if (a.isGuarding > b.isGuarding) return 1;
        return 0;
      },
      render: (text, row) => {
        const className = row.isGuarding ? "active" : "inactive";

        return (
          <>
            <StatusTag className={className}>
              {row.isGuarding ? "Да" : "Нет"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Опрыскивание",
      dataIndex: "isSpraying",
      key: "isSpraying",
      sorter: (a, b) => {
        if (a.isSpraying < b.isSpraying) return -1;
        if (a.isSpraying > b.isSpraying) return 1;
        return 0;
      },
      render: (text, row) => {
        const className = row.isSpraying ? "active" : "inactive";

        return (
          <>
            <StatusTag className={className}>
              {row.isSpraying ? "Да" : "Нет"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Контроль качества",
      dataIndex: "isQuality",
      key: "isQuality",
      sorter: (a, b) => {
        if (a.isQuality < b.isQuality) return -1;
        if (a.isQuality > b.isQuality) return 1;
        return 0;
      },
      render: (text, row) => {
        const className = row.isQuality ? "active" : "inactive";

        return (
          <>
            <StatusTag className={className}>
              {row.isQuality ? "Да" : "Нет"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Проверяемый",
      dataIndex: "hasQualityCheck",
      key: "hasQualityCheck",
      sorter: (a, b) => {
        if (a.hasQualityCheck < b.hasQualityCheck) return -1;
        if (a.hasQualityCheck > b.hasQualityCheck) return 1;
        return 0;
      },
      render: (text, row) => {
        const className = row.hasQualityCheck ? "active" : "inactive";

        return (
          <>
            <StatusTag className={className}>
              {row.hasQualityCheck ? "Да" : "Нет"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Actions",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            {hasAccessToEdit ? (
              <a onClick={() => handleModal(row)} href="#">
                <i className="ion-android-create" />
              </a>
            ) : (
              <a href="#" disabled>
                <i className="ion-android-create" style={{ color: "#aaa" }} />
              </a>
            )}
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Виды работ <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {section.id ? "Изменить параметр " : "Добавить"}
                <br />
                {_.get(section, "name", "")}
                <br />
                <small>{section.id ? section.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", section)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Название" hasFeedback>
                  <Input
                    placeholder="Имя объекта"
                    value={section.name}
                    onChange={({ target: { value } }) =>
                      sectionUpdate("name", value)
                    }
                  />
                </FormItem>
                <FormItem label="Описание" hasFeedback>
                  <Textarea
                    rows={4}
                    placeholder="Введите описание этого вида работ"
                    value={
                      section.description === "undefined"
                        ? ""
                        : _.get(section, "description", "")
                    }
                    onChange={({ target: { value } }) =>
                      sectionUpdate("description", value)
                    }
                  ></Textarea>
                </FormItem>
                <FormItem label="Модуль обработки" hasFeedback>
                  <RadioGroup
                    onChange={({ target: { value } }) =>
                      sectionUpdate("sectionType", value)
                    }
                    name={"sectionType"}
                    value={
                      (section.isHarvesting && "isHarvesting") ||
                      (section.isPhenology && "isPhenology") ||
                      (section.isGuarding && "isGuarding") ||
                      (section.isSpraying && "isSpraying") ||
                      (section.isQuality && "isQuality") ||
                      (section.isRegular && "isRegular")
                    }
                  >
                    <Radio value={"isRegular"}>Нет</Radio>
                    <br />
                    <Radio value={"isHarvesting"}>Сбор</Radio>
                    <br />
                    <Radio value={"isPhenology"}>Фенология</Radio>
                    <br />
                    <Radio value={"isGuarding"}>Мониторинг</Radio>
                    <br />
                    <Radio value={"isSpraying"}>Опрыскивание</Radio>
                    <br />
                    <Radio value={"isQuality"}>Контроль качества</Radio>
                    <br />
                  </RadioGroup>
                </FormItem>
                <FormItem label="Контроль качества" hasFeedback>
                  <Checkbox
                    checked={section.hasQualityCheck}
                    onChange={({ target: { checked } }) =>{
                    sectionUpdate('hasQualityCheck', checked)}
                  }
                  >
                    Проверяемый
                  </Checkbox>
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
