import React, { useState, useEffect } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import settingActions from '@iso/redux/setting/actions';
import _ from 'lodash';
import LayoutContentWrapper from '@iso/components/utility/layoutWrapper';
import Box from '@iso/components/utility/box';
import ContentHolder from '@iso/components/utility/contentHolder';
import { TitleWrapper, ComponentTitle, TitleRow } from './Setting.styles';
import {
  Spin,
  TimePicker,
  Button,
  Form,
  InputNumber,
  Notification,
  Select,
} from 'antd';

import moment from 'moment-timezone';
import 'moment/locale/ru';

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  loadTechnicalWindows,
  addTechnicalWindow,
  removeTechnicalWindow,
  addTechnicalWindowSection,
  addTechnicalWindowSections,
  removeTechnicalWindowSection,
  loadSections,
  changeTechnicalWindow,
} = settingActions;

const TechnicalWindow = ({ technicalWindow, filteredGreenhouseSections }) => {
  const dispatch = useDispatch();
  const { statusTechnicalWindows } = useSelector((state) => state.Setting);
  const selectedNameSections = technicalWindow.technicalWindowSections
    .map((t) => t.section.name)
    .sort();

  const { role } = useSelector((state) => state.Auth);
  const hasAccessForEdit = _.includes(['superuser', 'tech-support'], role);

  const handleAddTechnicalWindowSection = (sectionId) => {
    dispatch(
      addTechnicalWindowSection({
        technicalWindowId: technicalWindow.id,
        sectionId,
      })
    );
  };
  const handleRemoveTechnicalWindowSection = (sectionName) => {
    dispatch(
      removeTechnicalWindowSection({
        technicalWindowId: technicalWindow.id,
        sectionId: _.get(
          _.find(
            technicalWindow.technicalWindowSections,
            (s) => s.section.name === sectionName
          ),
          'section.id'
        ),
      })
    );
  };
  const handleChangeTechnicalWindow = ({ id, paramName, paramValue }) => {
    dispatch(changeTechnicalWindow({ id, paramName, paramValue }));
  };
  return (
    <FormItem
      style={{ fontWeight: 'bold', marginBottom: '20px' }}
      label="Рабочее время:"
      hasFeedback
    >
      <div
        style={{
          display: 'flex',
          justifyContent: 'space-between',
          alignItems: 'center',
        }}
      >
        <TitleRow>{'Начало'}</TitleRow>
        <TimePicker
          placeholder="Выберите время"
          disabled={!hasAccessForEdit}
          allowClear={false}
          inputReadOnly={true}
          style={{ minWidth: 180, marginRight: '10px' }}
          value={moment.utc(technicalWindow.start).local()}
          minuteStep={10}
          format="HH:mm"
          onChange={(time, timeString) => {
            handleChangeTechnicalWindow({
              id: technicalWindow.id,
              paramName: 'start',
              paramValue: moment(timeString, 'HH:mm'),
            });
          }}
        />
        <TitleRow>{'окончание'}</TitleRow>
        <TimePicker
          placeholder="Выберите время"
          disabled={!hasAccessForEdit}
          allowClear={false}
          inputReadOnly={true}
          style={{ minWidth: 180 }}
          value={moment.utc(technicalWindow.finish).local()}
          minuteStep={10}
          format="HH:mm"
          onChange={(time, timeString) => {
            handleChangeTechnicalWindow({
              id: technicalWindow.id,
              paramName: 'finish',
              paramValue: moment(timeString, 'HH:mm'),
            });
          }}
        />
        <Button
          type="danger"
          icon="delete"
          disabled={!hasAccessForEdit}
          style={{ marginLeft: 'auto' }}
          onClick={() => dispatch(removeTechnicalWindow(technicalWindow.id))}
        >
          Удалить технологическое окно
        </Button>
      </div>
      <Select
        mode="multiple"
        placeholder="Выберите вид работы"
        value={selectedNameSections}
        disabled={!hasAccessForEdit}
        onSelect={handleAddTechnicalWindowSection}
        onDeselect={handleRemoveTechnicalWindowSection}
        style={{ width: '100%', marginTop: '15px' }}
        loading={statusTechnicalWindows === 'loading'}
      >
        {filteredGreenhouseSections.map((item) => (
          <Select.Option key={item.id} value={item.sectionId}>
            {item.name}
          </Select.Option>
        ))}
      </Select>
      <Button
        type="dashed"
        icon="plus"
        disabled={statusTechnicalWindows === 'loading' || !hasAccessForEdit}
        onClick={() =>
          dispatch(
            addTechnicalWindowSections({
              technicalWindowId: technicalWindow.id,
              greenhouseSections: filteredGreenhouseSections,
            })
          )
        }
      >
        Добавить все доступные виды работ
      </Button>
    </FormItem>
  );
};
const Setting = (props) => {
  const { getFieldDecorator, validateFields } = props.form;

  const dispatch = useDispatch();

  const [timeSettings, setTimeSettings] = useState({});

  const { role } = useSelector((state) => state.Auth);
  const { greenhouseId } = useSelector((state) => state.App);
  const {
    isLoading,
    isSaving,
    settings,
    technicalWindows,
    statusTechnicalWindows,
    greenhouseSections,
    redifinedSections,
  } = useSelector((state) => state.Setting);

  const {
    blocksPermissions: { settings: settingsPermissions },
  } = useSelector((state) => _.get(state, 'Auth.userPermissions'));

  const { edit: hasAccessToEdit } = settingsPermissions;

  useEffect(() => {
    dispatch(loadFromAgroInvest());
    dispatch(loadSections());
    dispatch(loadTechnicalWindows());
  }, [dispatch, greenhouseId]);

  useEffect(() => {
    if (settings) {
      setTimeSettings(settings.workingTimeConfig);
    }
  }, [isLoading, settings]);

  useEffect(() => {
    if (settings) {
      setTimeSettings(settings.workingTimeConfig);
    }
  }, [isSaving, settings]);

  const saveSettings = (e) => {
    e.preventDefault();
    validateFields((err, values) => {
      if (err) {
        Notification.error({
          message: 'Проверьте вводимые данные',
          duration: 5,
        });
        return;
      }
      const data = {
        workingTimeConfig: {
          start: timeSettings.start || '03:00',
          finish: timeSettings.finish || '17:00',
        },
        numberOfTask: {
          morningPeriod: values.morningPeriod,
          dayTimePeriod: values.dayTimePeriod,
        },
        numberOfTaskForController: {
          morningPeriod: values.morningPeriodController,
          dayTimePeriod: values.dayTimePeriodController,
        },
        freqRating: {
          numberInMounth: values.numberInMounth,
        },
      };
      dispatch(saveIntoAgroInvest(data));
    });
  };
  const saveTechnicalWindows = (e) => {
    e.preventDefault();
    dispatch(settingActions.saveTechnicalWindows({}));
  };

  const hasAccessForEdit = _.includes(['superuser', 'tech-support'], role);
  const filteredGreenhouseSections = greenhouseSections.filter(
    ({ sectionId }) => {
      return !redifinedSections.map((r) => r.section.id).includes(sectionId);
    }
  );
  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: 'hidden' }}>
          <TitleWrapper>
            <ComponentTitle>Общие настройки комплекса</ComponentTitle>
          </TitleWrapper>
          {isLoading ? (
            <Spin
              style={{ textAlign: 'center', width: '100%', margin: '20px 0' }}
            />
          ) : (
            <Form layout="horizontal" onSubmit={saveSettings}>
              <FormItem
                style={{ fontWeight: 'bold' }}
                label="Рабочее время:"
                hasFeedback
              >
                <TitleRow>{'Начало'}</TitleRow>
                <TimePicker
                  placeholder="Выберите время"
                  disabled={isSaving || !hasAccessForEdit}
                  allowClear={false}
                  inputReadOnly={true}
                  style={{ minWidth: 180 }}
                  value={
                    !_.isEmpty(timeSettings)
                      ? moment.utc(timeSettings.start, 'HH:mm').local()
                      : undefined
                  }
                  minuteStep={10}
                  format="HH:mm"
                  onChange={(time, timeString) => {
                    setTimeSettings((prev) => ({
                      ...prev,
                      start: moment(timeString, 'HH:mm').utc().format('HH:mm'),
                    }));
                  }}
                />
                &nbsp;
                <TitleRow>{'окончание'}</TitleRow>
                <TimePicker
                  placeholder="Выберите время"
                  disabled={isSaving || !hasAccessForEdit}
                  allowClear={false}
                  inputReadOnly={true}
                  style={{ minWidth: 180 }}
                  value={
                    !_.isEmpty(timeSettings)
                      ? moment.utc(timeSettings.finish, 'HH:mm').local()
                      : undefined
                  }
                  minuteStep={10}
                  format="HH:mm"
                  onChange={(time, timeString) => {
                    setTimeSettings((prev) => ({
                      ...prev,
                      finish: moment(timeString, 'HH:mm').utc().format('HH:mm'),
                    }));
                  }}
                />
              </FormItem>
              <FormItem
                label="Число задач при автоматическом распределении:"
                style={{ fontWeight: 'bold' }}
                required={false}
                hasFeedback
              >
                <TitleRow>{'С начала рабочего дня и до 15-00'}</TitleRow>
                {getFieldDecorator('morningPeriod', {
                  initialValue: settings.numberOfTask.morningPeriod
                    ? Number(settings.numberOfTask.morningPeriod)
                    : -1,
                  rules: [
                    {
                      type: 'number',
                      min: 0,
                      required: true,
                      message: 'Неверно введено число',
                    },
                  ],
                })(<InputNumber disabled={!hasAccessToEdit} />)}
              </FormItem>
              <FormItem hasFeedback>
                <TitleRow>{'С 15-00 и до окончания рабочего дня'}</TitleRow>
                {getFieldDecorator('dayTimePeriod', {
                  initialValue: settings.numberOfTask.dayTimePeriod
                    ? Number(settings.numberOfTask.dayTimePeriod)
                    : -1,
                  rules: [
                    {
                      type: 'number',
                      min: 0,
                      required: true,
                      message: 'Неверно введено число',
                    },
                  ],
                })(<InputNumber disabled={!hasAccessToEdit} />)}
              </FormItem>
              <FormItem
                label="Общее число задач по контролю качества у мастера-контролера:"
                style={{ fontWeight: 'bold' }}
                required={false}
                hasFeedback
              >
                <TitleRow>{'За период до 12-00'}</TitleRow>
                {getFieldDecorator('morningPeriodController', {
                  initialValue: settings.numberOfTaskForController.morningPeriod
                    ? Number(settings.numberOfTaskForController.morningPeriod)
                    : -1,
                  rules: [
                    {
                      type: 'number',
                      min: 0,
                      required: true,
                      message: 'Неверно введено число',
                    },
                  ],
                })(<InputNumber disabled={!hasAccessToEdit} />)}
              </FormItem>
              <FormItem hasFeedback>
                <TitleRow>{'За период после 12-00'}</TitleRow>
                {getFieldDecorator('dayTimePeriodController', {
                  initialValue: settings.numberOfTaskForController.dayTimePeriod
                    ? Number(settings.numberOfTaskForController.dayTimePeriod)
                    : -1,
                  rules: [
                    {
                      type: 'number',
                      min: 0,
                      required: true,
                      message: 'Неверно введено число',
                    },
                  ],
                })(<InputNumber disabled={!hasAccessToEdit} />)}
              </FormItem>
              <FormItem
                label="Частота автоматической смены рейтингов сотрудников:"
                style={{ fontWeight: 'bold' }}
                required={false}
                hasFeedback
              >
                <TitleRow>{'Число месяцев между автообновлением рейтинга'}</TitleRow>
                {getFieldDecorator('numberInMounth', {
                  initialValue: settings.freqRating.numberInMounth
                    ? Number(settings.freqRating.numberInMounth)
                    : -1,
                  rules: [
                    {
                      type: 'number',
                      min: 0,
                      required: true,
                      message: 'Неверно введено число',
                    },
                  ],
                })(<InputNumber disabled={!hasAccessToEdit} />)}
                <div />
              </FormItem>
              {hasAccessForEdit && (
                <Button
                  type="primary"
                  loading={isSaving}
                  disabled={!hasAccessToEdit}
                  htmlType="submit"
                >
                  Сохранить
                </Button>
              )}
            </Form>
          )}
        </ContentHolder>
      </Box>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: 'hidden' }}>
          <TitleWrapper>
            <ComponentTitle>Настройки выбранной теплицы</ComponentTitle>
          </TitleWrapper>
          {statusTechnicalWindows === 'idle' &&
          statusTechnicalWindows === 'loading' ? (
            <Spin
              style={{ textAlign: 'center', width: '100%', margin: '20px 0' }}
            />
          ) : (
            <Form layout="horizontal" onSubmit={saveTechnicalWindows}>
              {_.sortBy(technicalWindows, 'id').map((t) => (
                <TechnicalWindow
                  key={t.id}
                  technicalWindow={t}
                  filteredGreenhouseSections={filteredGreenhouseSections}
                />
              ))}
              <Button
                type="primary"
                icon="plus"
                disabled={!hasAccessToEdit}
                onClick={() =>
                  dispatch(addTechnicalWindow({ id: technicalWindows.length }))
                }
              >
                Добавить технологическое окно
              </Button>
            </Form>
          )}
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
};

const WrappedSetting = Form.create()(Setting);
export default WrappedSetting;
