import React from "react";
import { useDispatch, useSelector } from "react-redux";
import structureActions from "@iso/redux/structure/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
} from "./Structure.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
} = structureActions;

export default function Structure() {
  const {
    structures,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    structure,
    filters,
  } = useSelector((state) => state.Structure);

  const { greenhouses } = useSelector((state) => state.App);

  const {
    blocksPermissions: { structure: structurePermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = structurePermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchStructureName: "",
    filterStructureNameDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchStructureName } = state;

    setState({
      ...state,
      filterStructureNameDropdownVisible: false,
    });

    const newFilters = _.cloneDeep(filters);
    newFilters.name = searchStructureName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(structures).map((structure, index) => {
    return dataSource.push({
      ...structures[structure],
      key: structure,
    });
  });

  const handleRecord = (actionName, structure) => {
    dispatch(saveIntoAgroInvest(structure, actionName));
  };

  const handleModal = (structure = null) => {
    dispatch(toggleModal(structure));
  };

  const structureUpdate = (key, value) => {
    if (key) structure[key] = value;
    dispatch(update(structure));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.name = filters.name;

    dispatch(setFilters(newFilters, pagination.current));
  };

  const columns = [
    {
      title: "КОД",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchStructureName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterStructureNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterStructureNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Название",
      dataIndex: "description",
      key: "description",
      sorter: (a, b) => {
        if (a.description < b.description) return -1;
        if (a.description > b.description) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.description;
      },
    },
    {
      title: "Теплица",
      dataIndex: "rawGreenHouseId",
      key: "rawGreenHouseId",
      sorter: (a, b) => {
        if (a.rawGreenHouseId < b.rawGreenHouseId) return -1;
        if (a.rawGreenHouseId > b.rawGreenHouseId) return 1;
        return 0;
      },
      render: (text, row) => {
        return _.get(
          greenhouses,
          "[" + _.random(0, 10) + "].description",
          "n/a"
        );
      },
    },
    {
      title: "Площадь",
      dataIndex: "landingArea",
      key: "landingArea",
      sorter: (a, b) => {
        if (a.landingArea < b.landingArea) return -1;
        if (a.landingArea > b.landingArea) return 1;
        return 0;
      },
      render: (text, row) => {
        return _.get(row, "landingArea", "n/a");
      },
    },
    {
      title: "Топология",
      dataIndex: "regularExpr",
      key: "regularExpr",
      sorter: (a, b) => {
        if (a.regularExpr < b.regularExpr) return -1;
        if (a.regularExpr > b.regularExpr) return 1;
        return 0;
      },
      render: (text, row) => {
        return _.truncate(_.get(row, "regularExpr", "n/a"));
      },
    },
    {
      title: "Изменить",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            <a
              onClick={() => handleModal(row)}
              disabled={!hasAccessToEdit}
              href="# "
            >
              <i className="ion-android-create" />
            </a>
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Посевные площади <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="plus"
                onClick={() => dispatch(toggleModal(null))}
                style={{ marginLeft: "10px" }}
                disabled={!hasAccessToEdit}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                {structure.id
                  ? "Изменить параметр " + structure.name
                  : "Добавить"}
                <br />
                <small>{structure.id ? structure.id : "n/a"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => {
              if (!hasAccessToEdit) return;
              handleRecord("update", structure);
            }}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
          >
            <Form>
              <Fieldset>
                <Label>Основные свойства</Label>
                <FormItem label="Название" hasFeedback>
                  <Input
                    placeholder="Имя объекта"
                    value={structure.name}
                    onChange={({ target: { value } }) =>
                      structureUpdate("name", value)
                    }
                  />
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
