import React, { useState, useRef, useEffect } from "react";
import _ from "lodash";
import TableWrapper from "../AntTables.styles";
import { Input, Form } from "antd";

const EditableCell = ({
  titleColumn,
  editable,
  checkEditable,
  children,
  dataIndex,
  record,
  handleSave,
  validator = (rule, value, callback) => {
    callback();
  },
  normalize = (fn) => fn,
  form,
  ...restProps
}) => {
  const hasEditableAccess = checkEditable ? checkEditable(record) : true;

  const [editing, setEditing] = useState(false);
  const initialValue = _.get(record, dataIndex);
  const inputRef = useRef();
  const nameCell = `${titleColumn}${dataIndex}`.split(".").join("");
  const {
    getFieldDecorator,
    setFieldsValue,
    getFieldValue,
    validateFields,
  } = form;

  useEffect(() => {
    if (editing) {
      inputRef.current.focus();
      inputRef.current.value = children[2];
    }
  }, [editing, children]);

  const toggleEdit = () => {
    setEditing(!editing);
  };

  const save = async (e) => {
    toggleEdit();
    validateFields((err, values) => {
      if (!err) {
        try {
          _.set(record, dataIndex, normalize(getFieldValue(nameCell)));
          const itemKey = dataIndex.split(".").slice(0, 1).join(".");
          const changedItem = _.get(record, itemKey);
          handleSave({ changedItem, changedRecord: record });
        } catch (errInfo) {
          console.log("Save failed:", errInfo);
        }
      }
    });
  };

  let childNode = children;

  if (editable && hasEditableAccess) {
    const content = [...children];
    content[2] = content[2] || "---";

    childNode = editing ? (
      <Form.Item
        style={{
          margin: 0,
        }}
        name={dataIndex}
      >
        {getFieldDecorator(`${nameCell}`, {
          initialValue,
          validateTrigger: ["onChange", "onBlur"],
          rules: [
            {
              validator,
            },
          ],
        })(
          <Input
            ref={inputRef}
            onPressEnter={save}
            onBlur={save}
            onChange={(e) => setFieldsValue({ [nameCell]: e.target.value })}
          />
        )}
      </Form.Item>
    ) : (
      <div
        className="editable-cell-value-wrap"
        style={{
          paddingRight: 24,
        }}
        onClick={toggleEdit}
      >
        {content}
      </div>
    );
  }

  return <td {...restProps}>{childNode}</td>;
};

const EditView = (props) => {
  const [dataSource, setDataSource] = useState(null);

  useEffect(() => {
    setDataSource(props.dataSource);
  }, [props.dataSource]);

  const components = {
    body: {
      cell: Form.create()(EditableCell),
    },
  };

  const handleSave = ({ changedItem, changedRecord }) => {
    const newData = [...dataSource];

    const rowIndex = _.findIndex(
      newData,
      (record) => _.get(record, "key") === _.get(changedRecord, "key")
    );

    const row = newData[rowIndex];

    const itemIndex = _.findKey(row, (item) => {
      return item.key === changedItem.key;
    });

    newData[rowIndex][itemIndex] = changedItem;

    setDataSource(newData);
    props.handleSave(changedItem, changedRecord);
  };

  const columns = props.columns.map((col) => {
    if (!col.editable) {
      return col;
    }

    return {
      ...col,
      onCell: (record) => ({
        record,
        editable: col.editable,
        checkEditable: col.checkEditable,
        dataIndex: col.dataIndex,
        titleColumn: col.title,
        handleSave,
        validator: props.validator,
        normalize: props.normalize,
      }),
    };
  });

  return (
    <TableWrapper
      {...props}
      components={components}
      dataSource={dataSource}
      columns={columns}
    />
  );
};

export default EditView;
