import React, { useEffect, useState, useRef, useMemo } from "react";
import { useDispatch, useSelector } from "react-redux";
import CsvDownloader from "react-csv-downloader";
import { Link, useLocation } from "react-router-dom";
import { request } from "@iso/lib/helpers/utility";
import isTimeBetween from "@iso/lib/helpers/isTimeBetween";
import _ from "lodash";

import { Icon } from "antd";
import taskActions from "@iso/redux/tasks/actions";
import workerActions from "@iso/redux/workers/actions";
import { defaultFilters } from "@iso/redux/tasks/reducers";
import Button from "@iso/components/uielements/button";
import LayoutContentWrapperScrollable from "@iso/components/utility/layoutWrapperScrollable";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { quarantineRows, trolleyId } from "@iso/config/sections.config";
import { IconSvg } from "@iso/components/ScrumBoard/IconSvg/IconSvg";
import ModalTask from "@iso/components/ModalTask/ModalTask";
import TaskWorkerSelector from "./TaskWorkersSelector";
import TransferWithinAstationBlack from "@iso/assets/images/icon/transfer_within_a_station-black-18dp.svg";

import {
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
  StatusTag,
} from "./Task.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const {
  loadFromAgroInvest,
  loadLastFinishedTaskFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
  sync1C,
  getSectionsGreenhouse,
  assignTaskToWorker,
  prepareDownload,
} = taskActions;

const { loadAutocomplete } = workerActions;

export default function Tasks() {
  const {
    tasks,
    isLoading,
    isLoadingLastFinishedTask,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    task,
    filters,
    greenhouseSections,
    workers,
    csvColumns,
    csvRows,
  } = useSelector((state) => state.Tasks);

  // const { autocomplete } = useSelector(
  //   state => state.Workers
  // );
  
  const {
    allowedGreenhouses,
    greenhouseId,
    greenhouses,
    appSettings,
  } = useSelector((state) => state.App);

  const [technicalWindowRange, setTechnicalWindow] = useState(_.get(appSettings, "workingTimeConfig", {}))

  const { role } = useSelector((state) => state.Auth);

  const {
    blocksPermissions: { tasks: tasksPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const {
    ["light-edit"]: {
      edit: hasAccessToTasksLightEdit,
      sectionsEdit: accessSectionsForTasksLightEdit,
    },
    ["hard-edit"]: { edit: hasAccessToTasksHardEdit },
    ["task-logs"]: { read: hasAccessToTasksLogsRead },
  } = tasksPermissions.blocks;

  const [currentRowSelected, setCurrentRowSelected] = useState(null);

  const dispatch = useDispatch();

  const { state: locationState } = useLocation();
  useEffect(() => {
    if (locationState?.sectionId) {
      const { sectionId, selectedDay } = locationState;
      const unionFilters = {
        ...defaultFilters,
        sectionId: [sectionId],
        startAt: [
          {
            before: moment().day(selectedDay),
            after: moment().day(selectedDay - 1),
          },
        ],
      };
      dispatch(setFilters(unionFilters));
    } else {
      dispatch(loadFromAgroInvest());
    }
    dispatch(loadAutocomplete());
    dispatch(getSectionsGreenhouse());
  }, [dispatch, greenhouseId, locationState]);

  const downloadButtonRef = useRef(null);

  useEffect(() => {
    if (csvColumns && csvRows) {
      downloadButtonRef.current.handleClick();
    }
  }, [csvColumns, csvRows]);

  const [state, setState] = React.useState({
    searchRowNumber: "",
    filterRowNumberDropdownVisible: false,
    searchWorkerName: "",
    filterWorkerNameDropdownVisible: false,
    searchWorkerCode: "",
    filterWorkerCodeDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchRowNumber, searchWorkerName, searchWorkerCode } = state;

    setState({
      ...state,
      filterRowNumberDropdownVisible: false,
      filterWorkerNameDropdownVisible: false,
      filterWorkerCodeDropdownVisible: false,
    });

    const newFilters = _.cloneDeep(filters);
    newFilters.rowNumber = searchRowNumber;
    newFilters["worker.searchable"] = searchWorkerName;
    newFilters["worker.code"] = searchWorkerCode;

    dispatch(setFilters(newFilters, 1));
  };

  const dataSource = [];
  Object.keys(tasks).map((task, index) => {
    return dataSource.push({
      ...tasks[task],
      key: task,
    });
  });

  const onOkModal = () => {
    dispatch(saveIntoAgroInvest(task, "update"));
  };

  const handleModal = (task = null) => {
    dispatch(toggleModal(task));
  };

  const onChangeTask = (key, value) => {
    if (key) task[key] = value;
    dispatch(update(task));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const loadLastFinishedTask = (sectionId, rowNumber) => {
    dispatch(loadLastFinishedTaskFromAgroInvest(sectionId, rowNumber));
  };

  const callSync1C = (syncId, isPatch) => {
    dispatch(sync1C(syncId, isPatch));
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const downloadTasks = () => {
    dispatch(prepareDownload());
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.rowNumber = filters.rowNumber;
    newFilters["worker.searchable"] = filters["worker.searchable"];
    newFilters["worker.code"] = filters["worker.code"];

    dispatch(setFilters(newFilters, pagination.current));
  };

  const handleWorkerSelect = (id, name) => {
    onChangeTask("workerId", id ? id : null);
    onChangeTask("workerName", name ? name : null);
  };

  const onModalTask = () => {
    dispatch(toggleModal(null));
  };
  const handleRangeSelect = (value) => {
    if (_.isEmpty(value)) {
      task.startedAt = null;
      task.finishedAt = null;
    } else {
      task.startedAt = value[0].format();
      task.finishedAt = value[1].format();
    }

    dispatch(update(task));
  };

  const onChangeProperties = (id, value) => {
    const p = _.find(task.properties, { id });

    if (p) {
      p.value = value;

      dispatch(update(task));
    }
  };

  const columnsSizes = {
    name: 130,
    row: 100,
    worker: 170,
    section: 150,
    planAt: 180,
    workAt: 220,
    status: 120,
    plan: 100,
    syncStatus: 210,
    syncAction: 120,
    actions: 90,
  };

  const sumWidthAllColumns = _.sum(_.values(columnsSizes));
  const boxWrapper = document.querySelector(".isoBoxWrapper");
  const boxHeight = boxWrapper ? boxWrapper.clientHeight : 0;

  const [isTaskLoading, setIsTaskLoading] = useState(false);
  useEffect(() => {
    const fetchTechnicalWindowSections = async () => {
      setIsTaskLoading(true);
      const currentTask = task.id ? task : currentRowSelected;
      if (currentTask?.id) {
        const response = await request('technical_window_sections', {
          sectionId: currentTask.sectionId,
          'technicalWindow.greenhouseId': currentTask.greenhouseId,
          page: 1,
        });

        const technicalWindowRange =
          response['hydra:member'].length === 0
            ?   _.get(appSettings, "workingTimeConfig", {})
            : {
                start: moment.utc(response['hydra:member'][0].technicalWindow.start).format('HH:mm'),
                finish: moment.utc(response['hydra:member'][0].technicalWindow.finish).format('HH:mm'),
              };
        setTechnicalWindow(technicalWindowRange);
        setIsTaskLoading(false);
      }
    };

    fetchTechnicalWindowSections();
  }, [appSettings, task.greenhouseId, task.id, task.sectionId, currentRowSelected]);

  const isWorkingTime = () => {
    const { start, finish } = technicalWindowRange;
    if (!start || !finish) return false;
    const currentTime = moment().utc().format("HH:mm");
    return isTimeBetween(start, finish, currentTime);
  };

  const hasAccessToEditWorker =
    isWorkingTime() || _.includes(["superuser", "tech-support"], role);

  const generateActionButtons = (row) => {
    const allowedGreenhouseIds = _.map(allowedGreenhouses, (g) => g.id);
    const isAttachedUser = _.includes(allowedGreenhouseIds, row.greenhouseId);

    const sectionModule = _.get(row, "section.module");
    const hasAccessToEdit =
      hasAccessToTasksLightEdit &&
      (!accessSectionsForTasksLightEdit ||
        (accessSectionsForTasksLightEdit &&
          _.includes(accessSectionsForTasksLightEdit, sectionModule)));

    const style =
      isAttachedUser && hasAccessToEdit
        ? {}
        : {
            opacity: "0.3",
          };

    return (
      <ActionWrapper>
        <div>
          <a
            onClick={() => handleModal(row)}
            disabled={!isAttachedUser || !hasAccessToEdit}
            href="# "
          >
            <i className="ion-android-create" style={{ ...style }} />
          </a>
          {hasAccessToTasksLogsRead ? (
            <Link to={`/dashboard/audit/${row.id}`}>
              <Icon type="history" />
            </Link>
          ) : (
            <Link to="#" disabled style={{ opacity: 0.3 }}>
              <Icon type="history" />
            </Link>
          )}
        </div>
      </ActionWrapper>
    );
  };

  const handleAssignTaskToWorker = (taskId, workerId) => {
    const checkedWorkerId = workerId === "NOT_ASSIGNED" ? null : workerId;

    setCurrentRowSelected(null);
    dispatch(assignTaskToWorker(taskId, checkedWorkerId));
  };

  const dateFilters = [
    {
      text: "Понедельник",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(1).endOf("day").format(),
        after: moment().utc().isoWeekday(1).startOf("day").format(),
      }),
    },
    {
      text: "Вторник",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(2).endOf("day").format(),
        after: moment().utc().isoWeekday(2).startOf("day").format(),
      }),
    },
    {
      text: "Среда",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(3).endOf("day").format(),
        after: moment().utc().isoWeekday(3).startOf("day").format(),
      }),
    },
    {
      text: "Четверг",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(4).endOf("day").format(),
        after: moment().utc().isoWeekday(4).startOf("day").format(),
      }),
    },
    {
      text: "Пятница",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(5).endOf("day").format(),
        after: moment().utc().isoWeekday(5).startOf("day").format(),
      }),
    },
    {
      text: "Суббота",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(6).endOf("day").format(),
        after: moment().utc().isoWeekday(6).startOf("day").format(),
      }),
    },
    {
      text: "Воскресенье",
      value: JSON.stringify({
        before: moment().utc().isoWeekday(7).endOf("day").format(),
        after: moment().utc().isoWeekday(7).startOf("day").format(),
      }),
    },
    {
      text: "Эта неделя",
      value: JSON.stringify({
        before: moment().utc().endOf("week").format(),
        after: moment().utc().startOf("week").format(),
      }),
    },
    {
      text: "Слeдующая неделя",
      value: JSON.stringify({
        before: moment().utc().add(1, "weeks").endOf("week").format(),
        after: moment().utc().add(1, "weeks").startOf("week").format(),
      }),
    },
    {
      text: "Прошлая неделя",
      value: JSON.stringify({
        before: moment().utc().subtract(1, "weeks").endOf("week").format(),
        after: moment().utc().subtract(1, "weeks").startOf("week").format(),
      }),
    },
  ];

  if (
    moment().utc().endOf("day").format() !==
    moment().utc().isoWeekday(1).endOf("day").format()
  ) {
    // Today is monday
    dateFilters.unshift({
      text: "По сегодняшний день",
      value: JSON.stringify({
        before: moment().utc().endOf("day").format(),
        after: moment().utc().isoWeekday(1).startOf("day").format(),
      }),
    });
  }

  const columns = [
    {
      title: "Название",
      dataIndex: "name",
      key: "pass",
      width: columnsSizes.name,
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filters: [
        {
          text: "Проход 1",
          value: 0,
        },
        {
          text: "Проход 2",
          value: 1,
        },
        {
          text: "Проход 3",
          value: 2,
        },
        {
          text: "Проход 4",
          value: 3,
        },
        {
          text: "Проход 5",
          value: 4,
        },
        {
          text: "Проход 6",
          value: 5,
        },
        {
          text: "Проход 7",
          value: 6,
        },
        {
          text: "Проход 8",
          value: 7,
        },
        {
          text: "Проход 9",
          value: 8,
        },
        {
          text: "Проход 10",
          value: 9,
        },
        {
          text: "Проход 11",
          value: 10,
        },
        {
          text: "Проход 12",
          value: 11,
        },
        {
          text: "Проход 13",
          value: 12,
        },
        {
          text: "Проход 14",
          value: 13,
        },
        {
          text: "Проход 15",
          value: 14,
        },
        {
          text: "Проход 16",
          value: 15,
        },
      ],
      filteredValue: filters["pass"] || [],
      filterMultiple: false,
      render: (text, row) => {
        const trimByWord = (sentence) => {
          let result = sentence;
          let resultArray = result.split(" ");
          if (resultArray.length > 7) {
            resultArray = resultArray.slice(0, 7);
            result = resultArray.join(" ") + "...";
          }
          return result;
        };

        const pass = _.get(row, "pass", 0);

        return (
          <>
            {trimByWord(row.name)}
            <br />
            {_.get(row, "greenhouse.description", "n/a")}
            <div>
              <small>Проход:&nbsp;{pass + 1}</small>
            </div>
          </>
        );
      },
    },
    {
      title: "Ряд",
      dataIndex: "rowNumber",
      key: "rowNumber",
      width: columnsSizes.row,
      align: "center",
      sorter: (a, b) => {
        if (a.rowNumber < b.rowNumber) return -1;
        if (a.rowNumber > b.rowNumber) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchRowNumber", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.rowNumber ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterRowNumberDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterRowNumberDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return (
          <>
            {_.includes(quarantineRows, row.rowNumber) ? (
              <StatusTag key="rowNumber" className="rowNumberQuarantine">
                Ряд: {row.rowNumber}
                &nbsp;
                <Icon type="warning" />
              </StatusTag>
            ) : (
              <StatusTag key="rowNumber" className="rowNumber">
                Ряд: {row.rowNumber}
              </StatusTag>
            )}
            <br />
            <StatusTag
              key="exclusive"
              className={"rowNumberExclusive-" + row.exclusive}
            >
              Группа: {row.exclusive}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Работник",
      dataIndex: "workerName",
      key: "worker.searchable",
      width: columnsSizes.worker,
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchWorkerName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters["worker.searchable"] ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterWorkerNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterWorkerNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      sorter: (a, b) => {
        if (a.workerName < b.workerName) return -1;
        if (a.workerName > b.workerName) return 1;
        return 0;
      },
      render: (text, row) => {
        const trimByWord = (sentence) => {
          let result = sentence;
          let resultArray = result.split(" ");
          if (resultArray.length > 20) {
            resultArray = resultArray.slice(0, 20);
            result = resultArray.join(" ") + "...";
          }
          return result;
        };
        const code = _.get(row, "worker.code", null);
        const isAccessForUpdate =
          row.isFinished === false &&
          row.inProgress === false &&
          !row.startedAt;

        const workerNameBlock = () => {
          return isAccessForUpdate ? (
            <div
              onClick={() => setCurrentRowSelected(row)}
              style={{ cursor: "pointer" }}
            >
              <span>
                {trimByWord(_.get(row, "workerName", ""))} &nbsp;
                {!!_.size(code) && `/ ${code}`}
              </span>
            </div>
          ) : (
            <span>
              {trimByWord(_.get(row, "workerName", ""))} &nbsp;
              {!!_.size(code) && `/ ${code}`}
            </span>
          );
        };

        const sectionModule = _.get(row, "section.module");
        const hasAccessToEdit =
          hasAccessToTasksLightEdit &&
          (!accessSectionsForTasksLightEdit ||
            (accessSectionsForTasksLightEdit &&
              _.includes(accessSectionsForTasksLightEdit, sectionModule)));

        return row.id === currentRowSelected?.id &&
          hasAccessToEditWorker && !isTaskLoading ? (
          <TaskWorkerSelector
            greenhouseId={greenhouseId}
            greenhouses={greenhouses}
            taskId={row.id}
            handleAssignTaskToWorker={handleAssignTaskToWorker}
            setCurrentRowSelected={setCurrentRowSelected}
          />
        ) : row.workerId ? (
          <>
            {workerNameBlock()}
            {moment(row.lockedTill) > moment() && row.workerName ? (
              <div>
                <Icon type="star" className="warning" />
                &nbsp;
                <small>
                  Закреплено до {moment(row.lockedTill).format("LLLL")}
                </small>
              </div>
            ) : null}
            <br />
            {_.map(row.properties, (p) => {
              return p.parentId === trolleyId && p.value ? (
                <StatusTag key={p.id} className="trolleyNumber">
                  {p.value}
                  &nbsp;
                  <Icon type="shopping-cart" />
                </StatusTag>
              ) : null;
            })}
            <br />
            <small>{_.get(row, "worker.notes", "")}</small>
          </>
        ) : hasAccessToEdit ? (
          <span
            onClick={() => setCurrentRowSelected(row)}
            style={{ cursor: "pointer", color: "blue" }}
          >
            Назначить работника
          </span>
        ) : (
          "Не назначено"
        );
      },
    },
    {
      title: "Секция",
      dataIndex: "section",
      key: "sectionId",
      width: columnsSizes.section,
      filters: _.map(_.sortBy(greenhouseSections, ["name"]), (s) => {
        return {
          text: s.name,
          value: s.sectionId,
        };
      }),
      filteredValue: filters["sectionId"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (_.get(a, "section.name", "") < _.get(b, "section.name", ""))
          return -1;
        if (_.get(a, "section.name", "") > _.get(b, "section.name", ""))
          return 1;
        return 0;
      },
      render: (text, row) => {
        const trimByWord = (sentence) => {
          let result = sentence;
          let resultArray = result.split(" ");
          if (resultArray.length > 8) {
            resultArray = resultArray.slice(0, 8);
            result = resultArray.join(" ") + "...";
          }
          return result;
        };

        return (
          <>
            <div>{trimByWord(_.get(row, "section.name", ""))}</div>
            <div>
              <small>Рейтинг:&nbsp;{row.rating}</small>
            </div>
            {row.isQuality && row.qualityTask && (
              <small>
                <br />
                <div>Проверяемая задача:</div>
                <div>{_.get(row, "qualityTask.sectionName")}</div>
                <div>{_.get(row, "qualityTask.workerName")}</div>
                <div>Завершена:</div>
                <div>
                  {_.get(row, "qualityTask.finishedAt")
                    ? moment(_.get(row, "qualityTask.finishedAt")).format(
                        "LLLL"
                      )
                    : null}
                </div>
              </small>
            )}
          </>
        );
      },
    },
    {
      title: "Запланировано на",
      dataIndex: "startAt",
      className: "noWrapCell",
      key: "startAt",
      width: columnsSizes.planAt,
      align: "center",
      filters: dateFilters,
      filteredValue: filters["startAt"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (moment(a.startAt) < moment(b.startAt)) return -1;
        if (moment(a.startAt) > moment(b.startAt)) return 1;
        return 0;
      },

      render: (text, row) => {
        return row.startAt ? moment(row.startAt).format("LL") : "N/A";
      },
    },
    {
      title: "Дата начала и окончания",
      dataIndex: "finishedAt",
      className: "noWrapCell",
      key: "finishedAt",
      width: columnsSizes.workAt,
      filters: [
        {
          text: "Понедельник",
          value: JSON.stringify({
            before: moment().utc().day(1).endOf("day").format(),
            after: moment().utc().day(1).startOf("day").format(),
          }),
        },
        {
          text: "Вторник",
          value: JSON.stringify({
            before: moment().utc().day(2).endOf("day").format(),
            after: moment().utc().day(2).startOf("day").format(),
          }),
        },
        {
          text: "Среда",
          value: JSON.stringify({
            before: moment().utc().day(3).endOf("day").format(),
            after: moment().utc().day(3).startOf("day").format(),
          }),
        },
        {
          text: "Четверг",
          value: JSON.stringify({
            before: moment().utc().day(4).endOf("day").format(),
            after: moment().utc().day(4).startOf("day").format(),
          }),
        },
        {
          text: "Пятница",
          value: JSON.stringify({
            before: moment().utc().day(5).endOf("day").format(),
            after: moment().utc().day(5).startOf("day").format(),
          }),
        },
        {
          text: "Суббота",
          value: JSON.stringify({
            before: moment().utc().day(6).endOf("day").format(),
            after: moment().utc().day(6).startOf("day").format(),
          }),
        },
        {
          text: "Воскресенье",
          value: JSON.stringify({
            before: moment().utc().day(7).endOf("day").format(),
            after: moment().utc().day(7).startOf("day").format(),
          }),
        },
        {
          text: "Эта неделя",
          value: JSON.stringify({
            before: moment().utc().endOf("week").format(),
            after: moment().utc().startOf("week").format(),
          }),
        },
        {
          text: "Слeдующая неделя",
          value: JSON.stringify({
            before: moment().utc().add(1, "weeks").endOf("week").format(),
            after: moment().utc().add(1, "weeks").startOf("week").format(),
          }),
        },
        {
          text: "Прошлая неделя",
          value: JSON.stringify({
            before: moment().utc().subtract(1, "weeks").endOf("week").format(),
            after: moment().utc().subtract(1, "weeks").startOf("week").format(),
          }),
        },
        {
          text: "Не начата",
          value: "N/A",
        },
      ],
      filteredValue: filters["finishedAt"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (moment(a.finishedAt) < moment(b.finishedAt)) return -1;
        if (moment(a.finishedAt) > moment(b.finishedAt)) return 1;
        return 0;
      },
      render: (text, row) => {
        const startedAt = row.startedAt
          ? moment(row.startedAt).calendar()
          : "N/A";
        const finishedAt = row.finishedAt
          ? moment(row.finishedAt).calendar()
          : "N/A";

        return (
          <div style={{ textAlign: "center" }}>
            <div>{startedAt}</div>
            <div>-</div>
            <div>{finishedAt}</div>
          </div>
        );
      },
    },
    {
      title: "Статус",
      dataIndex: "isFinished",
      className: "noWrapCell",
      key: "isFinished",
      width: columnsSizes.status,
      align: "center",
      filters: [
        {
          text: "Завершена",
          value: true,
        },
        {
          text: "Не завершена",
          value: false,
        },
      ],
      filteredValue: filters["isFinished"] || [],
      filterMultiple: false,
      render: (text, row) => {
        const inProgress = _.get(row, "inProgress", false);
        const isFinished = _.get(row, "isFinished");

        return (
          <>
            <div>
              <StatusTag className={isFinished ? "finished" : "notFinished"}>
                {isFinished ? "Завершена" : "Не завершена"}
              </StatusTag>
            </div>
            <div>
              {inProgress && (
                <div>
                  <IconSvg
                    src={TransferWithinAstationBlack}
                    border="none"
                    padding={"0"}
                    alt="TransferWithinAstationBlack Icon"
                  />
                </div>
              )}
            </div>
          </>
        );
      },
    },
    {
      title: "План",
      dataIndex: "isActive",
      className: "noWrapCell",
      key: "isActive",
      width: columnsSizes.plan,
      align: "center",
      filters: [
        {
          text: "В плане",
          value: true,
        },
        {
          text: "Исключена",
          value: false,
        },
      ],
      filteredValue: filters["isActive"] || [],
      filterMultiple: false,
      render: (text, row) => {
        const isActive = _.get(row, "isActive", false);

        return (
          <>
            <div>
              <StatusTag className={isActive ? "finished" : "statusRed"}>
                {isActive ? "В плане" : "Исключена"}
              </StatusTag>
            </div>
          </>
        );
      },
    },
    {
      title: "Синхронизация",
      dataIndex: "syncedAt",
      className: "noWrapCell",
      key: "exists",
      width: columnsSizes.syncStatus,
      filters: [
        {
          text: "Ошибка Данных",
          value: JSON.stringify({
            status: true,
          }),
        },
        {
          text: "Без ошибок данных",
          value: JSON.stringify({
            status: false,
          }),
        },
      ],
      filteredValue: filters["exists"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        const aSyncedAt = _.get(a, "sync.syncedAt", null);
        const bSyncedAt = _.get(b, "sync.syncedAt", null);

        if (moment(aSyncedAt) < moment(bSyncedAt)) return -1;
        if (moment(aSyncedAt) > moment(bSyncedAt)) return 1;

        return 0;
      },

      render: (text, row) => {
        const syncedAt = _.get(row, "sync.syncedAt", null);
        const syncStatus = _.get(row, "sync.status", null);

        const isFinished = _.get(row, "isFinished");
        const startedAt = _.get(row, "startedAt", null);
        const finishedAt = _.get(row, "finishedAt", null);

        let statusText;
        let statusTextClassName;
        if (_.isNull(startedAt) && _.isNull(finishedAt)) {
          statusText = "Запланировано";
          statusTextClassName = "nfcOn";
        } else if (!_.isNull(startedAt) && _.isNull(finishedAt)) {
          statusText = "В работе";
          statusTextClassName = "nfcOff";
        } else if (!_.isNull(startedAt) && !_.isNull(finishedAt)) {
          statusText = "ПАУЗА";
          statusTextClassName = "statusRed";
        }

        return (
          <>
            Статус МУП:{" "}
            {row.status ? (
              <StatusTag className="statusRed">{row.status}</StatusTag>
            ) : (
              (!isFinished && (
                <StatusTag className={statusTextClassName}>
                  {statusText}
                </StatusTag>
              )) || <StatusTag className="finished">{200}</StatusTag>
            )}
            <br />
            Время: <b>{syncedAt ? moment(syncedAt).calendar() : "N/A"}</b>
            <br />
            ID: <b>{_.get(row, "sync.id", "N/A")}</b>
            <br />
            1C: <b>{_.get(row, "sync.name", "N/A")}</b> (
            {_.get(row, "sync.toSend", false) ? "V" : "X"})
            <br />
            Статус из 1С:{" "}
            <StatusTag
              className={
                syncStatus === "200" || syncStatus === "201"
                  ? "finished"
                  : "statusRed"
              }
            >
              {syncStatus ? syncStatus : "N/A"}
            </StatusTag>
          </>
        );
      },
    },
    {
      title: "Синхр-вать",
      dataIndex: "toSend",
      className: "noWrapCell",
      key: "toSend",
      width: columnsSizes.syncAction,
      align: "center",
      sorter: (a, b) => {
        const aToSend = _.get(a, "sync.toSend", null);
        const bToSend = _.get(b, "sync.toSend", null);

        if (moment(aToSend) < moment(bToSend)) return -1;
        if (moment(aToSend) > moment(bToSend)) return 1;

        return 0;
      },

      render: (text, row) => {
        return (
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="reload"
            onClick={() => callSync1C(row.id, !!row.sync)}
            disabled={!(row.sync || row.isFinished)}
          />
        );
      },
    },
    {
      title: "Действия",
      key: "action",
      align: "center",
      width: columnsSizes.actions,
      className: "noWrapCell",
      render: (text, row) => {
        return generateActionButtons(row);
      },
    },
  ];

  const csvDownloader = (
    <CsvDownloader
      filename={`Задачи-${moment(new Date()).format("MMMM-Do-YYYY-h:mm:ss")}`}
      separator=";"
      wrapColumnChar={'"'}
      columns={csvColumns}
      datas={csvRows}
      style={{ display: "none" }}
      ref={downloadButtonRef}
    />
  );

  return (
    <LayoutContentWrapperScrollable>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Задачи <small>всего найдено: {totalItems}</small>
            </ComponentTitle>
            <div>
              {csvDownloader}
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="cloud-download"
                onClick={downloadTasks}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>
          <ModalTask
            task={task}
            hasAccessToLightEdit={hasAccessToTasksLightEdit}
            hasAccessToHardEdit={hasAccessToTasksHardEdit}
            isWorkingTime={isWorkingTime()}
            isModalTask={modalActive}
            onOkModal={onOkModal}
            onSelectQualitySection={loadLastFinishedTask}
            isLoadingLastFinishedTask={isLoadingLastFinishedTask}
            greenhouseSections={greenhouseSections}
            onModalTask={onModalTask}
            onChangeTask={onChangeTask}
            handleRangeSelect={handleRangeSelect}
            onChangeProperties={onChangeProperties}
            handleWorkerSelect={handleWorkerSelect}
            hasAccessToEditWorker={hasAccessToEditWorker}
          />
          <TableWrapper
            style={{ minHeight: "100vh" }}
            rowKey="key"
            columns={columns}
            scroll={{ y: boxHeight - 200, x: sumWidthAllColumns }}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapperScrollable>
  );
}
