import React from "react";
import { useDispatch, useSelector } from "react-redux";
import { Link } from "react-router-dom";
import trolleyActions from "@iso/redux/trolleys/actions";
import workerActions from "@iso/redux/workers/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import DatePicker from "@iso/components/uielements/datePicker";

import AutoComplete, {
  AutoCompleteOption,
} from "@iso/components/uielements/autocomplete";

import locale from "antd/lib/locale-provider/ru_RU";

import { ConfigProvider } from "antd";

import {
  Fieldset,
  Label,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
  StatusTag,
} from "./Trolley.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
  sync1C,
  toggleDisabled,
} = trolleyActions;

const { loadAutocomplete, setAutocomplete } = workerActions;

export default function Trolleys() {
  const {
    trolleys,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    trolley,
    filters,
    isDisabled,
  } = useSelector((state) => state.Trolleys);

  const { autocomplete } = useSelector((state) => state.Workers);

  const {
    blocksPermissions: { ["trolleys"]: trolleysPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = trolleysPermissions;

  const dispatch = useDispatch();

  const [state, setState] = React.useState({
    searchTrolleyNumber: "",
    filterTrolleyNumberDropdownVisible: false,
    searchWorkerName: "",
    filterWorkerNameDropdownVisible: false,
  });

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = () => {
    let { searchTrolleyNumber, searchWorkerName } = state;

    const newFilters = _.cloneDeep(filters);
    newFilters.trolley = searchTrolleyNumber;
    newFilters["worker.searchable"] = searchWorkerName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
    dispatch(loadAutocomplete());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(trolleys).map((trolley, index) => {
    return dataSource.push({
      ...trolleys[trolley],
      key: trolley,
    });
  });

  const handleRecord = (actionName, trolley) => {
    dispatch(saveIntoAgroInvest(trolley, actionName));
  };

  const handleModal = (trolley = null) => {
    dispatch(toggleModal(trolley));
  };

  const callToggleDisabled = () => {
    dispatch(toggleDisabled());
  };

  const trolleyUpdate = (key, value) => {
    if (key) trolley[key] = value;
    dispatch(update(trolley));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const callSync1C = (syncId) => {
    dispatch(sync1C(syncId));
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters.trolley = filters.trolley;
    newFilters["worker.searchable"] = filters["worker.searchable"];

    dispatch(setFilters(newFilters, pagination.current));
  };

  const handleWorkerSearch = (value) => {
    dispatch(setAutocomplete(value));
  };

  const handleWorkerSelect = (value) => {
    trolleyUpdate("workerId", value ? value : null);
  };

  const handleDateSelect = (value) => {
    trolley.registeredAt = value ? value.format() : null;

    dispatch(update(trolley));
  };

  // const propertiesUpdate = (id, value) => {
  //   const p = _.find(trolley.properties, { id });

  //   if(p) {
  //     p.value = value;

  //     dispatch(update(trolley));
  //   }
  // }

  const handleWorkerSearchThrottled = _.throttle(handleWorkerSearch, 2000, {
    leading: false,
  });

  const columns = [
    {
      title: "Тележка",
      dataIndex: "trolley",
      key: "trolley",
      sorter: (a, b) => {
        if (a.trolley < b.trolley) return -1;
        if (a.trolley > b.trolley) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchTrolleyNumber", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.trolley ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterTrolleyNumberDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterTrolleyNumberDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return (
          <StatusTag key="trolleyNumber" className="trolleyNumber">
            {row.trolley}
          </StatusTag>
        );
      },
    },
    {
      title: "Тип",
      dataIndex: "type",
      className: "noWrapCell",
      key: "type",
      filters: [
        {
          text: "Взятие",
          value: "start",
        },
        {
          text: "Отвязка",
          value: "finish",
        },
      ],
      filteredValue: filters["type"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (a.type < b.type) return -1;
        if (a.type > b.type) return 1;
        return 0;
      },

      render: (text, row) => {
        return (
          <StatusTag className={row.type}>
            {row.type === "start" ? "Взятие" : "Отвязка"}
          </StatusTag>
        );
      },
    },
    {
      title: "Сотрудник",
      dataIndex: "workerName",
      key: "worker.searchable",
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchWorkerName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.workerId ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterWorkerNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterWorkerNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      sorter: (a, b) => {
        if (a.workerName < b.workerName) return -1;
        if (a.workerName > b.workerName) return 1;
        return 0;
      },
      render: (text, row) => {
        return (
          <>
            {_.get(row, "workerName", "")}
            <br />
            Код:&nbsp;
            {_.get(row, "worker.code", " - ")}
          </>
        );
      },
    },
    {
      title: "Зарегистрировано",
      dataIndex: "registeredAt",
      className: "noWrapCell",
      key: "registeredAt",
      filters: [
        {
          text: "Вчера",
          value: JSON.stringify({
            before: moment().utc().subtract(1, "days").endOf("day").format(),
            after: moment().utc().subtract(1, "days").startOf("day").format(),
          }),
        },
        {
          text: "Сегодня",
          value: JSON.stringify({
            before: moment().utc().endOf("day").format(),
            after: moment().utc().startOf("day").format(),
          }),
        },
        {
          text: "Завтра",
          value: JSON.stringify({
            before: moment().utc().add(1, "days").endOf("day").format(),
            after: moment().utc().add(1, "days").startOf("day").format(),
          }),
        },
        {
          text: "Эта неделя",
          value: JSON.stringify({
            before: moment().utc().endOf("week").format(),
            after: moment().utc().startOf("week").format(),
          }),
        },
        {
          text: "Прошлая неделя",
          value: JSON.stringify({
            before: moment().utc().subtract(1, "weeks").endOf("week").format(),
            after: moment().utc().subtract(1, "weeks").startOf("week").format(),
          }),
        },
      ],
      filteredValue: filters["registeredAt"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (moment(a.registeredAt) < moment(b.registeredAt)) return -1;
        if (moment(a.registeredAt) > moment(b.registeredAt)) return 1;
        return 0;
      },

      render: (text, row) => {
        return row.registeredAt
          ? moment(row.registeredAt).format("LLL")
          : "N/A";
      },
    },
    {
      title: "Синхронизация",
      dataIndex: "syncedAt",
      className: "noWrapCell",
      key: "syncedAt",
      sorter: (a, b) => {
        const aSyncedAt = _.get(a, "sync.syncedAt", null);
        const bSyncedAt = _.get(b, "sync.syncedAt", null);

        if (moment(aSyncedAt) < moment(bSyncedAt)) return -1;
        if (moment(aSyncedAt) > moment(bSyncedAt)) return 1;

        return 0;
      },

      render: (text, row) => {
        const syncedAt = _.get(row, "sync.syncedAt", null);

        return (
          <>
            Время: <b>{syncedAt ? moment(syncedAt).calendar() : "N/A"}</b>
            <br />
            ID: <b>{_.get(row, "sync.id", "N/A")}</b>
            <br />
            1C: <b>{_.get(row, "sync.name", "N/A")}</b> (
            {_.get(row, "sync.toSend", false) ? "V" : "X"})
            <br />
            Статус: <b>{_.get(row, "sync.status", "N/A")}</b>
          </>
        );
      },
    },
    {
      title: "Синхронизация",
      dataIndex: "toSend",
      className: "noWrapCell",
      width: "50px",
      key: "toSend",
      sorter: (a, b) => {
        const aToSend = _.get(a, "sync.toSend", null);
        const bToSend = _.get(b, "sync.toSend", null);

        if (moment(aToSend) < moment(bToSend)) return -1;
        if (moment(aToSend) > moment(bToSend)) return 1;

        return 0;
      },

      render: (text, row) => {
        return (
          <Button
            type="primary"
            shape="circle"
            loading={isLoading}
            icon="reload"
            onClick={() => callSync1C(row.id)}
            disabled={!row.sync}
          />
        );
      },
    },
    {
      title: "Действия",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => {
        return (
          <ActionWrapper>
            <div>
              {hasAccessToEdit ? (
                <a onClick={() => handleModal(row)} href="# ">
                  <i className="ion-android-create" />
                </a>
              ) : (
                <a disabled href="#">
                  <i className="ion-android-create" style={{ color: "#aaa" }} />
                </a>
              )}

              <Link to={`/dashboard/audit/${row.id}`}>
                <Icon type="history" />
              </Link>
            </div>
          </ActionWrapper>
        );
      },
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Тележки <small>всего найдено: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            onClose={() => dispatch(toggleModal(null))}
            title={
              <>
                <span onDoubleClick={callToggleDisabled}>
                  {"Изменить тележку " + trolley.trolley}
                </span>
                <br />
                <small>{trolley.id}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord("update", trolley)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: isDisabled || !hasAccessToEdit }}
            cancelButtonProps={{ disabled: isDisabled }}
          >
            <Form>
              <Fieldset>
                <Label>Основные параметры</Label>

                <FormItem label="Сотрудник" hasFeedback>
                  <AutoComplete
                    onSearch={handleWorkerSearchThrottled}
                    onSelect={handleWorkerSelect}
                    placeholder="Input here"
                    defaultValue={trolley.workerName}
                    disabled={isDisabled}
                  >
                    <AutoCompleteOption value="" key={0}>
                      [не назначено]
                    </AutoCompleteOption>
                    {autocomplete.map((w) => {
                      return (
                        <AutoCompleteOption value={w.id} key={w.id}>
                          {w.name}
                        </AutoCompleteOption>
                      );
                    })}
                  </AutoComplete>
                </FormItem>

                <FormItem label="Дата и время" hasFeedback>
                  <ConfigProvider locale={locale}>
                    <DatePicker
                      locale={locale}
                      value={
                        trolley.registeredAt
                          ? moment(trolley.registeredAt)
                          : null
                      }
                      showTime
                      format="YYYY/MM/DD HH:mm:ss"
                      onChange={handleDateSelect}
                      disabled={isDisabled}
                      allowClear={false}
                    />
                  </ConfigProvider>
                </FormItem>
              </Fieldset>
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
