import React from "react";
import { useDispatch, useSelector } from "react-redux";
import userActions from "@iso/redux/users/actions";
import Button from "@iso/components/uielements/button";
import _ from "lodash";
import LayoutContentWrapper from "@iso/components/utility/layoutWrapper";
import Box from "@iso/components/utility/box";
import ContentHolder from "@iso/components/utility/contentHolder";
import Modal from "@iso/components/Feedback/Modal";
import { FilterDropdown } from "@iso/components/Tables/HelperCells";
import { Icon } from "antd";
import Form from "@iso/components/uielements/form";
import Input from "@iso/components/uielements/input";
import Select, { SelectOption } from "@iso/components/uielements/select";
import Popconfirms from '@iso/components/Feedback/Popconfirm';
import Collapse from '@iso/components/uielements/collapse';

import roles from "@iso/config/roles.config";

import {
  Fieldset,
  TitleWrapper,
  ComponentTitle,
  TableWrapper,
  ActionWrapper,
  Label,
} from "./User.styles";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const { Panel } = Collapse;
const Option = SelectOption;

const FormItem = Form.Item;

const {
  loadFromAgroInvest,
  saveIntoAgroInvest,
  toggleModal,
  update,
  setFilters,
  deleteUser,
} = userActions;

export default function User() {
  const {
    users,
    isLoading,
    totalItems,
    modalActive,
    itemsPerPage,
    page,
    user,
    filters,
    errorMessage,
  } = useSelector((state) => state.Users);

  const dispatch = useDispatch();

  const [state, setState] = React.useState({});

  const { greenhouses } = useSelector((state) => state.App);

  const { role } = useSelector((state) => state.Auth);

  const {
    blocksPermissions: { users: usersPermissions },
  } = useSelector((state) => _.get(state, "Auth.userPermissions"));

  const { edit: hasAccessToEdit } = usersPermissions;

  const onInputChange = (key, value) => {
    setState({ ...state, [key]: value });
  };

  const onInputChangeThrottled = _.throttle(
    (key, value) => onInputChange(key, value),
    500,
    {
      leading: false,
    }
  );

  const onSearch = (e) => {
    const { searchUserEmail, searchUserName } = state;

    setState({
      ...state,
      filterUserEmailDropdownVisible: false,
      filterUserNameDropdownVisible: false,
    });

    const newFilters = _.cloneDeep(filters);

    newFilters["email"] = searchUserEmail;
    newFilters["name"] = searchUserName;

    dispatch(setFilters(newFilters, 1));
  };

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const dataSource = [];
  Object.keys(users).map((user, index) => {
    return dataSource.push({
      ...users[user],
      key: user,
    });
  });

  const handleRecord = (user) => {
    dispatch(saveIntoAgroInvest(user));
  };

  const handleDeleteRecord = (user) => {
    dispatch(deleteUser(user));
  };

  const handleModal = async (user = null) => {
    console.log(user);
    await dispatch(toggleModal(user));
  };

  const userUpdate = (key, value) => {
    if (key) user[key] = value;
    dispatch(update(user));
  };

  const reload = () => {
    dispatch(loadFromAgroInvest());
  };

  const dropFilters = () => {
    dispatch(setFilters({}, 1));
  };

  const handleTableChange = (pagination, newFilters, sorter) => {
    newFilters = _.assign(filters, newFilters);

    dispatch(setFilters(newFilters, pagination.current));
  };

  const getGreenhouseName = (greenhouse) => {
    return _.get(
      _.find(
        greenhouses,
        (g) => g.id === greenhouse.greenhouseId && greenhouse.isActive
      ),
      "description",
      null
    );
  };

  const generateActionButtons = (row) => {
    const style = hasAccessToEdit
      ? {}
      : {
          opacity: "0.3",
        };

    return (
      <ActionWrapper>
        <a
          onClick={() => handleModal(row)}
          disabled={!hasAccessToEdit}
          href="# "
        >
          <i className="ion-android-create" style={{ ...style }} />
        </a>
        {role === 'superuser' && (
          <Popconfirms
            title={`Вы действительно хотите удалить из системы профиль пользователя ${row.name}?`}
            okText="Да"
            cancelText="Нет"
            placement="topRight"
            onConfirm={() => handleDeleteRecord(row)}
          >
            <a className="deleteBtn" href="# ">
              <i className="ion-close" />
            </a>
          </Popconfirms>
        )}
      </ActionWrapper>
    );
  };

  const columns = [
    {
      title: "Имя",
      dataIndex: "name",
      key: "name",
      sorter: (a, b) => {
        if (a.name < b.name) return -1;
        if (a.name > b.name) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchUserName", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.name ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterUserNameDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterUserNameDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.name;
      },
    },
    {
      title: "Email",
      dataIndex: "email",
      key: "email",
      sorter: (a, b) => {
        if (a.email < b.email) return -1;
        if (a.email > b.email) return 1;
        return 0;
      },
      filterDropdown: (
        <FilterDropdown
          searchText=""
          onInputChange={({ target: { value } }) =>
            onInputChangeThrottled("searchUserEmail", value)
          }
          onSearch={onSearch}
        />
      ),
      filterIcon: (
        <Icon
          type="search"
          style={{ color: filters.email ? "#108ee9" : "#aaa" }}
        />
      ),
      filterDropdownVisible: state.filterUserEmailDropdownVisible,
      onFilterDropdownVisibleChange: (visible) => {
        setState({ ...state, filterUserEmailDropdownVisible: visible });
        document.getElementById("tableFilterInput").focus();
      },
      render: (text, row) => {
        return row.email;
      },
    },
    {
      title: "Роль",
      dataIndex: "role",
      key: "role",
      filters: _.map(roles, (r) => {
        return {
          text: r,
          value: r,
        };
      }),
      filteredValue: filters["role"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (a.role < b.role) return -1;
        if (a.role > b.role) return 1;
        return 0;
      },
      render: (text, row) => {
        return row.role;
      },
    },
    {
      title: "Теплицы",
      dataIndex: "greenhouses",
      key: "greenhouses.greenhouseId",
      filters: _.map(greenhouses, (g) => {
        return {
          text: g.description,
          value: g.id,
        };
      }),
      filteredValue: filters["greenhouses.greenhouseId"] || [],
      filterMultiple: false,
      sorter: (a, b) => {
        if (a.greenhouses.length < b.greenhouses.length) return -1;
        if (a.greenhouses.length > b.greenhouses.length) return 1;
        return 0;
      },
      render: (text, row) => {
        return _.map(_.get(row, "greenhouses", []), (greenhouse) => (
          <div key={greenhouse.id}>{getGreenhouseName(greenhouse)}</div>
        ));
      },
    },
    {
      title: "Регистрация в системе",
      dataIndex: "createdAt",
      key: "createdAt",
      sorter: (a, b) => {
        if (a.createdAt < b.createdAt) return -1;
        if (a.createdAt > b.createdAt) return 1;
        return 0;
      },
      render: (text, row) => {
        return moment(row.createdAt).format("DD-MM-YYYY h:mm:ss");
      },
    },
    {
      title: "Последний вход в систему",
      dataIndex: "lastLoggedAt",
      key: "lastLoggedAt",
      sorter: (a, b) => {
        if (a.lastLoggedAt < b.lastLoggedAt) return -1;
        if (a.lastLoggedAt > b.lastLoggedAt) return 1;
        return 0;
      },
      render: (text, row) => {
        return moment(row.lastLoggedAt).format("DD-MM-YYYY h:mm:ss");
      },
    },
    {
      title: "Действия",
      key: "action",
      width: "60px",
      className: "noWrapCell",
      render: (text, row) => generateActionButtons(row),
    },
  ];

  return (
    <LayoutContentWrapper>
      <Box>
        <ContentHolder style={{ marginTop: 0, overflow: "hidden" }}>
          <TitleWrapper>
            <ComponentTitle>
              Пользователи <small>всего: {totalItems}</small>
            </ComponentTitle>
            <div>
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                disabled={!hasAccessToEdit}
                icon="plus"
                onClick={() => handleModal()}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="filter"
                onClick={dropFilters}
                style={{ marginLeft: "10px" }}
              />
              <Button
                type="primary"
                shape="circle"
                loading={isLoading}
                icon="reload"
                onClick={reload}
                style={{ marginLeft: "10px" }}
              />
            </div>
          </TitleWrapper>

          <Modal
            visible={modalActive}
            title={
              <>
                {user.id ? `Изменить параметр "${user.email}"` : "Добавить"}
                <br />
                <small>{user.id ? user.id : "Новый пользователь"}</small>
              </>
            }
            okText={"Сохранить"}
            onOk={() => handleRecord(user)}
            onCancel={() => dispatch(toggleModal(null))}
            okButtonProps={{ disabled: !hasAccessToEdit }}
            cancelButtonProps={{ disabled: false }}
            confirmLoading={isLoading}
          >
            <Form>
              <Fieldset>
                <span style={{ color: "red" }}>{errorMessage}</span>
                <FormItem label="Имя" hasFeedback>
                  <Input
                    size="large"
                    value={_.get(user, "name", undefined)}
                    placeholder="Введите имя"
                    onChange={({ target: { value } }) =>
                      userUpdate("name", value)
                    }
                  />
                </FormItem>
                {!user.id && (
                  <>
                    <FormItem label="Email" hasFeedback>
                      <Input
                        size="large"
                        value={_.get(user, "email", undefined)}
                        placeholder="Введите почту"
                        onChange={({ target: { value } }) =>
                          userUpdate("email", value)
                        }
                      />
                    </FormItem>
                    <FormItem label="Пароль" hasFeedback>
                      <Input
                        size="large"
                        value={_.get(user, "password", undefined)}
                        placeholder="Введите пароль"
                        onChange={({ target: { value } }) =>
                          userUpdate("password", value)
                        }
                      />
                    </FormItem>
                    <FormItem label="Подтверждение пароля" hasFeedback>
                      <Input
                        size="large"
                        value={_.get(user, "confirmPassword", undefined)}
                        placeholder="Повторите пароль"
                        onChange={({ target: { value } }) =>
                          userUpdate("confirmPassword", value)
                        }
                      />
                    </FormItem>
                  </>
                )}
                <FormItem label="Роль" hasFeedback>
                  <Select
                    size="large"
                    value={_.get(user, "role", undefined)}
                    placeholder="Выберите роль"
                    onChange={(value) => userUpdate("role", value)}
                  >
                    {_.map(roles, (r) => (
                      <Option value={r} key={r}>
                        {r}
                      </Option>
                    ))}
                  </Select>
                </FormItem>
                <FormItem label="Привязанные теплицы" hasFeedback>
                  <Select
                    mode="multiple"
                    size="large"
                    value={
                      !_.isEmpty(user.greenhouses)
                        ? user.greenhouses
                        : undefined
                    }
                    placeholder="Выберите теплицы"
                    onChange={(value) => userUpdate("greenhouses", value)}
                  >
                    {_.map(greenhouses, (g) => (
                      <Option value={g.id} key={g.id}>
                        {g.description}
                      </Option>
                    ))}
                  </Select>
                </FormItem>
              </Fieldset>
              {user.id && role === 'superuser' && (
                <Fieldset>
                  <Collapse accordion>
                    <Panel header="Изменить пароль" key="1">
                      <FormItem label="Пароль" hasFeedback>
                        <Input
                          size="large"
                          value={_.get(user, "password", undefined)}
                          placeholder="Введите пароль"
                          onChange={({ target: { value } }) =>
                            userUpdate("password", value)
                          }
                        />
                      </FormItem>
                      <FormItem label="Подтверждение пароля" hasFeedback>
                        <Input
                          size="large"
                          value={_.get(user, "confirmPassword", undefined)}
                          placeholder="Повторите пароль"
                          onChange={({ target: { value } }) =>
                            userUpdate("confirmPassword", value)
                          }
                        />
                      </FormItem>
                    </Panel>
                  </Collapse>
                </Fieldset>
              )}
            </Form>
          </Modal>

          <TableWrapper
            rowKey="key"
            columns={columns}
            bordered={true}
            dataSource={dataSource}
            loading={isLoading}
            className="isoSimpleTable"
            onChange={handleTableChange}
            pagination={{
              defaultPageSize: itemsPerPage,
              hideOnSinglePage: true,
              total: totalItems,
              current: page,
              showTotal: (total, range) => {
                return `Показано ${range[0]}-${range[1]} из ${totalItems} записей`;
              },
            }}
          />
        </ContentHolder>
      </Box>
    </LayoutContentWrapper>
  );
}
