import React, { useEffect } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import valveLimitActions from '@iso/redux/valveLimit/actions';
import _ from 'lodash';
import ContentHolder from '@iso/components/utility/contentHolder';
import { Alert, Icon } from 'antd';
import Input from '@iso/components/uielements/input';
import { MEASUREMENTS } from '@iso/containers/Phenology/constants';
import { TitleWrapper, ComponentTitle, TableWrapper, InputWrapper } from './ValveLimit.styles';
import * as yup from 'yup';
import moment from 'moment-timezone';
import 'moment/locale/ru';

const schema = yup.number().moreThan(-0.001).required();
moment.tz.setDefault('Europe/Moscow');
moment.locale('ru');

const { loadFromAgroInvest, saveIntoAgroInvest } = valveLimitActions;

const ValidateInput = ({ value, onAccept, isValid, disable }) => {
  const [state, setState] = React.useState({
    value,
    editable: false,
  });
  useEffect(() => {
    setState((prevState) => ({ ...prevState, value }));
  }, [value]);
  const handleChange = (event) => {
    setState({ ...state, value: event.target.value });
  };
  const check = () => {
    if (!isValid(state.value)) {
      return;
    }
    if (onAccept) {
      onAccept(state.value);
    }
    setState({ ...state, editable: false });
  };
  const edit = () => {
    setState({ ...state, editable: true });
  };

  return (
    <InputWrapper valid={isValid(state.value)} className="isoEditData">
      {state.editable && !disable ? (
        <>
          <div className="isoEditDataWrapper">
            <Input
              value={state.value}
              onChange={handleChange}
              onPressEnter={check}
              onBlur={check}
            />
            <Icon type="check" className="isoEditIcon" onClick={check} />
          </div>
          {isValid(state.value) || (
            <Alert message="Введите верное значение" type="error" showIcon />
          )}
        </>
      ) : (
        <div
          style={{
            padding: '5px 12px',
            cursor: 'pointer',
          }}
          onClick={edit}
        >
          {state.value}
        </div>
      )}
    </InputWrapper>
  );
};

const ValveLimit = () => {
  const { valveLimits, isLoading } = useSelector((state) => state.ValveLimit);
  const { 'valve-limit': valveLimitPermissions } = useSelector((state) =>
    _.get(state, 'Auth.userPermissions.blocksPermissions.phenology.blocks'),
  );

  const { edit: hasAccessToEdit } = valveLimitPermissions;

  const dispatch = useDispatch();

  React.useEffect(() => {
    dispatch(loadFromAgroInvest());
  }, [dispatch]);

  const handleRecord = (record, actionName) => {
    dispatch(saveIntoAgroInvest(record, actionName));
  };

  const columns = [
    {
      title: 'Вид работы',
      dataIndex: 'workDescription',
      render: (value, row, index) => {
        const rowSpans = [2, 0, 3, 0, 0, 3, 0, 0];
        return {
          children: _.capitalize(value),
          props: {
            rowSpan: rowSpans[index],
          },
        };
      },
    },
    {
      title: 'Наименование измерений',
      dataIndex: 'measurementDescription',
      render: (value) => MEASUREMENTS[value],
    },
    {
      title: 'Нижняя граница',
      dataIndex: 'minLimit',
      width: '300px',
      render: (text, record) => (
        <ValidateInput
          value={text}
          onAccept={(value) => handleRecord({ ...record, minLimit: Number(value) })}
          isValid={schema.isValidSync.bind(schema)}
          disable={!hasAccessToEdit}
        />
      ),
    },
    {
      title: 'Верхняя граница',
      dataIndex: 'maxLimit',
      width: '300px',
      render: (text, record) => (
        <ValidateInput
          value={text}
          onAccept={(value) => handleRecord({ ...record, maxLimit: Number(value) })}
          isValid={schema.isValidSync.bind(schema)}
          disable={!hasAccessToEdit}
        />
      ),
    },
  ];
  return (
    <ContentHolder style={{ marginTop: 0, overflow: 'hidden' }}>
      <TitleWrapper>
        <ComponentTitle>Граничные значения по сбору вытяжек</ComponentTitle>
      </TitleWrapper>
      <TableWrapper
        columns={columns}
        bordered
        dataSource={valveLimits}
        loading={isLoading}
        className="isoEditableTable"
        pagination={false}
        rowKey={(record) => record.id}
      />
    </ContentHolder>
  );
};

export default ValveLimit;
