import React, { useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import {
  PlanWidgetWrapper,
  TitleWrapper,
  ComponentTitle,
  ActionWrapper,
  BoldText,
} from "./PlanWidget.styles";
import { Spin } from "antd";
import Button from "@iso/components/uielements/button";
import Modal from "@iso/components/Feedback/Modal";
import Input from "@iso/components/uielements/input";
import Select, { SelectOption } from "@iso/components/uielements/select";

import _ from "lodash";
import actions from "@iso/redux/dashboard/actions";
import uuidV4 from "uuid/v4";
import moment from "moment-timezone";
import "moment/locale/ru";

import { TableViews } from "@iso/containers/Tables/AntTables/AntTables";

import {
  Ranges,
  isValid,
  normalize,
  getCountRow,
} from "@iso/lib/helpers/Ranges";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

const {
  loadRowPlanFromAgroinvest,
  saveRowRangeToAgroinvest,
  loadMiniPlansFromAgroInvest,
  selectPlan,
  changePlanToAgroInvest,
} = actions;

const Option = SelectOption;

const PlanWidget = ({ tab, planPermissions, modules }) => {
  const { role } = useSelector((state) => state.Auth);
  const {
    rowPlan,
    miniPlans,
    selectedPlan,
    greenhouseSections,
    isLoadingRowPlan,
    isLoadingModule,
    module,
  } = useSelector((state) => state.Dashboard);
  const {
    sectionsEdit,
    sectionsRead,
    actions,
    tabsEdit,
  } = planPermissions;

  const hasAccessToEdit = (
    tabsEdit === null
      || (
        Array.isArray(tabsEdit)
          && tabsEdit.includes(tab)
      )
  );

  const hasAccessToChangeCells =
    hasAccessToEdit && (!actions || _.includes(actions, "change-rows"));
  const hasAccessToChangePlans = role === 'agronomist' && module === "guarding" ? false :
    hasAccessToEdit && (!actions || _.includes(actions, "change-plans"));

  const [modal, setModal] = useState({ active: false, type: "add" });
  const dispatch = useDispatch();
  const [planName, setPlanName] = useState("");

  const [filterSections, setFilterSections] = useState([]);
  const { greenhouseId, greenhouses } = useSelector((state) => state.App);
  const currentGreenhouse = _.find(
    greenhouses,
    (item) => item.id === greenhouseId
  );

  const availableRowPlan = _.isNil(sectionsRead)
  ? (rowPlan)
  : _.filter(rowPlan, (item) =>
  _.includes(sectionsRead, _.get(item, 'greenhouseSection.module'))
  );

  const filteredRowPlan = availableRowPlan.filter(
    ({ greenhouseSection: { sectionId, module: sectionModule } }) => {
      return !filterSections.includes(sectionId) && modules.includes(sectionModule);
    }
  );

  useEffect(() => {
    if (!_.isEmpty(greenhouseId)) {
      dispatch(loadMiniPlansFromAgroInvest());
    }
  }, [greenhouseId, dispatch]);

  const activePlan = _.find(miniPlans, (p)=>p.isActive && !p.isDeleted);

  useEffect(() => {
    if (greenhouseSections && selectedPlan?.id) {
      dispatch(loadRowPlanFromAgroinvest());
    } else if (activePlan) {
      dispatch(selectPlan(activePlan));
    }
  }, [greenhouseSections, selectedPlan, activePlan, dispatch]);

  const validator = (rule, value, callback) => {
    if (isValid(value)) {
      callback();
    } else {
      callback("Ошибка");
    }
  };

  const getPlanById = (id) => _.find(miniPlans, { id });
  const closeModal = () => {
    setModal((prevState) => ({ ...prevState, active: false }));
    setPlanName("");
  };
  const handleAddPlan = () => {
    const createdPlan = {
      id: uuidV4(),
      description: planName,
      isActive: false,
      isDeleted: false,
      greenhouse: currentGreenhouse["@id"],
      module
    };
    dispatch(changePlanToAgroInvest({ action: "addPlan", plan: createdPlan }));
    closeModal();
  };

  const handleChangePlanName = () => {
    const updatedPlan = { ...selectedPlan, description: planName };
    dispatch(
      changePlanToAgroInvest({ action: "changeDescription", plan: updatedPlan })
    );
    closeModal();
  };

  const handleSetActivePlan = (id) => () => {
    const updatedPlan = { ...selectedPlan, id, isActive: true };
    const handleSetActive = () => {
      dispatch(
        changePlanToAgroInvest({ action: "setActive", plan: updatedPlan })
      );
      closeModal();
    };
    Modal.confirm({
      onOk: handleSetActive,
      okText: "Да",
      cancelText: "Нет",
      okType: "primary",
      title: (
        <span>
          Вы уверены, что хотите мини-план{" "}
          <BoldText>{selectedPlan?.description} </BoldText> сделать активным?
        </span>
      ),
    });
  };

  const handleDeletePlan = (id) => () => {
    const updatedPlan = { ...selectedPlan, id, isDeleted: true };
    dispatch(
      changePlanToAgroInvest({ action: "removePlan", plan: updatedPlan })
    );
  };

  const handleChangePlan = (plan) => {
    dispatch(selectPlan(plan));
  };

  const handleDeleteModal = (id) => () => {
    id === activePlan?.id
      ? Modal.error({
          title: "Не допустимое действие",
          content: "Нельзя удалить активный план",
        })
      : Modal.confirm({
          onOk: handleDeletePlan(id),
          okText: "Да",
          cancelText: "Нет",
          okType: "danger",
          text: "Удалить",
          title: (
            <span>
              Вы уверены, что хотите удалить мини-план{" "}
              <BoldText>{selectedPlan?.description} </BoldText> без возможности
              восстановления?
            </span>
          ),
        });
  };

  const checkEditable = (cellData) => {
    const module = _.get(cellData, "greenhouseSection.module");

    const hasAccessToEditCell =
      hasAccessToEdit &&
      hasAccessToChangeCells &&
      (_.isNil(sectionsEdit) || _.includes(sectionsEdit, module));

    return hasAccessToEditCell;
  };

  const dayColumns = moment.weekdaysShort(true).map((day, dayIndex) => ({
    title: day,
    dataIndex: `${dayIndex}.ranges.string`,
    width: 80,
    editable: true,
    checkEditable,
    render: (cell) => {
      return {
        props: {
          title: getCountRow(cell),
        },
        children: cell,
      };
    },
  }));

  const columns = [
    {
      title: "Вид работ",
      dataIndex: "greenhouseSection.name",
      key: "greenhouseSection.sectionId",
      width: 100,
    },
    ...dayColumns,
  ];

  const modals = {
    add: {
      title: "Создать новый план",
      onOk: handleAddPlan,
    },
    rename: {
      title: `Переименовать мини-план: ${selectedPlan?.description}`,
      onOk: handleChangePlanName,
    },
  };
  return <Spin spinning={isLoadingModule}>{renderPlanWidget()}</Spin>;

  function renderPlanWidget() {
    return (
      <PlanWidgetWrapper>
        <>
          <TitleWrapper>
            <ComponentTitle>
              Активный мини-план:{' '}
              <BoldText>
                {activePlan?.description || 'Ничего не выбрано'}
              </BoldText>
            </ComponentTitle>
          </TitleWrapper>

          <ActionWrapper>
            <Select
              value={selectedPlan?.description || 'Ничего не выбрано'}
              style={{
                marginBottom: '20px',
                marginRight: '10px',
                minWidth: '300px',
              }}
              onChange={(id) => handleChangePlan(getPlanById(id))}
            >
              {miniPlans.map((plan) => {
                return (
                  <Option value={plan.id} key={plan.id}>
                    {plan.description}
                  </Option>
                );
              })}
            </Select>
            <Button
              onClick={() => {
                setModal({ active: true, type: 'add' });
              }}
              disabled={!hasAccessToChangePlans}
              type="primary"
              style={{ marginRight: '10px' }}
            >
              Создать
            </Button>
            <Button
              onClick={() => {
                setModal({ active: true, type: 'rename' });
              }}
              disabled={!selectedPlan?.description || !hasAccessToChangePlans}
              type="primary"
              style={{ marginRight: '10px' }}
            >
              Переименовать
            </Button>
            <Button
              onClick={handleSetActivePlan(selectedPlan?.id)}
              disabled={!selectedPlan?.description || !hasAccessToChangePlans}
              type="primary"
              style={{ marginRight: '10px' }}
            >
              Сделать активным
            </Button>
            <Button
              onClick={handleDeleteModal(selectedPlan?.id)}
              disabled={!selectedPlan?.description || !hasAccessToChangePlans}
              type="danger"
              style={{ marginRight: '10px' }}
            >
              Удалить план
            </Button>
          </ActionWrapper>
          <ActionWrapper>
            <div style={{ marginTop: '-20px' }}>
              {'Исключить из отображения виды работ: '}
            </div>
            <Select
              mode="multiple"
              placeholder="Выберите вид работы"
              value={filterSections}
              onSelect={(e) => setFilterSections((prev) => [...prev, e])}
              onDeselect={(e) =>
                setFilterSections((prev) => prev.filter((el) => el !== e))
              }
              style={{ width: '100%', marginTop: '15px' }}
            >
              {greenhouseSections.map((g) => (
                <SelectOption key={g.id} value={g.sectionId}>
                  {g.name}
                </SelectOption>
              ))}
            </Select>
          </ActionWrapper>
          {modal && (
            <Modal
              visible={modal.active}
              onClose={closeModal}
              title={modals[modal.type].title}
              okText={'Сохранить'}
              okButtonProps={{
                disabled: !planName,
              }}
              onOk={modals[modal.type].onOk}
              onCancel={closeModal}
            >
              <Input
                placeholder="Введите новое имя"
                value={planName}
                onChange={(e) => setPlanName(e.target.value)}
              />
            </Modal>
          )}
          <TableViews.EditView
            bordered
            pagination={false}
            handleSave={(rowRange) => {
              rowRange.ranges = new Ranges(rowRange.ranges.string);
              rowRange.range = rowRange.ranges.string;
              dispatch(saveRowRangeToAgroinvest(rowRange));
            }}
            loading={isLoadingRowPlan}
            normalize={normalize}
            validator={validator}
            columns={columns}
            dataSource={selectedPlan?.id ? filteredRowPlan : []}
          />
        </>
      </PlanWidgetWrapper>
    );
  }
};

export { PlanWidget };
