import { Map } from "immutable";
import _ from "lodash";
import { store } from "../../redux/store";
import appActions from "../../redux/app/actions";
import { feathersApp } from "../feathers/feathers";

export function clearToken() {
  feathersApp.logout();
}

export function clearEmail() {
  localStorage.removeItem("email");
}

export function getToken() {
  try {
    const idToken = localStorage.getItem("auth");
    return new Map({ idToken });
  } catch (err) {
    clearToken();
    return new Map();
  }
}

export function getEmail() {
  try {
    const email = localStorage.getItem("email");
    return new Map({ email });
  } catch (err) {
    clearEmail();
    return new Map();
  }
}

export function arrayEqual(array1, array2) {
  return array1.sort().toString() === array2.sort().toString();
}

export function timeDifference(givenTime) {
  givenTime = new Date(givenTime);
  const milliseconds = new Date().getTime() - givenTime.getTime();
  const numberEnding = (number) => {
    return number > 1 ? "s" : "";
  };
  const number = (num) => (num > 9 ? "" + num : "0" + num);
  const getTime = () => {
    let temp = Math.floor(milliseconds / 1000);
    const years = Math.floor(temp / 31536000);
    if (years) {
      const month = number(givenTime.getUTCMonth() + 1);
      const day = number(givenTime.getUTCDate());
      const year = givenTime.getUTCFullYear() % 100;
      return `${day}-${month}-${year}`;
    }
    const days = Math.floor((temp %= 31536000) / 86400);
    if (days) {
      if (days < 28) {
        return days + " day" + numberEnding(days);
      } else {
        const months = [
          "Jan",
          "Feb",
          "Mar",
          "Apr",
          "May",
          "Jun",
          "Jul",
          "Aug",
          "Sep",
          "Oct",
          "Nov",
          "Dec",
        ];
        const month = months[givenTime.getUTCMonth()];
        const day = number(givenTime.getUTCDate());
        return `${day} ${month}`;
      }
    }
    const hours = Math.floor((temp %= 86400) / 3600);
    if (hours) {
      return `${hours} hour${numberEnding(hours)} ago`;
    }
    const minutes = Math.floor((temp %= 3600) / 60);
    if (minutes) {
      return `${minutes} minute${numberEnding(minutes)} ago`;
    }
    return "a few seconds ago";
  };
  return getTime();
}

export function stringToInt(value, defValue = 0) {
  if (!value) {
    return 0;
  } else if (!isNaN(value)) {
    return parseInt(value, 10);
  }
  return defValue;
}
export function stringToPosetiveInt(value, defValue = 0) {
  const val = stringToInt(value, defValue);
  return val > -1 ? val : defValue;
}

export const isJSON = (data) => {
  let ret = true;

  try {
    JSON.parse(data);
  } catch (e) {
    ret = false;
  }

  return ret;
};

export const objectToQueryString = (obj) => {
  return Object.keys(obj)
    .flatMap((key) => {
      const elm = _.isArray(obj[key])
        ? _.map(obj[key], (v) => {
            return key + "[]=" + encodeURIComponent(v);
          })
        : _.isObject(obj[key])
        ? _.map(
            obj[key],
            (v, k) => key + "[" + k + "]=" + encodeURIComponent(v)
          )
        : key + "=" + encodeURIComponent(obj[key]);

      return elm;
    })
    .join("&");
};

export const camundaRequest = async (camundaProcess, params) => {
  const method = "POST";

  const options = {
    method,
    timeout: 60000,
    headers: {
      "Content-Type": "application/json", // we will be sending JSON
    },
  };

  if (params) {
    options.body = JSON.stringify(params); // body should match Content-Type in headers option
  }

  return await fetch(
    process.env.REACT_APP_CAMUNDA_CORS_URL +
      "/engine-rest/process-definition/key/" +
      camundaProcess +
      "/start",
    options
  )
    .then((res) => res.json())
    .catch((error) => error);
};

const errorData = { errorResolved: true };

export const request = async (url, params, method = "GET", mode = "cors") => {
  const contentTypes = {
    PATCH: "application/merge-patch+json",
    POST: "application/ld+json",
    GET: "application/json",
    DELETE: "",
  };

  if (method != "GET") {
    const state = store.getState();

    const userRole = _.get(state, "Auth.role");

    if (userRole === "reader") {
      if (errorData.errorResolved) {
        const error = {
          message: "Пользователи с ролью reader не могут вносить изменения",
        };
        store.dispatch(appActions.setError(error));
        errorData.errorResolved = false;
        setTimeout(() => (errorData.errorResolved = true), 5000);
      }

      return;
    }
  }

  const options = {
    method,
    mode,
    headers: new Headers({
      "Content-Type": contentTypes[method]
        ? contentTypes[method]
        : contentTypes.GET,
      Authorization: process.env.REACT_APP_API_KEY,
      "X-Username": getEmail().get("email"),
    }),
  };

  // if params exists and method is GET, add query string to url
  // otherwise, just add params as a "body" property to the options object
  if (params) {
    if (method === "GET") {
      url += "?" + objectToQueryString(params);
    } else {
      options.body = JSON.stringify(params); // body should match Content-Type in headers option
    }
  }

  return await fetch(process.env.REACT_APP_BACKEND_HOST + "/" + url, options)
    .then((res) => res.json())
    .catch((error) => error);
};

export const oneCRequest = async (url, params) => {
  const options = {
    method: "GET",
  };

  // if params exists and method is GET, add query string to url
  // otherwise, just add params as a "body" property to the options object
  if (params) {
    url += "?" + objectToQueryString(params);
  }

  // url = 'test.json'; // !REMOVE

  return await fetch(process.env.REACT_APP_ONEC_URL + "/" + url, options)
    .then((res) => res.json())
    .catch((error) => error);
};

export const cacheRequest = async (url) => {
  const options = {
    method: "GET",
  };

  return await fetch(process.env.REACT_APP_CAMUNDA_URL+":9999" + url, options)
    .then((res) => res.json())
    .catch((error) => error);
};
