import { all, takeEvery, put, fork, call } from 'redux-saga/effects';
import { createBrowserHistory } from 'history';

import { request } from '@iso/lib/helpers/utility';
import { getGreenhouseIdFromLocalStorage } from '@iso/lib/helpers/getGreenhouseId';
import moment from 'moment';
import { getToken, clearToken, getEmail, clearEmail } from '@iso/lib/helpers/utility';
import actions from './actions';
import appActions from '../app/actions';
import authConfig from '@iso/config/auth0.config.js';
import _ from 'lodash';

import { getUserPermissions } from '@iso/lib/helpers/getUserPermissions';
import { reAuthenticate } from '../../library/feathers/feathers.authentication.util';

const history = createBrowserHistory();
const fakeApiCall = false; // auth0 or express JWT

export function* loginRequest() {
  yield takeEvery('LOGIN_REQUEST', function* ({ payload }) {
    const { token, userData } = payload;
    if (token && userData) {
      const userPermissions = getUserPermissions(userData.role);
      yield call(request, `managers/${userData.id}`, { lastLoggedAt: userData.lastLoggedAt }, 'PATCH');
      yield put({
        type: actions.LOGIN_SUCCESS,
        token,
        profile: userData,
        userPermissions,
      });
    } else if (fakeApiCall) {
      yield put({
        type: actions.LOGIN_SUCCESS,
        token: 'secret token',
        profile: 'Profile',
      });
    } else {
      yield put({ type: actions.LOGIN_ERROR });
    }
  });
}

export function* loginSuccess() {
  yield takeEvery(actions.LOGIN_SUCCESS, function* (payload) {
    yield localStorage.setItem('email', payload.profile.email);
  });
}

export function* loginError() {
  yield takeEvery(actions.LOGIN_ERROR, function* () {});
}

export function* logout() {
  yield takeEvery(actions.LOGOUT, function* () {
    yield clearToken();
    yield clearEmail();
    history.push('/');
  });
}
export function* checkAuthorization() {
  yield takeEvery(actions.CHECK_AUTHORIZATION, function* () {
    const token = getToken().get('idToken');
    const email = getEmail().get('email');
    const response = yield call(request, 'managers', {
      email,
    });

    const authResponse = yield call(reAuthenticate);

    if (authResponse && token && !_.isEmpty(response['hydra:member'])) {
      const greenhouses = _.get(response['hydra:member'][0], 'greenhouses', []);
      const userRole = _.get(response['hydra:member'][0], 'role', null);
      const lastLoggedAt = _.get(response['hydra:member'][0], 'lastLoggedAt', null);

      const isNotSuperUser = userRole !== `superuser`;
      const expiredTimeLogin = moment().subtract(authConfig.forceRelogAfterDays, 'days');
      const isExpiredTimeLogin = moment(expiredTimeLogin).isAfter(lastLoggedAt);
      if (isNotSuperUser && isExpiredTimeLogin) {
        return;
      }
      const userPermissions = getUserPermissions(userRole);
      const greenhouseId = getGreenhouseIdFromLocalStorage(greenhouses, userRole);
      yield put(appActions.setGreenhouse(greenhouseId));

      yield put({
        type: actions.LOGIN_SUCCESS,
        token,
        profile: response['hydra:member'][0],
        userPermissions,
      });
    }
  });
}
export default function* rootSaga() {
  yield all([
    fork(checkAuthorization),
    fork(loginRequest),
    fork(loginSuccess),
    fork(loginError),
    fork(logout),
  ]);
}
