import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import _ from 'lodash'

import actions from './actions';
import { socket } from '@iso/lib/sockets'
import { request } from "@iso/lib/helpers/utility";

const COLLECTION_WORKER_NAME = 'workers';

const getSelectedChatRoom = state => state.Chat.selectedChatRoom;

const getCurrentUser = () => {
  return {
    userId: 'manager',
    user: {
      id: 'manager',
      profileImageUrl:
        '/static/media/user1.b81ffc69.png',
    },
  };
};

function* initChat() {
  const currentUser = getCurrentUser();

  const workersParams = {
    exists: {
      spreadTill: true,
    },
    order: {
      name: 'asc',
    },
  }

  const data = yield call(request, COLLECTION_WORKER_NAME, workersParams);

  const users = data['hydra:member'];
  const chatRooms = [];

  if (users) {
    for (let user of users) {
      chatRooms.push({
        id: `worker.${user.id}`,
        userId: currentUser.userId,
        otherUserId: user.id,
        otherUserInfo: user,
        lastMessage: '',
        lastMessageTime: 0
      });
    }
  }

  socket.emit('manager-join', _.map(chatRooms, 'id'))

  yield put({
    type: actions.CHAT_INIT_SAGA,
    user: currentUser.user,
    userId: currentUser.userId,
    users,
    chatRooms,
    messages: [],
  });

  yield put({
    type: actions.GET_ALL_UNREAD_MESSAGES,
  });

  if (!users) {
    return false;
  }

  yield put({
    type: actions.CHAT_UPDATE_CHATROOM_SAGA,
    payload: {
      chatRoom: chatRooms[0],
      selected: false,
    }
  });
}

function* sendMessage({ payload }) {
  const selectedChatRoom = yield select(getSelectedChatRoom);
  try {
    socket.emit('new-message', JSON.stringify({
      room: selectedChatRoom.id,
      text: payload,
      sender: selectedChatRoom.userId
    }))
  } catch (error) {
    console.error(error)
  }
}


function* updateChatroomSaga({ payload }) {
  const { chatRoom } = payload;
  let { selected } = payload;

  const selectedChatRoom = yield select(getSelectedChatRoom);

  if (selected || chatRoom.id === selectedChatRoom.id) {
    try {
      socket.emit('get-room-messages', {
        room: chatRoom.id
      })
    } catch (error) {
      console.error(error)
    }

    selected = true;
  }

  if (chatRoom && selected) {
    yield put({
      type: actions.CHAT_UPDATE_CHATROOM,
      chatRoom,
      messages: [],
      selected,
    });
  }
}

function* receiveChatroomMessages({ payload }) {
  const selectedChatRoom = yield select(getSelectedChatRoom);

  if (selectedChatRoom && payload) {
    yield put({
      type: actions.CHAT_UPDATE_CHATROOM,
      chatRoom: selectedChatRoom,
      messages: payload,
      selected: false,
    });
  }
}

function* getAllUnreadMessages() {
  yield select(getSelectedChatRoom);
  socket.emit('get-unread-messages')
}

function* readChatroomMessages({ payload }) {
  yield select(getSelectedChatRoom);
  socket.emit('read-chatroom-messages', {
    room: payload
  })
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.CHAT_INIT, initChat),
    takeEvery(actions.CHAT_UPDATE_CHATROOM_SAGA, updateChatroomSaga),
    takeEvery(actions.CHAT_SEND_MESSAGE, sendMessage),
    takeEvery(actions.RECEIVE_CHATROOM_MESSAGES, receiveChatroomMessages),
    takeEvery(actions.GET_ALL_UNREAD_MESSAGES, getAllUnreadMessages),
    takeEvery(actions.READ_CHATROOM_MESSAGES, readChatroomMessages),
  ]);
}
