import { all, takeEvery, put, call, select } from "redux-saga/effects";
import actions from "./actions";
import { request } from "@iso/lib/helpers/utility";
import _ from "lodash";
import moment from "moment";

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */
const COLLECTION_ROWS = "rows"; // change your collection
const COLLECTION_CHECK_METRICS = "check_metrics";
const COLLECTION_TASK_CHECKS = "task_checks";
const COLLECTION_GREENHOUSE_SECTIONS = `greenhouse_sections`;
const getGreenhouseId = (state) => state.App.greenhouseId;
const getGreenhouses = (state) => state.App.greenhouses;

const getItemsPerPage = (state) => state.CheckLists.itemsPerPage;
const getFilters = (state) => state.CheckLists.filters;

function* loadFromAgroinvest() {
  try {
    const filters = yield select(getFilters);
    const greenhouseId = yield select(getGreenhouseId);
    const greenhouses = yield select(getGreenhouses);

    const params = {
      ...filters,
      registeredAt: {
        after: moment(filters.registeredAt).startOf("day"),
        before: moment(filters.registeredAt).endOf("day"),
      },
      "task.greenhouseId": greenhouseId,
    };

    // получаем все задачи за период
    const prepareRequest = yield call(request, COLLECTION_TASK_CHECKS, {
      ...params,
      itemsPerPage: 1,
    });
    const totalItems = prepareRequest['hydra:totalItems'];
    const itemsPerPage = 100;
    const numberPages = Math.ceil(totalItems / itemsPerPage);

    const requests = _.range(1, numberPages + 1).map((page) => {
      const requestParam = { ...params, itemsPerPage, page };
      return call(request, COLLECTION_TASK_CHECKS, requestParam);
    });
    const resultDatas = yield all(requests);
    const taskChecks = resultDatas.reduce(
      (acc, data) => {
        return {
          ...acc,
          'hydra:member': [...acc['hydra:member'], ...data['hydra:member']],
          total: data['hydra:totalItems'],
        };
      },
      { 'hydra:member': [], total: 0 }
    );

    const greenhouseDescription = _.get(
      _.find(greenhouses, (g) => g.id === greenhouseId),
      "description"
    );
    const greenhouseMatch = greenhouseDescription
      ? greenhouseDescription.match(/\(\S+?\)/)
      : undefined;

    const checkMetricParams = {
      sectionId: filters["task.sectionId"],
      itemsPerPage: 1000,
    };
    if (!filters["task.sectionId"]) {
      delete checkMetricParams["sectionId"];
    }

    if (greenhouseMatch) {
      checkMetricParams.category = greenhouseMatch[0].slice(
        1,
        greenhouseMatch[0].length - 1
      );
    }

    const checkMetrics = yield call(
      request,
      COLLECTION_CHECK_METRICS,
      checkMetricParams
    );

    yield put(
      actions.loadFromAgroInvestSuccess(
        taskChecks["hydra:member"],
        checkMetrics["hydra:member"]
      )
    );
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* rowsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      order: {
        orderBy: "asc",
      },
    };

    const data = yield call(request, COLLECTION_ROWS, params);

    yield put(
      actions.rowsLoadFromAgroInvestSuccess(
        data["hydra:member"],
        data["hydra:totalItems"]
      )
    );
  } catch (error) {
    yield put(actions.rowsLoadFromAgroInvestError(error));
  }
}

function* sectionsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const params = {
      greenhouseId,
      ["section.isQuality"]: false,
    };

    const data = yield call(request, COLLECTION_GREENHOUSE_SECTIONS, params);

    yield put(
      actions.sectionsLoadFromAgroInvestSuccess(
        data["hydra:member"],
        data["hydra:totalItems"]
      )
    );
  } catch (error) {
    yield put(actions.sectionsLoadFromAgroInvestError(error));
  }
}



export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.ROWS_LOAD_FROM_AGROINVEST, rowsLoadFromAgroinvest),
    takeEvery(
      actions.SECTIONS_LOAD_FROM_AGROINVEST,
      sectionsLoadFromAgroinvest
    ),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.RESET_FILTERS, loadFromAgroinvest),
  ]);
}
