import { all, takeEvery, put, call, select } from "redux-saga/effects";
import actions from "./actions";
import _ from "lodash";
import moment from "moment";
import { isJSON, request } from "@iso/lib/helpers/utility";

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = "empas"; // change your collection
const COLLECTION_TASK_NAME = "tasks"; // change your collection
const COLLECTION_SYNC_NAME = "syncs"; // change your collection

const getPage = (state) => state.Empas.page;
const getItemsPerPage = (state) => state.Empas.itemsPerPage;
const getOrderBy = (state) => state.Empas.orderBy;
const getFilters = (state) => state.Empas.filters;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);

    const params = {};

    params.type = "refill";

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split("|"), v.split("|")));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;

    if (params.type === "refill") delete params["worker.name"];

    let data = yield call(request, COLLECTION_NAME, params);

    const empases = data["hydra:member"];

    if (params.type === "refill") {
      const nextEmpasesData = yield all(
        _.map(empases, (empas) => {
          return call(request, COLLECTION_NAME, {
            registeredAt: {
              strictly_after: empas.registeredAt,
            },
            properties: ["id", "empasNumber", "registeredAt"],
            empasNumber: empas.empasNumber,
            itemsPerPage: 1,
            page: 1,
            type: "refill",
          });
        })
      );

      const empasesDetail = _.map(nextEmpasesData, (nextEmpasData, index) => {
        return {
          empasId: empases[index].id,
          isLastEmpas: nextEmpasData["hydra:totalItems"] > 0 ? false : true,
          nextEmpas:
            nextEmpasData["hydra:totalItems"] > 0
              ? nextEmpasData["hydra:member"][0]
              : null,
        };
      });

      const linkedEmpasesData = yield all(
        _.map(empases, (empas, index) => {
          const empasDetail = empasesDetail[index];

          return call(request, COLLECTION_NAME, {
            registeredAt: {
              strictly_after: empas.registeredAt,
              strictly_before: empasDetail.isLastEmpas
                ? moment().format()
                : moment(empasDetail.nextEmpas.registeredAt).format(),
            },
            properties: [
              "id",
              "registeredAt",
              "workerId",
              "worker",
              "managerId",
              "type",
            ],
            empasNumber: empas.empasNumber,
            order: {
              registeredAt: "asc",
            },
            itemsPerPage: 1000,
          });
        })
      );

      _.forEach(linkedEmpasesData, (linkedEmpasData, index) => {
        const linkedEmpases = linkedEmpasData["hydra:member"];

        const workerIds = _.compact(
          _.uniq(_.map(linkedEmpases, (linkedEmpas) => linkedEmpas.workerId))
        );

        const workers = _.map(workerIds, (id) => {
          const currentEmpas = _.find(
            linkedEmpases,
            (linkedEmpas) => linkedEmpas.workerId === id
          );

          return {
            id: currentEmpas.workerId,
            name: currentEmpas.worker.name,
            code: currentEmpas.worker.code,
          };
        });

        empasesDetail[index].linkedEmpases = linkedEmpases;
        empasesDetail[index].workers = workers;
      });

      const tasksData = yield all(
        _.map(empases, (empas, index) => {
          const empasDetail = empasesDetail[index];
          const linkedEmpases = empasDetail.linkedEmpases;

          const workerIds = _.compact(
            _.uniq(_.map(linkedEmpases, (linkedEmpas) => linkedEmpas.workerId))
          );

          return call(request, COLLECTION_TASK_NAME, {
            finishedAt: {
              strictly_after: empas.registeredAt,
              strictly_before: empasDetail.isLastEmpas
                ? moment().format()
                : moment(empasDetail.nextEmpas.registeredAt).format(),
            },
            properties: ["id", "finishedAt", "rowNumber", "workerId"],
            workerId: workerIds,
            ["section.isSpraying"]: true,
            order: {
              rowNumber: "asc",
            },
            isFinished: true,
            itemsPerPage: 100,
          });
        })
      );

      _.forEach(tasksData, (taskData, index) => {
        empasesDetail[index].finishedTasksCount = taskData["hydra:totalItems"];
        empasesDetail[index].tasks = taskData["hydra:member"];
        empasesDetail[index].finishedRows = _.map(
          taskData["hydra:member"],
          (task) => task.rowNumber
        );
      });

      _.forEach(empasesDetail, (empasDetail) => {
        const currentEmpasIndex = _.findIndex(
          empases,
          (empas) => empas.id === empasDetail.empasId
        );
        empases[currentEmpasIndex].empasesDetail = empasDetail;
      });
    }

    yield put(
      actions.loadFromAgroInvestSuccess(empases, data["hydra:totalItems"])
    );
  } catch (error) {
    console.log(error);
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* sync1C({ payload }) {
  try {
    const params = {
      toSend: true,
    };

    const data = yield call(
      request,
      COLLECTION_SYNC_NAME + "/" + payload.syncId,
      params,
      "PATCH"
    );

    console.log(data);

    yield put(actions.sync1CSuccess());
  } catch (error) {
    yield put(actions.sync1CError(error));
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data, actionName } = payload;
  try {
    switch (actionName) {
      case "update":
        yield call(
          request,
          COLLECTION_NAME + "/" + data.id,
          {
            workerId: data.workerId,
            registeredAt: data.registeredAt,
          },
          "PATCH"
        );
        break;
      default:
        break;
    }
    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.SYNC_1C, sync1C),
    takeEvery(actions.SYNC_1C_SUCCESS, loadFromAgroinvest),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
  ]);
}
