import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import { isJSON, request } from '@iso/lib/helpers/utility';
import uuidV4 from 'uuid/v4';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = 'greenhouses'; // change your collection
const COLLECTION_NAME_SECTION = 'sections';
const COLLECTION_NAME_GREENHOUSE_SECTIONS = 'greenhouse_sections';
const COLLECTION_TECHNICAL_WINDOW_SECTIONS = 'technical_window_sections';
const getPage = (state) => state.Greenhouse.page;
const getItemsPerPage = (state) => state.Greenhouse.itemsPerPage;
const getOrderBy = (state) => state.Greenhouse.orderBy;
const getFilters = (state) => state.Greenhouse.filters;
const getFilterParent = (state) => state.Greenhouse.filterParent;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);
    const filterParent = yield select(getFilterParent);
    const params = {};

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;

    if (filterParent.value !== null) {
      params['exists[parentId]'] = filterParent.value;
    }

    const data = yield call(request, COLLECTION_NAME, params);
    yield put(actions.loadFromAgroInvestSuccess(data['hydra:member'], data['hydra:totalItems']));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data } = payload;
  try {
    const params = {
      type: data.type,
      name: data.name,
      description: data.description,
      length: +data.length,
    };

    if (!data.id) {
      yield call(
        request,
        COLLECTION_NAME,
        {
          ...params,
          id: uuidV4(),
        },
        'POST',
      );
    } else {
      yield call(request, `${COLLECTION_NAME}/${data.id}`, params, 'PATCH');
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* getSection(action) {
  try {
    const params = {
      pagination: false,
      isActive: true,
      itemsPerPage: 100,
    };
    const data = yield call(request, COLLECTION_NAME_SECTION, params, 'GET');

    yield put(actions.getSectionSuccess(data['hydra:member']));
  } catch (err) {
    console.log(err);

    yield put(actions.getSectionError());
  }
}

function* createGreenhouseSection(action) {
  try {
    const { selected, greenhouse } = action.payload;

    yield all(
      _.map(selected, (item) => {
        const params = {
          id: uuidV4(),
          greenhouse: `greenhouses/${greenhouse}`,
          section: item['@id'],
          onPause: false,
          minLeadTime: item.minLeadTime ? Number(item.minLeadTime) : 1,
        };
        return call(request, COLLECTION_NAME_GREENHOUSE_SECTIONS, params, 'POST');
      }),
    );
    yield put(actions.createGreenhouseSectionSuccess());
  } catch (err) {
    yield put(actions.createGreenhouseSectionError());
  }
}

function* deleteGreenhouseSection(action) {
  try {
    const { payload } = action;
    const resposeTechnicalWindowSection = yield all(
      _.map(payload, (item) => {
        const params = {
          sectionId: item.sectionId,
          'technicalWindow.greenhouseId': item.greenhouseId,
          properties: ['id'],
        };
        return call(request, COLLECTION_TECHNICAL_WINDOW_SECTIONS, params, 'GET');
      }),
    );
    const technicalWindowSections = _.flatMap(
      resposeTechnicalWindowSection,
      (el) => el['hydra:member'],
    );
    yield all(
      _.map(technicalWindowSections, (item) =>
        call(request, `${COLLECTION_TECHNICAL_WINDOW_SECTIONS}/${item.id}`, null, 'DELETE'),
      ),
    );
    yield all(
      _.map(payload, (item) =>
        call(request, `${COLLECTION_NAME_GREENHOUSE_SECTIONS}/${item.id}`, null, 'DELETE'),
      ),
    );

    yield put(actions.deleteGreenhouseSectionSuccess());
  } catch (err) {
    console.log(err);
    yield put(actions.deleteGreenhouseSectionError());
  }
}

function* createSectionGreenhouse(action) {
  try {
    const { stateSection, greenhouseId } = action.payload;

    const sectionData = {
      id: uuidV4(),
      isActive: true,
      isHarvesting: false,
      isPhenology: false,
      isGuarding: false,
      isSpraying: false,
    };

    _.map(stateSection, (item) => (sectionData[item.name] = item.value));

    const section = yield call(request, COLLECTION_NAME_SECTION, sectionData, 'POST');

    if (_.size(section)) {
      yield call(
        request,
        COLLECTION_NAME_GREENHOUSE_SECTIONS,
        {
          id: uuidV4(),
          section: section['@id'],
          greenhouse: `greenhouses/${greenhouseId}`,
          minLeadTime: 1,
        },
        'POST',
      );

      yield put(actions.saveSectionSuccess());
    }
  } catch (err) {
    console.log(err);
    yield put(actions.saveSectionError());
  }
}

function* changeTime(action) {
  try {
    const data = action.payload;

    yield all(
      _.map(data, (item) =>
        call(
          request,
          `${COLLECTION_NAME_GREENHOUSE_SECTIONS}/${item.id}`,
          {
            minLeadTime: Number(item.minLeadTime),
          },
          'PATCH',
        ),
      ),
    );
  } catch (err) {
    console.log(err);
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.GET_SECTION, getSection),
    takeEvery(actions.CREATE_GREENHOUSE_SECTION, createGreenhouseSection),
    takeEvery(actions.DELETE_GREENHOUSE_SECTION, deleteGreenhouseSection),
    takeEvery(actions.SAVE_NEW_SECTION_GREENHOUSE, createSectionGreenhouse),
    takeEvery(actions.CHANGE_TIME_GREENHOUSE_SECTION, changeTime),
  ]);
}
