import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import { request, cacheRequest } from '@iso/lib/helpers/utility';
import _ from 'lodash';
import moment from 'moment';

export const totalRatingId = 2;
export const durationRatingId = 4;

const COLLECTION_TASK_CHECKS = 'task_checks';
const COLLECTION_GREENHOUSE_SECTIONS = `greenhouse_sections`;
const COLLECTION_RATINGS = 'ratings';

const getGreenhouseId = (state) => state.App.greenhouseId;

const getFilters = (state) => state.HistoricalCheckLists.filters;

function* loadFromAgroinvest() {
  try {
    const filters = yield select(getFilters);

    const params = {
      ...filters,
      registeredAt: {
        after: moment(filters.registeredAt.after).startOf('day'),
        before: moment(filters.registeredAt.before).endOf('day'),
      },
    };

    // получаем все задачи по определенному человеку за период
    const prepareRequest = yield call(request, COLLECTION_TASK_CHECKS, {
      ...params,
      itemsPerPage: 1,
    });
    const totalItems = prepareRequest['hydra:totalItems'];
    const itemsPerPage = 100;
    const numberPages = Math.ceil(totalItems / itemsPerPage);

    const requests = _.range(1, numberPages + 1).map((page) => {
      const requestParam = { ...params, itemsPerPage, page };
      return call(request, COLLECTION_TASK_CHECKS, requestParam);
    });
    const resultDatas = yield all(requests);
    const taskChecks = resultDatas.reduce(
      (acc, data) => {
        return {
          ...acc,
          'hydra:member': [...acc['hydra:member'], ...data['hydra:member']],
          total: data['hydra:totalItems'],
        };
      },
      { 'hydra:member': [], total: 0 }
    );

    const groupedByCheckLists = _.groupBy(
      taskChecks['hydra:member'],
      'controllerTaskId'
    );

    const amounts = _(groupedByCheckLists)
      .toPairs()
      .reduce((acc, [controllerTaskId, checkList]) => {
        return {
          ...acc,
          [controllerTaskId]: {
            value: _.sumBy(checkList, ({ value }) => Number(value)),
          },
        };
      }, {});

    const durations = _(groupedByCheckLists)
      .toPairs()
      .reduce((acc, [controllerTaskId, checkList]) => {
        const finishedAt = moment(checkList[0].task.finishedAt);
        const startedAt = moment(checkList[0].task.startedAt);
        const diff = finishedAt.diff(startedAt, 'seconds');
        return {
          ...acc,
          [controllerTaskId]: {
            value: diff,
          },
        };
      }, {});

    const columns = _.toPairs(groupedByCheckLists).map(
      ([controllerTaskId, checkList]) => {
        return {
          name: controllerTaskId,
          registeredAt: checkList[0].registeredAt,
          rowNumber: checkList[0].task.rowNumber,
        };
      }
    );

    const ratingParams = {
      sectionId: filters['task.sectionId'],
      workerId: filters['task.worker.id'],
    };
    const ratingResponse = yield call(
      request,
      COLLECTION_RATINGS,
      ratingParams
    );

    const rating = ratingResponse['hydra:member'][0]
      ? ratingResponse['hydra:member'][0]['ratingInt']
      : '---';

    const dynamicRating = ratingResponse['hydra:member'][0]
      ? ratingResponse['hydra:member'][0]['dynamicRating']
      : '---';

    const totalRatings = _(groupedByCheckLists)
      .toPairs()
      .reduce((acc, [controllerTaskId, checkList]) => {
        return {
          ...acc,
          [controllerTaskId]: {
            value: checkList[0].task.ratingScore,
          },
        };
      }, {});

    const durationRatings = _(groupedByCheckLists)
      .toPairs()
      .reduce((acc, [controllerTaskId, checkList]) => {
        return {
          ...acc,
          [controllerTaskId]: {
            value: checkList[0].task.ratingTime,
          },
        };
      }, {});

    const customRows = [
      {
        checkMetricId: 1,
        checkMetricName: 'Сумма баллов',
        ...amounts,
        type: 'custom',
      },
      {
        checkMetricId: totalRatingId,
        checkMetricName: 'Оценка качества',
        ...totalRatings,
        type: 'custom',
      },
      {
        checkMetricId: 3,
        checkMetricName: 'Время выполнения задачи (сек.)',
        ...durations,
        type: 'custom',
      },
      {
        checkMetricId: durationRatingId,
        checkMetricName: 'Оценка времени',
        ...durationRatings,
        type: 'custom',
      },
    ];

    const uniqCheckMetric = _.uniqBy(
      taskChecks['hydra:member'],
      'checkMetricId'
    );
    const rows = _(uniqCheckMetric)
      .map(({ checkMetric }) => {
        const { id, name } = checkMetric;
        const checks = _(groupedByCheckLists)
          .toPairs()
          .reduce((acc, [controllerTaskId, checkList]) => {
            const findedCheckMetric = _.find(
              checkList,
              ({ checkMetricId }) => checkMetricId === id
            );
            return { ...acc, [controllerTaskId]: findedCheckMetric };
          }, {});

        return {
          checkMetricId: id,
          checkMetricName: name,
          type: 'checkMetric',
          ...checks,
        };
      })
      .value();
    const unionRows = rows.length > 0 ? [...rows, ...customRows] : [];

    const totalRating = unionRows.find(
      ({ checkMetricId }) => checkMetricId === totalRatingId
    );

    const durationRating = unionRows.find(
      ({ checkMetricId }) => checkMetricId === durationRatingId
    );

    const ratings = columns.map(({ name }) => ({
      sum: totalRating[name].value,
      duration: durationRating[name].value,
    }));
    const averageSum = _.meanBy(ratings, ({ sum }) => Number(sum)).toFixed(2);
    const averageDuration = _.meanBy(ratings, ({ duration }) =>
      Number(duration)
    ).toFixed(2);
    const averageDynamicRating = (averageSum * 0.6 + averageDuration * 0.4).toFixed(2);
    yield put(
      actions.loadFromAgroInvestSuccess({
        checkLists: unionRows,
        columns,
        rating,
        dynamicRating,
        averageSum,
        averageDuration,
        averageDynamicRating,
      })
    );
  } catch (error) {
    console.log(error);
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* sectionsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const params = {
      greenhouseId,
      'section.isQuality': false,
    };

    const data = yield call(request, COLLECTION_GREENHOUSE_SECTIONS, params);

    yield put(
      actions.sectionsLoadFromAgroInvestSuccess(
        data['hydra:member'],
        data['hydra:totalItems']
      )
    );
  } catch (error) {
    yield put(actions.sectionsLoadFromAgroInvestError(error));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(
      actions.SECTIONS_LOAD_FROM_AGROINVEST,
      sectionsLoadFromAgroinvest
    ),
  ]);
}
