import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import { isJSON, request } from '@iso/lib/helpers/utility';
import moment from 'moment';
/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */
const COLLECTION_PEST_INFESTATIONS = 'pest_infestations';
const COLLECTION_INTENSITYS = 'intensities';
const COLLECTION_PESTS = 'pests';
const COLLECTION_ROWS = 'rows';

const shelfs = _.range(1, 31);

const getGreenhouseId = (state) => state.App.greenhouseId;

const getFilters = (state) => state.MonitoringGraph.filters;
const getItemsPerPage = (state) => state.MonitoringGraph.itemsPerPage;
const getPestId = (state) => state.MonitoringGraph.pestId;
const getDateBefore = (state) => state.MonitoringGraph.dateBefore;
const getDateAfter = (state) => state.MonitoringGraph.dateAfter;
const getRowsTotal = (state) => state.MonitoringGraph.rowsTotal;

function* loadFromAgroinvest() {
  try {
    yield put(actions.resetFilters());
    yield all([
      call(rowsLoadFromAgroinvest),
      call(intensitysLoadFromAgroinvest),
      call(pestsLoadFromAgroinvest),
    ]);
    yield call(setFilters);
    yield put(actions.loadFromAgroInvestSuccess());
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* setFilters() {
  const filters = yield select(getFilters);
  const pestId = yield select(getPestId);
  const dateBefore = yield select(getDateBefore);
  const dateAfter = yield select(getDateAfter);
  const greenhouseId = yield select(getGreenhouseId);

  const newFilters = {
    ...filters,
    pestId,
    registeredAt: {
      before: dateBefore,
      after: dateAfter,
    },
    exists: {
      curedAt: false,
    },
    'task.greenhouseId': greenhouseId,
  };

  yield put(actions.setFilters(newFilters, 1));
}

function* rowsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const params = {
      greenhouseId,
      itemsPerPage: 1,
      page: 1,
      order: {
        orderBy: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_ROWS, params);
    const rowsTotal = data['hydra:totalItems'];

    
    yield put(
      actions.rowsLoadFromAgroInvestSuccess(rowsTotal)
    );
  } catch (error) {
    yield put(actions.rowsLoadFromAgroInvestError(error));
  }
}

function* pestsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      order: {
        name: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_PESTS, params);
    yield put(actions.pestsLoadFromAgroInvestSuccess(data['hydra:member']));
  } catch (error) {
    yield put(actions.pestsLoadFromAgroInvestError(error));
  }
}

function* pestInfestationsLoadFromAgroinvest() {
  try {

    const rowsTotal = yield select(getRowsTotal);
    const filters = yield select(getFilters);

    const params = {
      itemsPerPage: 1,
      order: {
        taskId: 'asc',
      },
      properties: [
        'id',
        'shelfNumber',
        'pestId',
        'registeredAt',
        'intensityId',
        'intensity'
      ],
      "properties[task]": ["rowNumber"],
    };

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });
    
    const getStartWeek = (date) => date.utc().isoWeekday(1).startOf('day');
    const yearWeeks = _.range(0, 52).reduce((acc, week) => {
      const key = getStartWeek(moment()).subtract(week, 'weeks').valueOf();
      return { ...acc, [key]: 0 };
    }, {});

    const data = yield call(request, COLLECTION_PEST_INFESTATIONS, params);

    const totalItems = data['hydra:totalItems'];
    const itemsPerPage = 100;
    const numberPages = Math.ceil(totalItems / itemsPerPage);

    const requests = _.range(1, numberPages + 1).map((page) => {
      const requestParam = { ...params, itemsPerPage, page };
      return call(request, COLLECTION_PEST_INFESTATIONS, requestParam);
    });
    const resultDatas = yield all(requests);
    const result = resultDatas.reduce(
      (acc, data) => {
        return {
          ...acc,
          member: [...acc.member, ...data['hydra:member']],
          total: data['hydra:totalItems'],
        };
      },
      { member: [], total: 0 }
    );
    const groupedByIntensityName = _.groupBy(
      result.member,
      'intensity.name'
    );
    const pestInfestationUnionCount = { ...yearWeeks };
    const pestIntensitys = Object.entries(groupedByIntensityName).map(
      ([intensityName, pestInfestations]) => {
        // подсчет количества совпадений в неделю
        const copyYearWeeks = { ...yearWeeks };
        let memo = {};
        pestInfestations.forEach(({ registeredAt, pestId, shelfNumber, task}) => {
          // проверка что в данной неделе ячейка уже учтена
          const startWeek = getStartWeek(moment(registeredAt)).valueOf();
          const id = `${startWeek}${pestId}${task.rowNumber}`;
          if (memo[id] && memo[id].includes(shelfNumber)) {
            return;
          }
          memo = {
            ...memo,
            [id]: [...(memo[id] || []), shelfNumber],
          };
          copyYearWeeks[startWeek]++;
          pestInfestationUnionCount[startWeek]++;
        });
        const data = Object.entries(copyYearWeeks).map(([key, value]) => ({
          x: Number(key),
          y: value,
        }));
        return { name: intensityName, data };
      }
    );
    const healthyIntensity = {
      name: 'Здоровые',
      data: Object.entries(pestInfestationUnionCount).map(([x, y]) => ({
        x: Number(x),
        y: shelfs.length * rowsTotal - y,
      })),
    };
    const pestInfestations = [healthyIntensity,...pestIntensitys];

    yield put(
      actions.pestInfestationsLoadFromAgroInvestSuccess(pestInfestations)
    );
  } catch (error) {
    yield put(actions.pestInfestationsLoadFromAgroInvestError(error));
  }
}

function* intensitysLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      order: {
        orderBy: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_INTENSITYS, params);

    yield put(
      actions.intensitysLoadFromAgroInvestSuccess(data['hydra:member'])
    );
  } catch (error) {
    yield put(actions.intensitysLoadFromAgroInvestError(error));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(
      actions.PEST_INFESTATIONS_LOAD_FROM_AGROINVEST,
      pestInfestationsLoadFromAgroinvest
    ),
    takeEvery(actions.SET_PEST_ID, setFilters),
    takeEvery(actions.SET_FILTERS, pestInfestationsLoadFromAgroinvest),
  ]);
}
