import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import { isJSON, request } from '@iso/lib/helpers/utility';
/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */
const COLLECTION_ROWS = 'rows';
const COLLECTION_PEST_INFESTATIONS = 'pest_infestations';

const COLLECTION_PESTS = 'pests';
const COLLECTION_INTENSITYS = 'intensities';
const COLLECTION_WORKERS = `workers`;
const shelfs = _.range(1, 31);

const getGreenhouseId = (state) => state.App.greenhouseId;

const getRows = (state) => state.MonitoringInfo.rows;
const getPestInfestations = (state) => state.MonitoringInfo.pestInfestations;
const getFilters = (state) => state.MonitoringInfo.filters;
const getWorkerId = (state) => state.MonitoringInfo.workerId;
const getPestId = (state) => state.MonitoringInfo.pestId;
const getDateBefore = (state) => state.MonitoringInfo.dateBefore;
const getDateAfter = (state) => state.MonitoringInfo.dateAfter;
const getItemsPerPage = (state) => state.MonitoringInfo.itemsPerPage;
const getRowStart = (state) => state.MonitoringInfo.rowStart;
const getRowEnd = (state) => state.MonitoringInfo.rowEnd;

function* loadFromAgroinvest() {
  try {
    yield all([
      call(rowsLoadFromAgroinvest),
      call(pestsLoadFromAgroinvest),
      call(intensitysLoadFromAgroinvest),
      call(workersLoadFromAgroinvest),
    ]);
    yield call(setFilters);
    yield put(actions.loadFromAgroInvestSuccess());
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* rowsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      order: {
        orderBy: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_ROWS, params);
    const rows = data['hydra:member'];

    const maxRow = _.maxBy(rows, 'orderBy');

    const filled = _.range(1, maxRow.orderBy + 1).map((orderIndex) => {
      const row = _.find(rows, { orderBy: orderIndex }) || { rowNumber: '---' };
      const cells = shelfs.reduce((acc, cell) => {
        return {
          ...acc,
          [cell]: [
            {
              rowNumber: row.rowNumber,
              pest: { name: null },
              intensity: {
                name: null,
                orderBy: 0,
              },
              shelfNumber: cell,
            },
          ],
        };
      }, {});
      return {
        rowNumber: row.rowNumber,
        isBlockEnd: row.isBlockEnd,
        orderBy: orderIndex,
        ...cells,
      };
    });
    yield put(
      actions.rowsLoadFromAgroInvestSuccess(filled, data['hydra:totalItems'])
    );
  } catch (error) {
    yield put(actions.rowsLoadFromAgroInvestError(error));
  }
}

function* pestInfestationsLoadFromAgroinvest() {
  try {
    yield put(actions.pestInfestationsLoadFromAgroInvest());
    const filters = yield select(getFilters);

    const params = {
      itemsPerPage: 1,
      order: {
        taskId: 'asc',
      },
      properties: [
        'id',
        'shelfNumber',
        'intensityId',
        'intensity',
        'pestId',
        'pest',
      ],
      "properties[task]": ["rowNumber"],
    };

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    const data = yield call(request, COLLECTION_PEST_INFESTATIONS, params);

    const totalItems = data['hydra:totalItems'];
    const itemsPerPage = 100;
    const numberPages = Math.ceil(totalItems / itemsPerPage);

    const requests = _.range(1, numberPages + 1).map((page) => {
      const requestParam = { ...params, itemsPerPage, page };
      return call(request, COLLECTION_PEST_INFESTATIONS, requestParam);
    });
    const resultDatas = yield all(requests);
    const result = resultDatas.reduce(
      (acc, data) => {
        return {
          ...acc,
          member: [...acc.member, ...data['hydra:member']],
          total: data['hydra:totalItems'],
        };
      },
      { member: [], total: 0 }
    );
    yield put(
      actions.pestInfestationsLoadFromAgroInvestSuccess(
        result.member,
        result.total
      )
    );
    yield call(setFillRows);
  } catch (error) {
    yield put(actions.pestInfestationsLoadFromAgroInvestError(error));
  }
}

function* setFillRows() {
  const rows = yield select(getRows);
  const rowStart = yield select(getRowStart);
  const rowEnd = yield select(getRowEnd);
  const pestInfestations = yield select(getPestInfestations);

  const minStart = rowStart < 1 ? 1 : Number(rowStart);
  const maxEnd =
    rows.length <= Number(rowEnd)
      ? rows.length
      : _.findIndex(rows, { rowNumber: Number(rowEnd) + 1 });

  const start = _.findIndex(rows, { rowNumber: minStart });
  const minStartEven = start % 2 === 0 ? start : start -1;
  const rowsCopy = [...rows].slice(minStartEven, maxEnd);
  const groupByRow = _.groupBy(pestInfestations, 'task.rowNumber');

  const groupByRowShelf = {};
  for (const key in groupByRow) {
    groupByRowShelf[key] = _.groupBy(groupByRow[key], 'shelfNumber');
  }

  Object.keys(groupByRowShelf).forEach((rowNumber) => {
    const index = _.findIndex(rowsCopy, (row) => {
      return row.rowNumber === Number(rowNumber);
    });
    rowsCopy[index] = { ...rowsCopy[index], ...groupByRowShelf[rowNumber] };
  });
  yield put(actions.setFillRows(rowsCopy));
}

function* pestsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      order: {
        name: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_PESTS, params);

    yield put(
      actions.pestsLoadFromAgroInvestSuccess(
        data['hydra:member'],
        data['hydra:totalItems']
      )
    );
  } catch (error) {
    yield put(actions.pestsLoadFromAgroInvestError(error));
  }
}

function* intensitysLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      order: {
        orderBy: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_INTENSITYS, params);

    yield put(
      actions.intensitysLoadFromAgroInvestSuccess(
        data['hydra:member'],
        data['hydra:totalItems']
      )
    );
  } catch (error) {
    yield put(actions.intensitysLoadFromAgroInvestError(error));
  }
}

function* workersLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const itemsPerPage = yield select(getItemsPerPage);

    const params = {
      greenhouseId,
      itemsPerPage,
      properties: ['id', 'name'],
      order: {
        orderBy: 'asc',
      },
    };

    const data = yield call(request, COLLECTION_WORKERS, params);

    yield put(
      actions.workersLoadFromAgroInvestSuccess(
        data['hydra:member'],
        data['hydra:totalItems']
      )
    );
  } catch (error) {
    yield put(actions.workersLoadFromAgroInvestError(error));
  }
}
function* storeIntoAgroinvest({ payload }) {
  const { data, actionName } = payload;
  try {
    switch (actionName) {
      case 'update':
        yield all(
          data.map(({ intensityId, id, curedAt }) => {
            const params = {
              intensityId,
              curedAt,
            };
            return call(
              request,
              COLLECTION_PEST_INFESTATIONS + '/' + id,
              params,
              'PATCH'
            );
          })
        );
        break;
      default:
        break;
    }
    yield call(pestInfestationsLoadFromAgroinvest);
  } catch (error) {
    yield put(actions.saveIntoAgroInvestError(error));
  }
}
function* setFilters() {
  const filters = yield select(getFilters);
  const workerId = yield select(getWorkerId);
  const pestId = yield select(getPestId);
  const dateBefore = yield select(getDateBefore);
  const dateAfter = yield select(getDateAfter);
  const greenhouseId = yield select(getGreenhouseId);

  const newFilters = {
    ...filters,
    workerId,
    pestId,
    registeredAt: {
      before: dateBefore,
      after: dateAfter,
    },
    exists: {
      curedAt: false,
    },
    'task.greenhouseId': greenhouseId,
  };

  yield put(actions.setFilters(newFilters, 1));
  yield call(pestInfestationsLoadFromAgroinvest);
}
export default function* rootSaga() {
  yield all([
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.REFRESH, loadFromAgroinvest),
    takeEvery(actions.RESET_FILTERS, pestInfestationsLoadFromAgroinvest),
    takeEvery(actions.SET_WORKER_ID, setFilters),
    takeEvery(actions.SET_DATE_BEFORE, setFilters),
    takeEvery(actions.SET_DATE_AFTER, setFilters),
    takeEvery(actions.SET_ROW_START, setFilters),
    takeEvery(actions.SET_ROW_END, setFilters),
    takeEvery(actions.SET_PEST_ID, setFilters),
  ]);
}
