import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import moment from 'moment';
import { isJSON, request } from '@iso/lib/helpers/utility';
import uuidV4 from 'uuid/v4';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = 'collectables'; // change your collection
const COLLECTION_VALVES = 'valves';
const COLLECTION_HISTORICAL_VALVES = 'historical_valves';

const getPage = state => state.Phenology.page;
const getItemsPerPage = state => state.Phenology.itemsPerPage;
const getOrderBy = state => state.Phenology.orderBy;
const getFilters = state => state.Phenology.filters;
const getGreenhouseId = state => state.App.greenhouseId;
const getValves = state => state.Phenology.valves;
const selectHistoricalValves = state => state.Phenology.historicalValves;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);
    const greenhouseId = yield select(getGreenhouseId);

    const params = {}

    Object.keys(filters).flatMap(key => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if(_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          })
        } else {
          params[key] = _.isObject(filters[key]) ? _.merge(params[key], filters[key]) : filters[key];
        }
      }
    });

    console.log(params);

    if(params && params.category && params.category === 'common') {
      params.category = '';
    }

    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;

    const data = yield call(request, COLLECTION_NAME, params);

    const valves = yield call(request, COLLECTION_VALVES, {
      itemsPerPage: 36,
      greenhouseId,
    });

    yield put(actions.loadFromAgroInvestSuccess(data['hydra:member'], data['hydra:totalItems'], valves['hydra:member']));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* getHistoricalValves() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const { pagination, filters } = yield select(selectHistoricalValves);

    const params = {
      'valve.greenhouseId': greenhouseId,
      itemsPerPage: pagination.itemsPerPage,
      page: pagination.page,
    };

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    const response = yield call(request, COLLECTION_HISTORICAL_VALVES, params);

    yield put(actions.getHistoricalValvesSuccess({
      items: response['hydra:member'],
      totalItems: response['hydra:totalItems'],
    }));
  } catch (e) {
    yield put(actions.getHistoricalValvesFail());
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data } = payload;
  try {
    const params = {
      type: data.type,
      name: data.name,
      isActive: data.isActive,
      category: data.category,
      parent: data.parentId  ? 'collectables/' + data.parentId : null,
    }

    console.log(data, params)

    if(!data.id) {
      yield call(request, COLLECTION_NAME, {
        ...params,
        id: uuidV4(),
      }, 'POST');
    } else {
      yield call(request, COLLECTION_NAME + '/' + data.id, params, 'PATCH');
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* updateReferenceBook({ payload }) {
  try {
    const valves = yield select(getValves);
    const valve = valves.find(valve => valve.id === payload.id);

    yield call(request, COLLECTION_VALVES + '/' + payload.id, {
      volume: valve.volume,
      electricConduction: valve.electricConduction,
      potentialHydrogen: valve.potentialHydrogen
    }, 'PATCH');

    yield put(actions.updateReferenceBookSuccess());
  } catch (e) {
    console.log(e);

    yield put(actions.updateReferenceBookFail());
  }
}

function* updateHistoricalValves({ payload }) {
  try {
    const { items: parameters } = yield select(selectHistoricalValves);
    const parameter = parameters.find(({ id }) => id === payload.id);

    yield call(request, COLLECTION_HISTORICAL_VALVES + '/' + payload.id, {
      volume: parameter.volume,
      electricConduction: parameter.electricConduction,
      potentialHydrogen: parameter.potentialHydrogen
    }, 'PATCH');

    yield put(actions.updateHistoricalValvesSuccess());
  } catch (e) {
    console.log(e);

    yield put(actions.updateHistoricalValvesFail());
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.SET_REFERENCE_BOOK_TABLE_VALUE, updateReferenceBook),

    takeEvery(actions.GET_HISTORICAL_VALVES_REQUEST, getHistoricalValves),
    takeEvery(actions.UPDATE_HISTORICAL_VALVES_REQUEST, updateHistoricalValves),
    takeEvery(actions.SET_HISTORICAL_VALVES_FILTERS, getHistoricalValves)
  ]);
}
