import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import { request } from '@iso/lib/helpers/utility';
import uuidV4 from 'uuid/v4';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = 'rows'; // change your collection
const COLLECTION_GREENHOUSE_NAME = 'greenhouses';
const COLLECTION_NAME_PLANS = 'plans'

const getPage = state => state.Plan.page;
const getItemsPerPage = state => state.Plan.itemsPerPage;
const getOrderBy = state => state.Plan.orderBy;
const getGreenhouseId = state => state.App.greenhouseId;
const getSectionId = state => state.Plan.sectionId;
const getDay = state => state.Plan.day;
const getDirection = state => state.Plan.direction;
const getShift = state => state.Plan.shift;
const getExclusive = state => state.Plan.exclusive;
const getTemplateId = state => state.Plan.templateId;
const getKeepExclusive = state => state.Plan.keepExclusive;
const getKeepShift = state => state.Plan.keepShift;
const getKeepDirection = state => state.Plan.keepDirection;
const getKeepTemplate = state => state.Plan.keepTemplate;
const getSelectedRows = state => state.Plan.selectedRows;
const getPass = state => state.Plan.pass;
const getKeepDay = state => state.Plan.keepDay;
const getPlanRowFrom = state => state.Plan.planRowFrom;
const getPlanRowTo = state => state.Plan.planRowTo;

const getTemplateEditingGreenhouseId = state => state.Plan.templateEditing.greenhouseId;
const getTemplateEditingName = state => state.Plan.templateEditing.templateEdit.name;
const getTemplateEditingJson = state => state.Plan.templateEditing.templateEdit.template;
const getTemplateEditingId = state => state.Plan.templateEditing.templateEdit.id;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const greenhouseId = yield select(getGreenhouseId);
    const sectionId = yield select(getSectionId);
    const pass = yield select(getPass);
    const planRowFrom = yield select(getPlanRowFrom);
    const planRowTo = yield select(getPlanRowTo);

    if (!sectionId) {
      yield put(actions.disableLoading());
      return;
    }

    const params = {}

    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;
    params.greenhouseId = greenhouseId;
    params.rowNumber = {
      gte: planRowFrom,
      lte: planRowTo,
    };

    const data = yield call(request, COLLECTION_NAME, params);

    const plansParams = {
      sectionId,
      pass: pass,
      'row.greenhouseId': greenhouseId,   
      'row.rowNumber': {
        gte: planRowFrom,
        lte: planRowTo,
      },
      itemsPerPage: data['hydra:totalItems']
    }

    const plans = yield call(request, 'plans', plansParams);

    _.forEach(data['hydra:member'], row => {
      _.forEach(plans['hydra:member'], plan => {
        if (row.id === plan.rowId) row.plan = plan
      })
    })

    const templatesParams = {
      greenhouseId,
    }

    const templates = yield call(request, 'plan_templates', templatesParams)

    yield put(actions.loadFromAgroInvestSuccess(data['hydra:member'],
      data['hydra:totalItems'],
      templates['hydra:member']));

  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* changeTaskStatus({ payload }) {
  try {
    const selectedRows = yield select(getSelectedRows);
    const sectionId = yield select(getSectionId);
    const day = yield select(getDay);
    const direction = yield select(getDirection);
    const shift = yield select(getShift);
    const exclusive = yield select(getExclusive);
    const keepExclusive = yield select(getKeepExclusive);
    const keepShift = yield select(getKeepShift);
    const keepDirection = yield select(getKeepDirection);
    const keepTemplate = yield select(getKeepTemplate);
    const templateId = yield select(getTemplateId);
    const pass = yield select(getPass);
    const keepDay = yield select(getKeepDay);

    const rows = []
    const plansForUpdate = [];
    const plansForCreate = [];

    _.each(Object.keys(selectedRows), (section) => {
      _.each(selectedRows[section], (row) => {
        if (_.isNull(row.rowNumber)) {
          return;
        }

        rows.push(row);
      });
    });

    _.each(rows, row => {
      if (row.plan) plansForUpdate.push(row)
      else plansForCreate.push(row)
    });

    const createParams = _.map(plansForCreate, row => {

      const params = {
        id: uuidV4(),
        row: 'rows/' + row.id,
        section: 'sections/' + sectionId,
        dow: day,
        isActive: payload,
        direction,
        shift,
        pass,
        exclusive,
      }

      if (!keepTemplate && templateId) params.planTemplate = 'plan_templates/' + templateId

      return params
    })

    const updateParams = _.map(plansForUpdate, row => {

      const params = {
        planId: row.plan.id,
        row: 'rows/' + row.id,
        section: 'sections/' + sectionId,
        isActive: payload,
        direction,
      }

      if (!keepExclusive) params.exclusive = exclusive
      if (!keepShift) params.shift = shift
      if (!keepDay) params.dow = day
      if (!keepDirection) params.direction = direction
      if (!keepTemplate && templateId) params.planTemplate = 'plan_templates/' + templateId

      return params
    })

    yield put(actions.setIsLoading());

    yield all(updateParams.map((params) => call(request, `plans/${params.planId}`, params, 'PATCH')));
    yield all(createParams.map((params) => call(request, `plans`, params, 'POST')));

    yield put(actions.loadFromAgroInvest(sectionId));
  } catch (error) {
    console.log(error);
    yield put(actions.tasksError(error));
  }
}

function* deletePlan() {
  try {
    const selectedRows = yield select(getSelectedRows);
    const sectionId = yield select(getSectionId);

    const plans = [];

    _.each(Object.keys(selectedRows), (section) => {
      _.each(selectedRows[section], (row) => {
        if (_.isNull(row.rowNumber) || _.isNull(row.plan)) {
          return;
        }

        plans.push(row.plan);
      });
    });

    yield put(actions.setIsLoading());

    yield all(plans.map((plan) => call(request, `plans/${plan.id}`, {}, 'DELETE')));

    yield put(actions.loadFromAgroInvest(sectionId));
  } catch (error) {
    console.log(error);
    yield put(actions.tasksError(error));
  }
}

function* loadTemplatesFromAgroInvest() {
  try {
    const greenhouseId = yield select(getTemplateEditingGreenhouseId)

    if (!greenhouseId) return;

    const params = {}

    params.greenhouseId = [greenhouseId];

    params.type = ['group'];
    params.order = {
      name: 'asc',
    };

    const data = yield call(request, 'plan_templates', params);

    yield put(actions.loadTemplatesFromAgroInvestSuccess(data['hydra:member']));

  } catch (error) {
    yield put(actions.loadTemplatesFromAgroInvestError(error));
  }
}

function* saveTemplateIntoAgroInvest() {
  try {
    const greenhouseId = yield select(getTemplateEditingGreenhouseId)
    const name = yield select(getTemplateEditingName)
    const template = yield select(getTemplateEditingJson)
    const id = yield select(getTemplateEditingId)

    const params = {
      greenhouseId,
      name,
      template: JSON.parse(template)
    }

    if (id) {
      yield call(request, `plan_templates/${id}`, params, 'PATCH');
    } else {
      params.id = uuidV4()
      yield call(request, `plan_templates`, params, 'POST');
      yield put(actions.clearTemplateEdit())
    }

    yield put(actions.saveTemplateIntoAgroInvestSuccess());
    yield put(actions.loadTemplatesFromAgroInvest())
    yield put(actions.loadFromAgroInvest())

  } catch (error) {
    yield put(actions.saveTemplateIntoAgroInvestError(error));
  }
}

function* getSectionsGreenhouse(action) {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const greenhouse = yield call(request, COLLECTION_GREENHOUSE_NAME + '/' + greenhouseId, {}, 'GET')

    yield put(actions.getSectionsGreenhouseSuccess(greenhouse.greenhouseSections))

  } catch(err){
    console.log(err)
    yield put(actions.getSectionsGreenhouseFailure())
  }
}

function* getPlanId(action) {
  try {
    const { planId } = action.payload

    const plans = yield call(request, COLLECTION_NAME_PLANS + '/' + planId, {}, "GET")
    
    yield put(actions.getIdSuccess(plans))
  } catch(err) {
    console.log(err)
    yield put(actions.getIdFailure())
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGE, loadFromAgroinvest),
    takeEvery(actions.SET_SECTION, loadFromAgroinvest),
    takeEvery(actions.SET_PASS, loadFromAgroinvest),
    takeEvery(actions.SET_ROWS, loadFromAgroinvest),
    takeEvery(actions.ENABLE_ROWS, changeTaskStatus),
    takeEvery(actions.DISABLE_ROWS, changeTaskStatus),
    takeEvery(actions.DELETE_ROWS, deletePlan),
    takeEvery(actions.LOAD_TEMPLATES_FROM_AGROINVEST, loadTemplatesFromAgroInvest),
    takeEvery(actions.SAVE_TEMPLATE_INTO_AGROINVEST, saveTemplateIntoAgroInvest),
    takeEvery(actions.SECTIONS_GREENHOUSE, getSectionsGreenhouse),
    takeEvery(actions.GET_ID, getPlanId)
  ]);
}
