import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import { isJSON, request } from '@iso/lib/helpers/utility';
import uuidV4 from 'uuid/v4';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_PESTS = 'pests'; // change your collection
const COLLECTION_INTENSITY = 'intensities'; // change your collection
const COLLECTION_INTENSITY_DESCRIPTIONS = 'intensity_descriptions';

const getPage = (state) => state.PlantProtection.page;
const getItemsPerPage = (state) => state.PlantProtection.itemsPerPage;
const getOrderBy = (state) => state.PlantProtection.orderBy;
const getFilters = (state) => state.PlantProtection.filters;
const getIntensities = (state) => state.PlantProtection.intensities;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);

    const params = {};

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;

    const data = yield call(request, COLLECTION_PESTS, params);
    const intensities = yield call(request, COLLECTION_INTENSITY, params);

    yield put(
      actions.loadFromAgroInvestSuccess(
        data['hydra:member'],
        data['hydra:totalItems'],
        intensities['hydra:member']
      )
    );
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadIntensityDescriptionsFromAgroinvest() {
  try {
    const params = {
      itemsPerPage: 200,
      order: {
        pestId: 'asc',
      },
    };
    const intensity_descriptions = yield call(
      request,
      COLLECTION_INTENSITY_DESCRIPTIONS,
      params
    );
    const grouped = _.groupBy(intensity_descriptions['hydra:member'], 'pestId');
    const intensityDescriptions = Object.entries(grouped).map(
      ([key, value]) => {
        return {
          pest: _.find(value, (el) => el.pest.id === key).pest,
          intensities: value,
        };
      }
    );
    yield put(
      actions.intensityDesriptionsLoadFromAgroInvestSuccess(
        intensityDescriptions
      )
    );
  } catch (error) {
    yield put(actions.intensityDesriptionsLoadFromAgroInvestError(error));
  }
}

function* storeIntoAgroinvestIntensityDescriptions({ payload }) {
  const { data } = payload;
  try {
    const response = yield call(request, COLLECTION_INTENSITY_DESCRIPTIONS, {
      id: data.id,
    });
    if (response['hydra:totalItems'] === 0) {
      yield call(
        request,
        COLLECTION_INTENSITY_DESCRIPTIONS,
        {
          pest: data.pest['@id'],
          intensity: data.intensity['@id'],
          description: data.description,
        },
        'POST'
      );
    } else {
      yield call(
        request,
        COLLECTION_INTENSITY_DESCRIPTIONS + '/' + data.id,
        { description: data.description },
        'PATCH'
      );
    }

    yield put({ type: actions.INTENSITY_DESCRIPTIONS_LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* addIntensityDescriptions({ payload }) {
  const { data } = payload;

  try {
    const intensities = yield select(getIntensities);

    const response = yield call(request, COLLECTION_INTENSITY_DESCRIPTIONS, {
      pestId: data,
    });

    if (response['hydra:totalItems'] === 0) {
      yield all(
        intensities.map((intensity) => {
          return call(
            request,
            COLLECTION_INTENSITY_DESCRIPTIONS,
            {
              pest: `/${COLLECTION_PESTS}/${data}`,
              intensity: intensity['@id'],
              description: intensity.name,
            },
            'POST'
          );
        })
      );
    }

    yield put({ type: actions.INTENSITY_DESCRIPTIONS_LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data } = payload;
  try {
    const params = {
      type: data.type,
      name: data.name,
      isActive: data.isActive,
      parent: data.parentId ? 'pests/' + data.parentId : null,
    };

    if (!data.id) {
      const pest = yield call(
        request,
        COLLECTION_PESTS,
        {
          ...params,
          id: uuidV4(),
        },
        'POST'
      );
      yield call(addIntensityDescriptions, { payload: { data: pest.id } })
    } else {
      yield call(request, COLLECTION_PESTS + '/' + data.id, params, 'PATCH');
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* deletePest({ payload }) {
  try {
    yield call(request, `pests/${payload.id}`, {}, 'DELETE');
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* storeIntoAgroinvestIntensity({ payload }) {
  const { data } = payload;
  try {
    const params = {
      name: data.name,
      color: data.color,
      orderBy: +data.orderBy,
    };

    if (!data.id) {
      yield call(
        request,
        COLLECTION_INTENSITY,
        {
          ...params,
          id: uuidV4(),
        },
        'POST'
      );
    } else {
      yield call(
        request,
        COLLECTION_INTENSITY + '/' + data.id,
        params,
        'PATCH'
      );
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* deleteIntensity({ payload }) {
  try {
    yield call(request, COLLECTION_INTENSITY + '/' + payload.id, {}, 'DELETE');

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(
      actions.SAVE_INTO_AGROINVEST_INTENSITY,
      storeIntoAgroinvestIntensity
    ),
    takeEvery(actions.DELETE_PEST, deletePest),
    takeEvery(actions.DELETE_INTENSITY, deleteIntensity),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(
      actions.INTENSITY_DESCRIPTIONS_LOAD_FROM_AGROINVEST,
      loadIntensityDescriptionsFromAgroinvest
    ),
    takeEvery(
      actions.SAVE_INTO_AGROINVEST_INTENSITY_DESCRIPTIONS,
      storeIntoAgroinvestIntensityDescriptions
    ),
    takeEvery(actions.ADD_INTENSITY_DESCRIPTIONS, addIntensityDescriptions),
  ]);
}
