import { all, takeEvery, put, call, select } from "redux-saga/effects";
import actions from "./actions";
import _ from "lodash";
import { isJSON, request } from "@iso/lib/helpers/utility";
import uuidV4 from "uuid/v4";
import { Notification } from 'antd';
/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_CHECK_METRICS = "check_metrics"; // change your collection
const COLLECTION_GREENHOUSE_SECTIONS = "greenhouse_sections"; // change your collection
const COLLECTION_QUALITY_LIMITS = "check_metric_quality_limits";
const getFilters = (state) => state.QualityControl.filters;
const getGreenhouseId = (state) => state.App.greenhouseId;
const getGreenhouses = (state) => state.App.greenhouses;

function* loadFromAgroinvest() {
  try {
    const filters = yield select(getFilters);

    const params = {
      itemsPerPage: 1000,
    };

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      } else if (!_.isNil(filters[key])) {
        params[key] = filters[key];
      }
    });

    const data = yield call(request, COLLECTION_CHECK_METRICS, params);

    yield put(
      actions.loadFromAgroInvestSuccess(
        data["hydra:member"],
        data["hydra:totalItems"]
      )
    );
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* sectionsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const params = {
      greenhouseId,
      ["section.isQuality"]: false,
      ["section.hasQualityCheck"]: true,
    };

    const data = yield call(request, COLLECTION_GREENHOUSE_SECTIONS, params);

    yield put(actions.sectionsLoadFromAgroInvestSuccess(data["hydra:member"]));
  } catch (error) {
    yield put(actions.sectionsLoadFromAgroInvestError(error));
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data } = payload;

  try {
    const greenhouses = yield select(getGreenhouses);
    const greenhouseId = yield select(getGreenhouseId);
    const currentGreenhouseDescription = _.get(
      _.find(greenhouses, (g) => g.id === greenhouseId),
      "description",
      ""
    );
    const categoryMatch = currentGreenhouseDescription.match(/\(\S+?\)/);

    const category = categoryMatch
      ? categoryMatch[0].slice(1, categoryMatch[0].length - 1)
      : null;

    const params = {
      name: data.name,
      section: "sections/" + data.sectionId,
      orderBy: 0,
    };

    params.category = category;

    if (!data.id) {
      const checkMetricId = uuidV4();
      yield call(
        request,
        COLLECTION_CHECK_METRICS,
        { id: checkMetricId, ...params },
        "POST"
      );
      const qualityLimits = [
        { mark: "5", minQuality: "0", maxQuality: "2" },
        { mark: "4", minQuality: "3", maxQuality: "6" },
        { mark: "3", minQuality: "7", maxQuality: "8" },
        { mark: "2", minQuality: "9", maxQuality: "10" },
        { mark: "1", minQuality: "11", maxQuality: "12" },
      ];
      const limitRequests = qualityLimits.map((qualityLimit) =>
        call(
          request,
          COLLECTION_QUALITY_LIMITS,
          {
            id: uuidV4(),
            checkMetric: `check_metrics/${checkMetricId}`,
            ...qualityLimit,
          },
          "POST"
        )
      );
      yield all(limitRequests);
    } else {
      yield call(
        request,
        COLLECTION_CHECK_METRICS + "/" + data.id,
        params,
        "PATCH"
      );
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    yield call(Notification.error, {
      message: error,
      duration: 5,
    });
    console.log(error);
  }
}

function* deleteCheck({ payload }) {
  try {
    const response = yield call(
      request,
      COLLECTION_CHECK_METRICS + "/" + payload.id,
      {},
      "DELETE"
    );
      if(response["@type"] === "hydra:Error")
      {
        throw response["hydra:description"]
      }
    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    yield call(Notification.error, {
      message: error,
      duration: 5,
    });
    console.error(error);
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.DELETE_CHECK, deleteCheck),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(
      actions.SECTIONS_LOAD_FROM_AGROINVEST,
      sectionsLoadFromAgroinvest
    ),
  ]);
}
