import {
  all,
  takeEvery,
  put,
  call,
  select,
  takeLatest,
} from "redux-saga/effects";
import actions from "./actions";
import _ from "lodash";
import { request } from "@iso/lib/helpers/utility";

import moment from "../../library/helpers/moment";

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_TASKS = "tasks"; // change your collection
const COLLECTION_WORKERS = "workers"; // change your collection
const COLLECTION_GREENHOUSE_NAME = "greenhouses";
const COLLECTION_SECTIONS = "sections";
// const COLLECTION_GREENHOUSE_SECTIONS_NAME = 'greenhouse_sections'

const getSectionId = (state) => state.Section.sectionId;
const getPass = (state) => state.Section.pass;
const getItemsPerPage = (state) => state.Section.itemsPerPage;
const getTotalRows = (state) => state.Section.totalRows;
const getSelectedTasks = (state) => state.Section.selectedTasks;
const getSelectedWorkers = (state) => state.Section.selectedWorkers;
const getShift = (state) => state.Section.shift;
const getDirection = (state) => state.Section.direction;
const getDay = (state) => state.Section.day;
const getPlanDay = (state) => state.Section.planDay;
const getKeepPlanDay = (state) => state.Section.keepPlanDay;
const getKeepOrderBy = (state) => state.Section.keepOrderBy;
const getKeepWorkerId = (state) => state.Section.keepWorkerId;
const getKeepExclusive = (state) => state.Section.keepExclusive;
const getExclusive = (state) => state.Section.exclusive;
const getGreenhouseId = (state) => state.App.greenhouseId;
const getRowFrom = (state) => state.Section.rowFrom;
const getRowTo = (state) => state.Section.rowTo;
const getTaskId = (state) => state.Section.task.id;
const getTaskData = (state) => state.Section.task;
const getTasks = (state) => state.Section.tasks;

function* loadFromAgroinvest(props) {
  const {
    payload: { currentTaskId },
  } = props;

  try {
    const itemsPerPage = yield select(getItemsPerPage);
    const sectionId = yield select(getSectionId);
    const pass = yield select(getPass);
    const day = yield select(getDay);
    const greenhouseId = yield select(getGreenhouseId);
    const totalRows = yield select(getTotalRows);
    const rowFrom = yield select(getRowFrom);
    const rowTo = yield select(getRowTo);

    const params = {
      itemsPerPage: itemsPerPage,
      sectionId: sectionId,
      pass: pass,
      greenhouseId: greenhouseId,
      rowNumber: {
        gte: rowFrom,
        lte: rowTo,
      },
      order: {
        rowNumber: "asc",
      },
      properties: [
        "id",
        "name",
        "rowNumber",
        "workerId",
        "workerName",
        "startAt",
        "finishAt",
        "startedAt",
        "finishedAt",
        "lockedTill",
        "isFinished",
        "orderBy",
        "day",
        "shift",
        "sectionName",
        "direction",
        "greenhouseId",
        "isActive",
        "exclusive",
        "inProgress",
        "rowOrderBy",
      ],
      startAt: {
        before: moment(day).utc().endOf("isoWeek").format(),
        after: moment(day).utc().startOf("isoWeek").format(),
      },
      groups: [
        'section'
      ],
    };

    let workers = yield call(request, COLLECTION_WORKERS, {
      // spreadTill: {
      //   after: moment().subtract(30, "days").format("YYYY-MM-DDTHH:mm:ss"),
      // },
      // exists: {
      //   spreadTill: true,
      // },
      properties: ["id", "name", "code", "notes", "spreadTill"],
      greenhouseId,
      isActive: true,
      order: {
        name: "asc",
      },
      itemsPerPage: 500,
      groups: [
        'section'
      ],
    });

    let tasks = null;
    let totalItems = 0;

    let onlineWorkers = [];
    let offlineWorkers = [];

    workers["hydra:member"].forEach((worker) => {
      if (worker.spreadTill > moment().format("YYYY-MM-DDTHH:mm:ss")) {
        onlineWorkers = [...onlineWorkers, worker];
      } else {
        offlineWorkers = [...offlineWorkers, worker];
      }
    });

    const sortByAlphabet = (array, param) => {
      return array.sort(function (a, b) {
        if (a[`${param}`] > b[`${param}`]) {
          return 1;
        }
        if (a[`${param}`] < b[`${param}`]) {
          return -1;
        }
        return 0;
      });
    };

    workers["hydra:member"] = workers["hydra:member"].sort(function (a, b) {
      if (a.spreadTill > b.spreadTill) {
        return -1;
      }
      if (a.spreadTill < b.spreadTill) {
        return 1;
      }
      return 0;
    });

    onlineWorkers = sortByAlphabet(onlineWorkers, "name");
    offlineWorkers = sortByAlphabet(offlineWorkers, "name");

    const sortedWorkers = onlineWorkers.concat(offlineWorkers);

    if (sectionId) {
      let page = 0;
      while (itemsPerPage * page++ < totalRows) {
        const tasksReponse = yield call(request, COLLECTION_TASKS, {
          ...params,
          page,
        });
        tasks = tasksReponse ? tasksReponse["hydra:member"] : [];
        totalItems = tasksReponse ? tasksReponse["hydra:totalItems"] : 0;
        yield put(
          actions.loadFromAgroInvestSuccess(
            tasks,
            !currentTaskId && sortedWorkers,
            totalItems
          )
        );
      }
    } else {
      yield put(
        actions.loadFromAgroInvestSuccess(
          [],
          !currentTaskId && sortedWorkers,
          0
        )
      );
    }
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadLastFinishedTaskFromAgroinvest({ payload }) {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const { sectionId, rowNumber } = payload;

    const params = {
      greenhouseId,
      sectionId,
      rowNumber,
      isFinished: true,
      order: {
        finishedAt: "desc",
      },
      itemsPerPage: 1,
    };

    let data = yield call(request, "tasks", params);

    const lastTask = data["hydra:member"][0];

    if (lastTask) {
      yield put(actions.changeTask("qualityTaskId", lastTask.id));
    }

    yield put(actions.loadLastFinishedTaskFromAgroInvestSuccess(lastTask));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* changeTaskStatus({ payload }) {
  try {
    // const page = yield select(getPage);
    const selectedTasks = yield select(getSelectedTasks);
    const selectedWorkers = yield select(getSelectedWorkers);
    const sectionId = yield select(getSectionId);
    const pass = yield select(getPass);
    const shift = yield select(getShift);
    const direction = yield select(getDirection);
    const planDay = yield select(getPlanDay);
    const keepPlanDay = yield select(getKeepPlanDay);
    const keepOrderBy = yield select(getKeepOrderBy);
    const keepWorkerId = yield select(getKeepWorkerId);
    const keepExclusive = yield select(getKeepExclusive);
    const exclusive = yield select(getExclusive);
    const day = yield select(getDay);
    // const itemsPerPage = yield select(getItemsPerPage);

    const tasks = [];
    let sortedTasks = [];

    _.each(Object.keys(selectedTasks), (section) => {
      _.each(selectedTasks[section], (task) => {
        if (_.isNull(task.taskId)) {
          return;
        }

        sortedTasks.push(task);
      });
    });

    sortedTasks = _.sortBy(sortedTasks, [
      (t) => {
        return t.rowNumber;
      },
    ]);

    _.each(sortedTasks, (task, index) => {
      const params = {};

      if (!keepOrderBy) {
        params.orderBy = _.toNumber(
          "1" +
            shift +
            _.padStart(
              "" + (direction ? task.rowNumber : 540 - task.rowNumber),
              3,
              "0"
            )
        );
      }

      if (!keepExclusive) {
        params.exclusive = exclusive;
      }

      if (payload && selectedWorkers.length) {
        if (!keepWorkerId || !task.workerId) {
          params.forceWorkerId =
            selectedWorkers[index % selectedWorkers.length].id;

          params.lockedTill = moment()
            .utc()
            .add(1, "days")
            .set("hour", 6)
            .set("minute", 30)
            .format();
        }
      }

      if (payload && !keepPlanDay) {
        params.startAt = moment(day)
          .utc()
          .isoWeekday(planDay)
          .startOf("day")
          .format();
      }

      tasks.push({ task, params });
    });

    yield all(
      tasks.map((pair) =>
        call(
          request,
          COLLECTION_TASKS +
            "/" +
            pair.task.taskId +
            "/" +
            (payload ? "activate" : "deactivate"),
          pair.params,
          "PATCH"
        )
      )
    );

    yield put(actions.loadFromAgroInvest(sectionId, pass));
  } catch (error) {
    console.log(error);
    yield put(actions.tasksError(error));
  }
}

function* getSectionsGreenhouse(action) {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const greenhouse = yield call(
      request,
      COLLECTION_GREENHOUSE_NAME + "/" + greenhouseId,
      {      
      },
      "GET"
    );

    yield put(
      actions.getSectionsGreenhouseSuccess(greenhouse.greenhouseSections)
    );
  } catch (err) {
    console.log(err);
    yield put(actions.getSectionsGreenhouseFailure());
  }
}

function* getSections() {
  try {
    const sections = yield call(
      request,
      COLLECTION_SECTIONS,
      { isActive: true, itemsPerPage: 100 },
      "GET"
    );

    yield put(actions.getSectionsSuccess(sections["hydra:member"]));
  } catch (error) {
    console.error(error);
    yield put(actions.getSectionsFailure(error.message));
  }
}

function* checkPresenceTaskPass(action) {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const { sectionId, day, countPass } = action.payload;
    // const date = new Date()
    const passActive = [];

    for (let i = 0; i < countPass; i++) {
      const response = sectionId
        ? yield call(
            request,
            COLLECTION_TASKS,
            {
              sectionId,
              greenhouseId,
              startAt: {
                before: moment(day).utc().endOf("isoWeek").format(),
                after: moment(day).utc().startOf("isoWeek").format(),
              },
              pass: i,
              page: 1,
              itemsPerPage: 0,
            },
            "GET"
          )
        : null;

      if (response && response["hydra:totalItems"] > 0) {
        passActive.push(true);
      } else {
        passActive.push(false);
      }
    }

    yield put(actions.checkTaskPassSuccess(passActive));
  } catch (err) {
    console.log("err");
    yield put(actions.checkTaskPassFailure());
  }
}

function* getTask(action) {
  const { taskId } = action.payload;
  try {
    if (taskId) {
      const task = yield call(
        request,
        COLLECTION_TASKS + "/" + taskId,
        {},
        "GET"
      );

      yield put(actions.getTaskSuccess(task));
    }
  } catch (err) {
    console.log(err);
    yield put(actions.getTaskError());
  }
}

function* updateTask(action) {
  try {
    const taskId = yield select(getTaskId);
    const task = yield select(getTaskData);
    const tasks = yield select(getTasks);
    const params = { ...action.payload };

    const prevTaskData = _.find(tasks, (task) => task.id === taskId);
    const isChangedWorkerId =
      prevTaskData && prevTaskData.workerId != task.workerId;

    if (
      isChangedWorkerId &&
      !_.includes([null, "NOT_ASSIGNED"], task.workerId)
    ) {
      const taskMetricData = {
        task: `tasks/${task.id}`,
        worker: `workers/${task.workerId}`,
        type: "assign",
        registeredAt: moment().format(),
      };

      yield call(request, "task_metrics", taskMetricData, "POST");
    }

    if (params.workerId) {
      delete params.workerId;
      params.forceWorkerId =
        action.payload.workerId === "NOT_ASSIGNED"
          ? null
          : action.payload.workerId;

      params.lockedTill = params.forceWorkerId
        ? moment().utc().add(1, "days").set("hour", 6).set("minute", 30).format()
        : moment().format();
    }

    const response = yield call(
      request,
      COLLECTION_TASKS + "/" + taskId,
      params,
      "PATCH"
    );

    yield put(actions.patchTaskSuccess(response));
  } catch (err) {
    console.log(err);
    yield put(actions.patchTaskError());
  }
}

export default function* rootSaga() {
  yield all([
    takeLatest(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(
      actions.LOAD_LAST_FINISHED_TASK_FROM_AGROINVEST,
      loadLastFinishedTaskFromAgroinvest
    ),
    takeEvery(actions.ENABLE_TASKS, changeTaskStatus),
    takeEvery(actions.DISABLE_TASKS, changeTaskStatus),
    takeEvery(actions.SET_PAGE, loadFromAgroinvest),
    takeEvery(actions.SECTIONS_GREENHOUSE, getSectionsGreenhouse),
    takeEvery(actions.CHECK_PRESENCE_TASK_PASS, checkPresenceTaskPass),
    takeEvery(actions.GET_SECTIONS, getSections),
    takeEvery(actions.GET_SELECTED_TASK, getTask),
    takeEvery(actions.PATCH_TASK, updateTask),
  ]);
}
