import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import appActions from '../app/actions';
import _ from 'lodash';
import { request } from '@iso/lib/helpers/utility';
import { Notification } from 'antd';
import uuidV4 from 'uuid/v4';
import moment from 'moment';

const COLLECTION_CONFIGURATIONS = 'configurations';
const COLLECTION_TECHNICAL_WINDOWS = 'technical_windows';
const COLLECTION_GREENHOUSE_SECTIONS = `greenhouse_sections`;
const COLLECTION_TECHNICAL_WINDOW_SECTIONS = `technical_window_sections`;

const getGreenhouseId = (state) => state.App.greenhouseId;
const getTechnicalWindows = (state) => state.Setting.technicalWindows;

function* loadFromAgroinvest() {
  try {
    const params = {
      properties: ['id', 'config'],
      id: [
        'workingTimeConfig',
        'numberOfTask',
        'numberOfTaskForController',
        'freqRating',
      ],
    };

    const settingsData = yield call(request, COLLECTION_CONFIGURATIONS, params);

    const settings = {};

    const settingsIds = _.map(
      settingsData['hydra:member'],
      (setting) => setting.id
    );

    _.forEach(settingsIds, (id) => {
      const currentSetting = _.find(
        settingsData['hydra:member'],
        (setting) => setting.id === id
      );

      settings[id] = currentSetting.config;
    });

    yield put(
      actions.loadFromAgroInvestSuccess({
        settings,
      })
    );
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadSections() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const responseGreenhouseSections = yield call(
      request,
      COLLECTION_GREENHOUSE_SECTIONS,
      {
        greenhouseId,
        itemsPerPage: 100,
        order: {
          'section.name': 'asc',
        },
      }
    );

    const responseTechnicalWindowSections = yield call(
      request,
      COLLECTION_TECHNICAL_WINDOW_SECTIONS,
      {
        ['technicalWindow.greenhouseId']: greenhouseId,
        itemsPerPage: 100,
        order: {
          'section.name': 'asc',
        },
      }
    );

    yield put(
      actions.loadSectionsSuccess({
        greenhouseSections: responseGreenhouseSections['hydra:member'],
        redifinedSections: responseTechnicalWindowSections['hydra:member'],
      })
    );
  } catch (error) {
    yield put(actions.loadSectionsError(error));
  }
}

function* saveIntoAgroinvest({ payload }) {
  try {
    const { settings } = payload;

    const savedSettings = yield all(
      _.map(_.keys(settings), (settingKey) => {
        const config = settings[settingKey];
        return call(
          request,
          COLLECTION_CONFIGURATIONS + '/' + settingKey,
          {
            config,
          },
          'PATCH'
        );
      })
    );

    const newSettings = {};

    const settingsIds = _.map(savedSettings, (setting) => setting.id);

    _.forEach(settingsIds, (id) => {
      const currentSetting = _.find(
        savedSettings,
        (setting) => setting.id === id
      );
      newSettings[id] = currentSetting.config;
    });

    yield put(actions.saveIntoAgroInvestSuccess(settings));
    yield put(appActions.loadAppSettingsFromAgroInvest());
    yield call(Notification.success, {
      message: 'Настройки успешно сохранены',
      duration: 5,
    });
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadTechnicalWindows() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const technical_windows = yield call(
      request,
      COLLECTION_TECHNICAL_WINDOWS,
      {
        greenhouseId,
        itemsPerPage: 100,
      }
    );

    yield put(
      actions.loadTechnicalWindowsSuccess({
        technicalWindows: technical_windows['hydra:member'].sort(
          (a, b) => a.id - b.id
        ),
      })
    );
  } catch (error) {
    console.error(error);
    yield put(actions.loadTechnicalWindowsError(error));
  }
}

function* addTechnicalWindow() {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const technicalWindow = yield call(
      request,
      COLLECTION_TECHNICAL_WINDOWS,
      {
        id: uuidV4(),
        greenhouse: `greenhouses/${greenhouseId}`,
        start: moment('8:00', 'HH:mm').utc().format('HH:mm'),
        finish: moment('20:00', 'HH:mm').utc().format('HH:mm'),
      },
      'POST'
    );

    yield call(Notification.success, {
      message: 'Новое технологическое окно успешно добавлено',
      duration: 5,
    });
    yield put(actions.addTechnicalWindowSuccess(technicalWindow));
  } catch (error) {
    yield call(Notification.error, {
      message: error,
      duration: 5,
    });
    yield put(actions.addTechnicalWindowError(error));
  }
}

function* changeTechnicalWindow({ payload }) {
  try {
    const { id, paramName, paramValue } = payload;
    const technicalWindow = yield call(
      request,
      `${COLLECTION_TECHNICAL_WINDOWS}/${id}`,
      {
        [paramName]: paramValue,
      },
      'PATCH'
    );

    yield put(actions.changeTechnicalWindowSuccess({ technicalWindow }));
  } catch (error) {
    yield call(Notification.error, {
      message: error,
      duration: 5,
    });
    yield put(actions.addTechnicalWindowError(error));
  }
}

function* removeTechnicalWindow({ payload: { id } }) {
  try {
    yield call(request, `${COLLECTION_TECHNICAL_WINDOWS}/${id}`, {}, 'DELETE');

    yield call(Notification.success, {
      message: 'Новое технологическое окно успешно удалено',
      duration: 5,
    });
    yield put(actions.removeTechnicalWindowSuccess(id));
    yield put(actions.loadSections());
    yield put(actions.loadTechnicalWindows());
  } catch (error) {
    yield call(Notification.error, {
      message: error,
      duration: 5,
    });
    yield put(actions.removeTechnicalWindowError(error));
  }
}

function* addTechnicalWindowSection({
  payload: { technicalWindowId, sectionId },
}) {
  try {
    const technicalWindowSection = yield call(
      request,
      COLLECTION_TECHNICAL_WINDOW_SECTIONS,
      {
        id: uuidV4(),
        technicalWindow: `technical_windows/${technicalWindowId}`,
        section: `sections/${sectionId}`,
      },
      'POST'
    );
    yield put(actions.addTechnicalWindowSectionSuccess(technicalWindowSection));
    yield put(actions.loadSections());
    yield put(actions.loadTechnicalWindows());
  } catch (error) {
    console.error(error);
    yield put(actions.addTechnicalWindowSectionError(error));
  }
}

function* addTechnicalWindowSections({
  payload: { technicalWindowId, greenhouseSections },
}) {
  try {
    yield all(
      greenhouseSections.map((g) =>
        call(
          request,
          COLLECTION_TECHNICAL_WINDOW_SECTIONS,
          {
            id: uuidV4(),
            technicalWindow: `technical_windows/${technicalWindowId}`,
            section: `sections/${g.sectionId}`,
          },
          'POST'
        )
      )
    );
    yield put(actions.addTechnicalWindowSectionsSuccess());
    yield put(actions.loadSections());
    yield put(actions.loadTechnicalWindows());
  } catch (error) {
    console.error(error);
    yield put(actions.addTechnicalWindowSectionError(error));
  }
}

function* removeTechnicalWindowSection({ payload }) {
  try {
    const { sectionId, technicalWindowId } = payload;
    const technicalWindowSection = yield call(
      request,
      COLLECTION_TECHNICAL_WINDOW_SECTIONS,
      {
        technicalWindowId,
        sectionId,
      }
    );

    yield call(
      request,
      `${COLLECTION_TECHNICAL_WINDOW_SECTIONS}/${technicalWindowSection['hydra:member'][0].id}`,
      {},
      'DELETE'
    );

    yield put(actions.loadSections());
    yield put(actions.loadTechnicalWindows());
    yield put(actions.removeTechnicalWindowSuccess());
  } catch (error) {
    yield call(Notification.error, {
      message: error,
      duration: 5,
    });
    yield put(actions.removeTechnicalWindowError(error));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, saveIntoAgroinvest),
    takeEvery(actions.LOAD_TECHNICAL_WINDOWS, loadTechnicalWindows),
    takeEvery(actions.ADD_TECHNICAL_WINDOW, addTechnicalWindow),
    takeEvery(actions.CHANGE_TECHNICAL_WINDOW, changeTechnicalWindow),
    takeEvery(actions.REMOVE_TECHNICAL_WINDOW, removeTechnicalWindow),
    takeEvery(actions.ADD_TECHNICAL_WINDOW_SECTION, addTechnicalWindowSection),
    takeEvery(
      actions.ADD_TECHNICAL_WINDOW_SECTIONS,
      addTechnicalWindowSections
    ),
    takeEvery(
      actions.REMOVE_TECHNICAL_WINDOW_SECTION,
      removeTechnicalWindowSection
    ),
    takeEvery(actions.LOAD_SECTIONS, loadSections),
  ]);
}
