import { all, takeEvery, put, call, select } from "redux-saga/effects";
import _ from "lodash";
import { isJSON, request } from "@iso/lib/helpers/utility";
import uuidV4 from "uuid/v4";
import actions from "./actions";

import moment from "moment-timezone";
import "moment/locale/ru";

moment.tz.setDefault("Europe/Moscow");
moment.locale("ru");

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = "task_metrics"; // change your collection
const COLLECTION_GREENHOUSE_NAME = "greenhouses"; // change your collection

const getPage = (state) => state.TaskMetric.page;
const getItemsPerPage = (state) => state.TaskMetric.itemsPerPage;
const getOrderBy = (state) => state.TaskMetric.orderBy;
const getFilters = (state) => state.TaskMetric.filters;
const getGreenhouseId = (state) => state.App.greenhouseId;
const totalItems = (state) => state.TaskMetric.totalItems;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);
    const greenhouseId = yield select(getGreenhouseId);

    if (filters.code) {
      const isUuid = !!filters.code.match(
        /^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/
      );

      if (!isUuid) filters.code = "00000000-0000-4000-8000-000000000000";
    }

    const params = {};

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split("|"), v.split("|")));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    if (greenhouseId) params["task.greenhouseId"] = greenhouseId;
    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;

    let data = yield call(request, COLLECTION_NAME, params);
    yield put(
      actions.loadFromAgroInvestSuccess(
        data["hydra:member"],
        data["hydra:totalItems"]
      )
    );
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadCurrentPlantsFromAgroInvest({ payload }) {
  try {
    const greenhouseId = payload;

    const params = {
      itemsPerPage: 200,
    };

    if (greenhouseId) params["row.greenhouseId"] = greenhouseId;

    const plants = yield call(request, "plant_phenologies", params);
    const sortPlants = _.sortBy(plants["hydra:member"], [
      "row.rowNumber",
      (p) => +_.get(p, "name", "").match(/\d+/),
    ]);
    yield put(actions.loadCurrentPlantsFromAgroInvestSuccess(sortPlants));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadSectionsGreenhouse(action) {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    if (!greenhouseId) actions.getSectionsGreenhouseSuccess([]);

    const greenhouse = yield call(
      request,
      COLLECTION_GREENHOUSE_NAME + "/" + greenhouseId,
      {},
      "GET"
    );

    yield put(
      actions.loadSectionsGreenhouseSuccess(greenhouse.greenhouseSections)
    );
  } catch (error) {
    console.log(error);
    yield put(actions.loadFromAgroInvest(error));
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data } = payload;
  try {
    const params = {
      value: data.value,
    };

    if (!data.id) {
      yield call(
        request,
        COLLECTION_NAME,
        {
          ...params,
          id: uuidV4(),
        },
        "POST"
      );
    } else {
      yield call(request, COLLECTION_NAME + "/" + data.id, params, "PATCH");
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* prepareDownloadMetric() {
  const itemsPerPage = yield select(totalItems);
  const orderBy = yield select(getOrderBy);
  const filters = yield select(getFilters);
  const greenhouseId = yield select(getGreenhouseId);

  if (filters.code) {
    const isUuid = !!filters.code.match(
      /^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/
    );

    if (!isUuid) filters.code = "00000000-0000-4000-8000-000000000000";
  }

  const params = {};

  Object.keys(filters).flatMap((key) => {
    if (!_.isEmpty(filters[key])) {
      if (_.isArray(filters[key])) {
        _.each(filters[key], (v, k) => {
          if (_.isString(v) && isJSON(v)) {
            params[key] = _.merge(params[key], JSON.parse(v));
          } else if (_.isString(v)) {
            _.merge(params, _.zipObject(key.split("|"), v.split("|")));
          } else {
            params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
          }
        });
      } else {
        params[key] = _.isObject(filters[key])
          ? _.merge(params[key], filters[key])
          : filters[key];
      }
    }
  });

  if (greenhouseId) params["task.greenhouseId"] = greenhouseId;

  params.order = orderBy;
  params.itemsPerPage = itemsPerPage;

  const data = yield call(request, COLLECTION_NAME, params);
  const metrics = data["hydra:member"];

  const dataSource = [];
  Object.keys(metrics).map((metric, index) => {
    return dataSource.push({
      ...metrics[metric],
      key: metric,
    });
  });

  const columns = [
    {
      title: "КОД",
      dataIndex: "code",
      csvText: (row) => `${row.code} \n`,
    },
    {
      title: "Растение",
      dataIndex: "plantId",
      csvText: (row) => `
        Ряд: ${_.get(row, "plant.row.rowNumber", "-")}, ${_.get(
        row,
        "plant.name",
        "n/a"
      )}
        id: ${_.get(row, "plantId", "n/a")}
      `,
    },
    {
      title: "Метрика",
      dataIndex: "propertyId",
      csvText: (row) => `
        Группа: ${_.get(row, "parent.name", "n/a")} \n
        Метрика: ${_.get(row, "property.name", "n/a")}
      `,
    },
    {
      title: "Детали",
      dataIndex: "taskId",
      csvText: (row) => `
        Сотрудник: ${_.get(row, "worker.name", "n/a")} \n
        Задача: ${_.get(row, "task.name", "n/a")}
      `,
    },
    {
      title: "Дата",
      dataIndex: "registeredAt",
      csvText: (row) => moment(row.registeredAt).calendar(),
    },
    {
      title: "Значение",
      dataIndex: "value",
      csvText: (row) =>
        row.property.type === "binary" && !!row.value
          ? "Red dot"
          : _.get(row, "value", "n/a"),
    },
  ];

  const csvColumns = _.map(columns, (column) => {
    return {
      id: column.dataIndex,
      displayName: column.title,
    };
  });

  const csvRows = _.map(dataSource, (row) => {
    const csvRow = _.reduce(
      columns,
      (acc, curr) => {
        return {
          ...acc,
          [curr.dataIndex]: curr.csvText(row),
        };
      },
      {}
    );
    return csvRow;
  });

  yield put({
    type: actions.PREPARE_DOWNLOAD_SUCCESS,
    payload: {
      csvColumns,
      csvRows,
    },
  });
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(
      actions.LOAD_CURRENT_PLANTS_FROM_AGROINVEST,
      loadCurrentPlantsFromAgroInvest
    ),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.LOAD_SECTIONS_GREENHOUSE, loadSectionsGreenhouse),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.PREPARE_DOWNLOAD, prepareDownloadMetric),
  ]);
}
