import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import { isJSON, request } from '@iso/lib/helpers/utility';
import _ from 'lodash';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */
const COLLECTION_GREENHOUSE_SECTIONS = 'greenhouse_sections';
const COLLECTION_CHECK_METRICS = 'check_metrics';
const COLLECTION_CHECK_METRIC_QUALITY_LIMITS = 'check_metric_quality_limits';
const COLLECTION_SECTION_QUALITY_TIME_LIMITS = 'section_quality_time_limits';

const getGreenhouseId = (state) => state.App.greenhouseId;
const getGreenhouses = (state) => state.App.greenhouses;

const getFilters = (state) => state.TaskQuality.filters;

const categoryTranslate = {
  Огурцы: 'cucumber',
  Томаты: 'tomato',
};

function* loadFromAgroinvest() {
  try {
    const filters = yield select(getFilters);
    const greenhouses = yield select(getGreenhouses);
    const greenhouseId = yield select(getGreenhouseId);
    const currentGreenhouseDescription = _.get(
      _.find(greenhouses, (g) => g.id === greenhouseId),
      'description',
      '',
    );
    const categoryMatch = currentGreenhouseDescription.match(/\(\S+?\)/);

    const category = categoryMatch ? categoryMatch[0].slice(1, categoryMatch[0].length - 1) : null;

    const params = {
      itemsPerPage: 1000,
    };

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      } else if (!_.isNil(filters[key])) {
        params[key] = filters[key];
      }
    });

    params.category = category;

    const responseData = yield call(request, COLLECTION_CHECK_METRICS, params);
    const checkMetrics = responseData['hydra:member'];

    const requests = checkMetrics.map(({ id }) => {
      const requestParam = {
        checkMetricId: id,
        itemsPerPage: 1000,
        page: 1,
      };
      return call(request, COLLECTION_CHECK_METRIC_QUALITY_LIMITS, requestParam);
    });
    const responseDatas = yield all(requests);

    const qualityLimits = _.filter(responseDatas, (r) => r['hydra:member'].length > 0).map(
      (data) => {
        const marks = _.groupBy(data['hydra:member'], 'mark');
        return {
          id: data['hydra:member'][0].checkMetricId,
          name: _.get(data['hydra:member'][0], 'checkMetric.name', 'empty'),
          ...marks,
        };
      },
    );

    const responseDataTimeLimits = yield call(request, COLLECTION_SECTION_QUALITY_TIME_LIMITS, {
      sectionId: filters.sectionId,
      category: categoryTranslate[category],
    });

    const marks = _.groupBy(responseDataTimeLimits['hydra:member'], 'mark');
    const timeLimits =
      responseDataTimeLimits['hydra:totalItems'] === 0
        ? []
        : [
            {
              type: 'Quality',
              name: 'Сумма баллов',
              ...marks,
            },
            {
              type: 'Time',
              name: 'Время выполнения (мин.)',
              ...marks,
            },
          ];

    yield put(actions.loadFromAgroInvestSuccess({ qualityLimits, timeLimits }));
  } catch (error) {
    console.error(error);
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* sectionsLoadFromAgroinvest() {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const params = {
      greenhouseId,
      'section.isQuality': false,
      'section.hasQualityCheck': true,
    };

    const data = yield call(request, COLLECTION_GREENHOUSE_SECTIONS, params);

    yield put(
      actions.sectionsLoadFromAgroInvestSuccess(data['hydra:member'], data['hydra:totalItems']),
    );
  } catch (error) {
    yield put(actions.sectionsLoadFromAgroInvestError(error));
  }
}

function* storeIntoAgroinvestCheckMetricLimits({ payload }) {
  const { data } = payload;
  try {
    const response = yield call(request, COLLECTION_CHECK_METRIC_QUALITY_LIMITS, {
      id: data.id,
    });
    if (response['hydra:totalItems'] === 1) {
      yield call(
        request,
        `${COLLECTION_CHECK_METRIC_QUALITY_LIMITS}/${data.id}`,
        { maxQuality: data.maxQuality, minQuality: data.minQuality },
        'PATCH',
      );
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* storeIntoAgroinvestSectionLimits({ payload }) {
  const { data } = payload;
  try {
    const response = yield call(request, COLLECTION_SECTION_QUALITY_TIME_LIMITS, {
      id: data.id,
    });
    if (response['hydra:totalItems'] === 1) {
      yield call(
        request,
        `${COLLECTION_SECTION_QUALITY_TIME_LIMITS}/${data.id}`,
        {
          maxQuality: data.maxQuality,
          minQuality: data.minQuality,
          maxTime: data.maxTime,
          minTime: data.minTime,
        },
        'PATCH',
      );
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SECTIONS_LOAD_FROM_AGROINVEST, sectionsLoadFromAgroinvest),
    takeEvery(
      actions.SAVE_INTO_AGROINVEST_CHECK_METRIC_LIMITS,
      storeIntoAgroinvestCheckMetricLimits,
    ),
    takeEvery(actions.SAVE_INTO_AGROINVEST_SECTION_LIMITS, storeIntoAgroinvestSectionLimits),

    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.RESET_FILTERS, loadFromAgroinvest),
  ]);
}
