import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import moment from 'moment';
import { isJSON, request } from '@iso/lib/helpers/utility';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = 'tasks'; // change your collection
const COLLECTION_TASK_METRIC_NAME = 'task_metrics';
const COLLECTION_SYNC_NAME = 'syncs'; // change your collection
const COLLECTION_GREENHOUSE_NAME = 'greenhouses';
const COLLECTION_WORKERS_NAME = 'workers';

const getPage = (state) => state.Tasks.page;
const getItemsPerPage = (state) => state.Tasks.itemsPerPage;
const getOrderBy = (state) => state.Tasks.orderBy;
const getFilters = (state) => state.Tasks.filters;
const getGreenhouseId = (state) => state.App.greenhouseId;
const totalItems = (state) => state.Tasks.totalItems;
const getTasks = (state) => state.Tasks.tasks;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);
    const greenhouseId = yield select(getGreenhouseId);

    const params = {
      itemsPerPage,
      page,
      // isActive: true,
      properties: [
        'name',
        'rowNumber',
        'exclusive',
        'workerId',
        'greenhouseId',
        'previousWorkerId',
        'workerName',
        'id',
        'startAt',
        'finishAt',
        'startedAt',
        'finishedAt',
        'lockedTill',
        'isFinished',
        'orderBy',
        'day',
        'shift',
        'direction',
        'isActive',
        'sectionName',
        'sectionId',
        'inProgress',
        'rowOrderBy',
        'status',
        'intervals',
        'dynamicRating',
        'rating',
        'isQuality',
        'qualityTaskId',
        'qualityTask',
        'pass',
      ],
      'properties[greenhouse]': ['description'],
      'properties[worker]': ['searchable', 'code', 'notes'],
      'properties[section]': ['name', 'module'],
      'properties[sync]': ['id', 'syncedAt', 'name', 'status', 'toSend'],
      order: orderBy,
    };

    if (greenhouseId) params.greenhouseId = greenhouseId;

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      }
    });

    if (params.startedAt && params.startedAt === 'N/A') {
      params.exists.startedAt = false;
    }

    const data = yield call(request, COLLECTION_NAME, params);
    yield put(actions.loadFromAgroInvestSuccess(data['hydra:member'], data['hydra:totalItems']));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadLastFinishedTaskFromAgroinvest({ payload }) {
  try {
    const greenhouseId = yield select(getGreenhouseId);

    const { sectionId, rowNumber } = payload;

    const params = {
      greenhouseId,
      sectionId,
      rowNumber,
      isFinished: true,
      order: {
        finishedAt: 'desc',
      },
      itemsPerPage: 1,
    };

    const data = yield call(request, COLLECTION_NAME, params);

    yield put(actions.loadLastFinishedTaskFromAgroInvestSuccess(data['hydra:member'][0]));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* sync1C({ payload }) {
  try {
    const params = payload.isPatch
      ? {
          toSend: true,
        }
      : {
          id: payload.syncId,
          task: `tasks/${payload.syncId}`,
          toSend: true,
        };

    yield call(
      request,
      COLLECTION_SYNC_NAME + (payload.isPatch ? `/${payload.syncId}` : ''),
      params,
      payload.isPatch ? 'PATCH' : 'POST',
    );

    yield put(actions.sync1CSuccess());
  } catch (error) {
    yield put(actions.sync1CError(error));
  }
}

function* storeIntoAgroinvest({ payload }) {
  const { data, actionName } = payload;
  try {
    const tasks = yield select(getTasks);

    const prevTaskData = _.find(tasks, (task) => task.id === data.id);
    const isChangedWorkerId = prevTaskData && prevTaskData.workerId != data.workerId;

    if (isChangedWorkerId && !_.includes([null, 'NOT_ASSIGNED'], data.workerId)) {
      const taskMetricData = {
        task: `tasks/${data.id}`,
        worker: `workers/${data.workerId}`,
        type: 'assign',
        registeredAt: moment().format(),
      };

      yield call(request, COLLECTION_TASK_METRIC_NAME, taskMetricData, 'POST');
    }

    switch (actionName) {
      case 'update':
        yield call(
          request,
          `${COLLECTION_NAME}/${data.id}`,
          {
            forceWorkerId: _.includes(['null', 'NOT_ASSIGNED'], data.workerId)
              ? null
              : data.workerId,
            lockedTill: data.workerId
              ? moment().add(1, 'days').set('hour', 9).set('minute', 30).format()
              : moment().format(),
            // properties: data.properties,
            isFinished: data.isFinished,
            forceStartedAt: data.startedAt,
            forceFinishedAt: data.finishedAt,
            qualityTaskId: data.qualityTaskId,
            exclusive: data.exclusive,
            rating: Number(data.rating),
          },
          'PATCH',
        );
        break;
      default:
        break;
    }
    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* assignTaskToWorker({ payload }) {
  try {
    const { taskId, workerId } = payload;

    const tasks = yield select(getTasks);

    const prevTaskData = _.find(tasks, (task) => task.id === taskId);
    const isChangedWorkerId = prevTaskData && prevTaskData.workerId != workerId;

    if (isChangedWorkerId && !_.includes([null, 'NOT_ASSIGNED'], workerId)) {
      const taskMetricData = {
        task: `tasks/${taskId}`,
        worker: `workers/${workerId}`,
        type: 'assign',
        registeredAt: moment().format(),
      };

      yield call(request, COLLECTION_TASK_METRIC_NAME, taskMetricData, 'POST');
    }

    const result = yield call(
      request,
      `${COLLECTION_NAME}/${taskId}`,
      {
        forceWorkerId: workerId,
        lockedTill: workerId
          ? moment().add(1, 'days').set('hour', 9).set('minute', 30).format()
          : moment().format(),
      },
      'PATCH',
    );

    yield put(actions.assignTaskToWorkerSuccess(result));
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* getSectionsGreenhouse(action) {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    if (!greenhouseId) actions.getSectionsGreenhouseSuccess([]);

    const greenhouse = yield call(
      request,
      `${COLLECTION_GREENHOUSE_NAME}/${greenhouseId}`,
      {},
      'GET',
    );

    yield put(actions.getSectionsGreenhouseSuccess(greenhouse.greenhouseSections));
  } catch (err) {
    console.log(err);
    yield put(actions.getSectionsGreenhouseFailure());
  }
}

function* getWorkersGreenhouse(action) {
  try {
    const greenhouseId = yield select(getGreenhouseId);
    const params = {
      // spreadTill: {
      //   after: moment().subtract(30, 'days').format('YYYY-MM-DDTHH:mm:ss'),
      // },
      // exists: {
      //   spreadTill: true,
      // },
      greenhouseId,
      itemsPerPage: 500,
      isActive: true,
      properties: ['id', 'name', 'code', 'notes'],
    };

    const data = yield call(request, COLLECTION_WORKERS_NAME, params, 'GET');

    yield put(actions.getWorkersSuccess(data['hydra:member']));
  } catch (err) {
    console.log(err);
    yield put(actions.getWorkersError());
  }
}

function* prepareDownloadTasks() {
  const itemsPerPage = yield select(totalItems);
  const orderBy = yield select(getOrderBy);
  const filters = yield select(getFilters);
  const greenhouseId = yield select(getGreenhouseId);

  if (filters.code) {
    const isUuid = !!filters.code.match(
      /^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/,
    );

    if (!isUuid) filters.code = '00000000-0000-4000-8000-000000000000';
  }

  const params = {};

  Object.keys(filters).flatMap((key) => {
    if (!_.isEmpty(filters[key])) {
      if (_.isArray(filters[key])) {
        _.each(filters[key], (v, k) => {
          if (_.isString(v) && isJSON(v)) {
            params[key] = _.merge(params[key], JSON.parse(v));
          } else if (_.isString(v)) {
            _.merge(params, _.zipObject(key.split('|'), v.split('|')));
          } else {
            params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
          }
        });
      } else {
        params[key] = _.isObject(filters[key]) ? _.merge(params[key], filters[key]) : filters[key];
      }
    }
  });

  if (greenhouseId) params.greenhouseId = greenhouseId;

  params.order = orderBy;
  params.itemsPerPage = itemsPerPage;

  const data = yield call(request, COLLECTION_NAME, params);
  const tasks = data['hydra:member'];

  const dataSource = [];
  Object.keys(tasks).map((task) =>
    dataSource.push({
      ...tasks[task],
      key: task,
    }),
  );

  const columns = [
    {
      title: 'Название',
      dataIndex: 'name',
      csvText: (row) => {
        const description = _.get(row, 'greenhouse.description');
        const pass = _.get(row, 'pass');
        return `
        Название: ${row.name} \n
        Теплица: ${description || 'n/a'} \n
        Проход: ${pass ? pass + 1 : 0 + 1}
      `;
      },
    },
    {
      title: 'Ряд',
      dataIndex: 'rowNumber',
      csvText: (row) => `
        Ряд: ${row.rowNumber} \n
        Группа: ${row.exclusive}
      `,
    },
    {
      title: 'Работник',
      dataIndex: 'workerName',
      csvText: (row) => {
        const workerName = _.get(row, 'worker.name');
        const code = _.get(row, 'worker.code');
        return `
        Работник: ${workerName || 'Работник не назначен'} ${code ? `/ ${code}` : ''} \n
        О работнике: ${_.get(row, 'worker.notes', '')}
      `;
      },
    },
    {
      title: 'Секция',
      dataIndex: 'section',
      csvText: (row) => {
        const sectionName = _.get(row, 'section.name');
        return `
        Секция: ${sectionName || ''} \n
        Рейтинг: ${row.rating}
      `;
      },
    },
    {
      title: 'Запланировано на',
      dataIndex: 'startAt',
      csvText: (row) => `
        Запланировано на: ${row.startAt ? moment(row.startAt).format('LL') : 'N/A'}
      `,
    },
    {
      title: 'Дата начала и окончания',
      dataIndex: 'finishedAt',
      csvText: (row) => `
        Дата начала и окончания: ${row.startedAt ? moment(row.startedAt).calendar() : 'N/A'} - ${
        row.finishedAt ? moment(row.finishedAt).calendar() : 'N/A'
      }
      `,
    },
    {
      title: 'Статус',
      dataIndex: 'isFinished',
      csvText: (row) => `
        Статус:  ${_.get(row, 'isFinished') ? 'Завершена' : 'Не завершена'}
      `,
    },
    {
      title: 'План',
      dataIndex: 'isActive',
      csvText: (row) => `
        План:  ${_.get(row, 'isActive') ? 'В плане' : 'Исключена'}
      `,
    },
    {
      title: 'Синхронизация',
      dataIndex: 'syncedAt',
      csvText: (row) => {
        const isFinished = _.get(row, 'isFinished');
        const startedAt = _.get(row, 'startedAt');
        const finishedAt = _.get(row, 'finishedAt');
        const time = _.get(row, 'sync.syncedAt');
        const id = _.get(row, 'sync.id');
        const nameOneS = _.get(row, 'sync.name');
        const statusOneS = _.get(row, 'sync.status');
        let statusText;

        if (_.isNull(startedAt) && _.isNull(finishedAt)) {
          statusText = 'Запланировано';
        } else if (!_.isNull(startedAt) && _.isNull(finishedAt)) {
          statusText = 'В работе';
        } else if (!_.isNull(startedAt) && !_.isNull(finishedAt)) {
          statusText = 'ПАУЗА';
        }
        return `
        Статус МУП: ${row.status ? row.status : isFinished ? 200 : statusText} \n
        Время: ${time ? moment(time).calendar() : 'N/A'} \n
        ID: ${id || 'N/A'} \n
        1C: ${nameOneS || 'N/A'} (${_.get(row, 'sync.toSend', false) ? 'V' : 'X'}) \n
        Статус из 1С: ${statusOneS || 'N/A'}
        `;
      },
    },
  ];

  const csvColumns = _.map(columns, (column) => ({
    id: column.dataIndex,
    displayName: column.title,
  }));

  const csvRows = _.map(dataSource, (row) => {
    const csvRow = _.reduce(
      columns,
      (acc, curr) => ({
        ...acc,
        [curr.dataIndex]: curr.csvText(row),
      }),
      {},
    );
    return csvRow;
  });

  yield put({
    type: actions.PREPARE_DOWNLOAD_SUCCESS,
    payload: {
      csvColumns,
      csvRows,
    },
  });
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.LOAD_LAST_FINISHED_TASK_FROM_AGROINVEST, loadLastFinishedTaskFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, storeIntoAgroinvest),
    takeEvery(actions.ASSIGN_TASK_TO_WORKER, assignTaskToWorker),
    takeEvery(actions.SYNC_1C, sync1C),
    takeEvery(actions.SYNC_1C_SUCCESS, loadFromAgroinvest),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.RESET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.SECTIONS_GREENHOUSE, getSectionsGreenhouse),
    takeEvery(actions.GET_WORKERS_GREENHOUSE, getWorkersGreenhouse),
    takeEvery(actions.PREPARE_DOWNLOAD, prepareDownloadTasks),
  ]);
}
