import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import _ from 'lodash';
import moment from 'moment';
import { isJSON, request } from '@iso/lib/helpers/utility';

import {  changePassword } from '@iso/lib/feathers/feathers.authentication.util';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_NAME = 'managers'; // change your collection

const getPage = state => state.Users.page;
const getItemsPerPage = state => state.Users.itemsPerPage;
const getOrderBy = state => state.Users.orderBy;
const getFilters = state => state.Users.filters;
const getUsers = state => state.Users.users;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const orderBy = yield select(getOrderBy);
    const filters = yield select(getFilters);

    const params = {}

    Object.keys(filters).flatMap(key => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else if (_.isString(v)) {
              _.merge(params, _.zipObject(key.split('|'), v.split('|')));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          })
        } else {
          params[key] = _.isObject(filters[key]) ? _.merge(params[key], filters[key]) : filters[key];
        }
      }
    });

    params.itemsPerPage = itemsPerPage;
    params.page = page;
    params.order = orderBy;

    const data = yield call(request, COLLECTION_NAME, params);
    yield put(actions.loadFromAgroInvestSuccess(data['hydra:member'], data['hydra:totalItems']));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* saveIntoAgroinvest({ payload }) {
  const { data } = payload;
  try {
    const users = yield select(getUsers)

    if (!data.id) {
      if (data.password !== data.confirmPassword) {
        yield put(actions.saveIntoAgroInvestError('Пароли не совпадают'));
        return
      }

      if (!data.name || !data.password || !data.email || !data.role) {
        yield put(actions.saveIntoAgroInvestError('Поля: Имя, Email, Пароль, Роль - должны быть заполнены'));
        return
      }

      const createUserParams = {
        name: data.name,
        email: data.email,
        role: data.role,
        lastLoggedAt: moment().format(),
        password: "password",
      }

      const result = yield call(request, COLLECTION_NAME, createUserParams, "POST");

      const managerGreenhouses = data.greenhouses || []

      if (result.id) {
        yield changePassword(result.id, data.password)
        yield all(managerGreenhouses.map(greenhouseId =>
          call(request, `manager_greenhouses`, { greenhouseId, manager: `managers/${result.id}`, isActive: true }, 'POST')
        ))
      } else {
        yield put(actions.saveIntoAgroInvestError('Ошибка сервера'));
        return
      }

    } else {
      if (data.password !== data.confirmPassword) {
        yield put(actions.saveIntoAgroInvestError('Пароли не совпадают'));
        return
      }

      if (data.password?.length > 0 && data.password === data.confirmPassword) {
        yield changePassword(data.id, data.password)
      }

      const currentGreenhouses = _.keyBy(
        _.get(
          _.find(users, user => user.id === data.id), 'greenhouses', []
        ), 'greenhouseId'
      )
      const greenhousesForEnable = []
      const greenhousesForCreate = []
      const greenhousesForDisable = []

      _.forEach(data.greenhouses, id => {
        if (_.has(currentGreenhouses, id)) {
          greenhousesForEnable.push(currentGreenhouses[id].id)
          delete currentGreenhouses[id]
        } else {
          greenhousesForCreate.push(id)
        }
      })

      _.forEach(Object.values(currentGreenhouses), mg => {
        greenhousesForDisable.push(mg.id)
      })

      yield all(greenhousesForEnable.map((id) => call(request, `manager_greenhouses/${id}`, { isActive: true }, 'PATCH')));
      yield all(greenhousesForDisable.map((id) => call(request, `manager_greenhouses/${id}`, { isActive: false }, 'PATCH')));
      yield all(greenhousesForCreate.map((id) =>
        call(request, `manager_greenhouses`, { greenhouseId: id, manager: `managers/${data.id}`, isActive: true }, 'POST'))
      );

      const managerParams = {
        name: data.name,
        email: data.email,
        role: data.role
      }

      yield call(request, `managers/${data.id}`, managerParams, 'PATCH')
    }

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error.message));
  }
}

function* deleteUser({ payload }) {
  const { data } = payload;
  try {
    yield all(data.greenhouses.map((g) => call(request, `manager_greenhouses/${g.id}`, {}, 'DELETE')));

    yield call(request, COLLECTION_NAME + '/' + data.id, {}, 'DELETE');

    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);
    yield put(actions.saveIntoAgroInvestError(error.message));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.SAVE_INTO_AGROINVEST, saveIntoAgroinvest),
    takeEvery(actions.DELETE_USER, deleteUser),
  ]);
}
