import { all, takeEvery, put, call, select } from 'redux-saga/effects';
import actions from './actions';
import moment from 'moment';
import _ from 'lodash';
import { initState } from './reducers';
import {
  isJSON,
  request,
  oneCRequest,
  camundaRequest,
  cacheRequest,
} from '@iso/lib/helpers/utility';

/**
 * DOC: https://redux-saga-firebase.js.org/reference/dev/AGROINVEST
 */

const COLLECTION_WORKER_NAME = 'workers'; // change your collection
const COLLECTION_RATING_NAME = 'ratings'; // change your collection
const COLLECTION_SECTION_NAME = 'sections'; // change your collection
const COLLECTION_GREENHOUSE_SECTIONS = 'greenhouse_sections';
const COLLECTION_CHECK_PRIORITIES = 'check_priorities';

const getPage = (state) => state.Workers.page;
const getItemsPerPage = (state) => state.Workers.itemsPerPage;
const getFilters = (state) => state.Workers.filters;
const getQuery = (state) => state.Workers.query;
const getWorker = (state) => state.Workers.worker;
const getTimelineDay = (state) => state.Workers.timelineDay;
const getChangeRatings = (state) => state.Workers.changeRatings;
const getManagerName = (state) => state.Auth.name;
// const getManagerId = (state) => state.Auth.id;
// const getGreenhouseId = state => state.App.greenhouseId;

function* loadFromAgroinvest() {
  try {
    const page = yield select(getPage);
    const itemsPerPage = yield select(getItemsPerPage);
    const filters = yield select(getFilters);
    // const greenhouseId = yield select(getGreenhouseId);

    const params = {
      // spreadTill: {
      //  after: moment()
      //    .subtract(30, 'days')
      //    .format('YYYY-MM-DDTHH:mm:ss'),
      // },
      // exists: {
      //   spreadTill: true,
      // },
      itemsPerPage,
      order: {
        isActive: 'desc',
        name: 'asc',
      },
      page,
    };

    // if (greenhouseId) params.greenhouseId = greenhouseId

    Object.keys(filters).flatMap((key) => {
      if (!_.isEmpty(filters[key])) {
        if (_.isArray(filters[key])) {
          _.each(filters[key], (v, k) => {
            if (_.isString(v) && isJSON(v)) {
              params[key] = _.merge(params[key], JSON.parse(v));
            } else {
              params[key] = _.isObject(v) ? _.merge(params[key], v) : v;
            }
          });
        } else {
          params[key] = _.isObject(filters[key])
            ? _.merge(params[key], filters[key])
            : filters[key];
        }
      } else if (!_.isNil(filters[key])) {
        params[key] = filters[key];
      }
    });

    const data = yield call(request, COLLECTION_WORKER_NAME, params);

    data['hydra:member'] = data['hydra:member'].map((worker) => {
      if (!worker.versionName) worker.versionName = 'empty';
      return worker;
    });

    yield put(actions.loadFromAgroInvestSuccess(data['hydra:member'], data['hydra:totalItems']));
  } catch (error) {
    yield put(actions.loadFromAgroInvestError(error));
  }
}

function* loadAutocomplete() {
  try {
    const page = 1;
    const itemsPerPage = 5;
    const query = yield select(getQuery);

    const data = yield call(request, COLLECTION_WORKER_NAME, {
      // spreadTill: {
      //  after: moment()
      //    .subtract(30, 'days')
      //    .format('YYYY-MM-DDTHH:mm:ss'),
      // },
      // exists: {
      //   spreadTill: true,
      // },
      properties: ['id', 'name', 'code'],
      searchable: query,
      isActive: true,
      itemsPerPage,
      page,
    });
    yield put(actions.loadAutocompleteSuccess(data['hydra:member']));
  } catch (error) {
    yield put(actions.loadAutocompleteError(error));
  }
}

function* loadTimeline() {
  try {
    const worker = yield select(getWorker);
    const day = yield select(getTimelineDay);

    const data = worker.id
      ? yield call(request, `${COLLECTION_WORKER_NAME}/${worker.id}/timeline`, {
          day: encodeURI(moment().day(day).format()),
        })
      : null;

    yield put(actions.loadTimelineSuccess(data['hydra:member']));
  } catch (error) {
    yield put(actions.loadTimelineError(error));
  }
}

function* load1C({ payload }) {
  try {
    const params = {
      variables: {
        payload: payload.workerId,
      },
      businessKey: 'AGROINVEST',
    };

    const response = yield call(
      request,
      `${COLLECTION_WORKER_NAME}/${payload.workerId}/import`,
      params,
    );

    yield put(actions.load1CSuccess(response));
  } catch (error) {
    yield put(actions.load1CError(error));
  }
}

function* loadCamunda1C({ payload }) {
  try {
    const params = {
      variables: {
        payload: {
          value: payload.workerId,
          type: 'String',
        },
      },
      businessKey: 'AGROINVEST',
    };

    const response = yield call(camundaRequest, 'WorkerImport', params);

    yield put(actions.loadCamunda1CSuccess(response));
  } catch (error) {
    yield put(actions.loadCamunda1CError(error));
  }
}

function* load1CData({ payload }) {
  try {
    const response = yield call(oneCRequest, payload.path);

    yield put(actions.load1CDataSuccess(response));
  } catch (error) {
    yield put(actions.load1CError(error));
  }
}

function* loadCache({ payload }) {
  try {
    const worker = yield select(getWorker);
    const tasksUrl = `/tasks/${worker.id}`;
    const tasksBetaUrl = `/t/${worker.id}/0`;
    const taskHistoryUrl = `/task-history/${worker.id}/0`;
    const trolleyHistoryUrl = `/trolley-history/${worker.id}/0`;
    const statusUrl = `/status/${worker.id}/0`;

    let tasks = null;

    if (_.includes(worker.versionName, 'Лоэнгрин')) {
      tasks = yield call(cacheRequest, tasksBetaUrl);
    } else {
      tasks = yield call(cacheRequest, tasksUrl);
    }

    const taskHistories = yield call(cacheRequest, taskHistoryUrl);
    const trolleyHistories = yield call(cacheRequest, trolleyHistoryUrl);
    const status = yield call(cacheRequest, statusUrl);

    yield put(
      actions.loadCacheSuccess({
        tasks,
        taskHistories,
        trolleyHistories,
        status,
      }),
    );
  } catch (error) {
    console.error(error);
    yield put(actions.loadCacheError(error));
  }
}

function* assignTasks({ payload }) {
  try {
    const response = yield call(
      request,
      `${COLLECTION_WORKER_NAME}/${payload.workerId}/assign`,
      null,
    );

    yield put(actions.assignTasksSuccess(response));
  } catch (error) {
    yield put(actions.assignTasksError(error));
  }
}

function* trolleyUnbind({ payload }) {
  try {
    const response = yield call(
      request,
      `${COLLECTION_WORKER_NAME}/${payload.worker.id}/trolley-unbind`,
      null,
    );

    yield put(actions.trolleyUnbindSuccess(response));
  } catch (error) {
    yield put(actions.trolleyUnbindError(error));
  }
}

function* switchNfc({ payload }) {
  try {
    const params = {
      nfcMode: !payload.worker.nfcMode,
    };

    const data = yield call(
      request,
      `${COLLECTION_WORKER_NAME}/${payload.worker.id}`,
      params,
      'PATCH',
    );

    yield call(request, `${COLLECTION_WORKER_NAME}/${payload.worker.id}/info`, null);

    yield put(actions.switchNfcSuccess(data));
  } catch (error) {
    console.error(error);
    yield put(actions.switchNfcError(error));
  }
}

function* makeScreenshot({ payload }) {
  try {
    const response = yield call(
      request,
      `${COLLECTION_WORKER_NAME}/${payload.workerId}/make-screenshot`,
      null,
    );
    yield put(actions.makeScreenshotSuccess(response));
  } catch (error) {
    console.error(error);
    yield put(actions.makeScreenshotError(error));
  }
}

function* uploadBackup({ payload }) {
  try {
    const response = yield call(
      request,
      `${COLLECTION_WORKER_NAME}/${payload.workerId}/upload-backup`,
      null,
    );
    yield put(actions.uploadBackupSuccess(response));
  } catch (error) {
    console.error(error);
    yield put(actions.uploadBackupError(error));
  }
}

function* signOut({ payload }) {
  try {
    const managerName = yield select(getManagerName);

    const params = {
      variables: {
        payload: {
          value: JSON.stringify({
            workerId: payload.workerId,
            reason: `Ручное завершение дня администратором: ${managerName}`,
          }),
          type: "Object",
          valueInfo: {
            objectTypeName: "java.util.HashMap<java.lang.Object,java.lang.Object>",
            serializationDataFormat: "application/json",
          },
        },
        type: {
          value: 'signOut',
          type: 'String',
        },
        meta: {
          value: JSON.stringify({
            model: "AGROMANAGER", 
            device: "WEB", 
            product: "BROWSER", 
            apiLevel: 999, 
            workerId: payload.workerId,
            osVersion: "999.999.999", 
            phoneName: "WEB BROWSER", 
            sessionId: "SESSION-ID", 
            businessKey: `signOut-${payload.workerId}`, 
            versionCode: 999, 
            versionName: "AGROMANAGER", 
            deltaTimestamp: 0, 
            deviceTimeMark: moment().format(), 
            serverTimeMark: moment().format(),
          }),
          type: "Object",
          valueInfo: {
            objectTypeName: "java.util.HashMap<java.lang.Object,java.lang.Object>",
            serializationDataFormat: "application/json",
          },
        },
      },
      businessKey: `signOut-${payload.workerId}`,
    };

    const response = yield call(camundaRequest, 'MobileRequests', params);

    yield put(actions.signOutSuccess(response));
  } catch (error) {
    console.error(error);
    yield put(actions.signOutError(error));
  }
}

function* saveIntoAgroinvest({ payload }) {
  const { data, actionName } = payload;

  try {
    switch (actionName) {
      case 'update':
        const worker = yield call(request, `${COLLECTION_WORKER_NAME}/${data.id}`, null);

        // обновление времени совпадающих рейтингов работника при переводе в другую теплицу
        if (data.greenhouseId !== worker.greenhouseId) {
          const responseGreenhouseSections = yield call(
            request,
            COLLECTION_GREENHOUSE_SECTIONS,
            {
              greenhouseId: data.greenhouseId,
              'section.hasQualityCheck': true,
            },
            'GET',
          );

          const hasQualitySectionIds = responseGreenhouseSections['hydra:member'].map(
            ({ sectionId }) => sectionId,
          );

          const ratings = worker.ratings.filter(
            ({ sectionId, isQuality }) =>
              hasQualitySectionIds.includes(sectionId) && isQuality === true,
          );

          yield all(
            ratings.map((r) =>
              call(
                request,
                `${COLLECTION_RATING_NAME}/${r.id}`,
                {
                  ratingUpdatedAt: moment().format(),
                },
                'PATCH',
              ),
            ),
          );
        }

        yield call(
          request,
          `${COLLECTION_WORKER_NAME}/${data.id}`,
          {
            notes: data.notes,
            exclusive: data.exclusive,
            isActive: data.isActive,
            greenhouseId: data.greenhouseId,
            phoneMode: data.phoneMode,
            phoneName: data.phoneName,
          },
          'PATCH',
        );
        break;
      case 'patchCheckPriorities':
        yield all(
          _.map(data, (item) =>
            call(
              request,
              `${COLLECTION_CHECK_PRIORITIES}/${item.id}`,
              {
                priority: item.priority,
              },
              'PATCH',
            ),
          ),
        );

        break;
      default:
        break;
    }
    const changeRatings = yield select(getChangeRatings);
    if (changeRatings.length > 0) {
      // запись рейтинга без изменния времени обновления
      const patchRatings = {};

      _.each(changeRatings, (item) => {
        patchRatings[item.ratingId] = {
          ratingInt: item.value,
          rating: (item.value - 1) * 0.1,
          isQuality: item.isQuality,
        };
      });

      const responsePatchedRatings = yield all(
        Object.keys(patchRatings).map((id) =>
          call(request, `${COLLECTION_RATING_NAME}/${id}`, null),
        ),
      );

      yield all(
        Object.keys(patchRatings).map((id) =>
          call(request, `${COLLECTION_RATING_NAME}/${id}`, patchRatings[id], 'PATCH'),
        ),
      );

      yield all(
        responsePatchedRatings.map((data) =>
          call(
            request,
            `${COLLECTION_RATING_NAME}/${data.id}`,
            {
              ratingUpdatedAt: data.ratingUpdatedAt,
            },
            'PATCH',
          ),
        ),
      );
    }
    yield put(actions.saveIntoAgroInvestSuccess({}));
    yield put({ type: actions.LOAD_FROM_AGROINVEST });
  } catch (error) {
    console.log(error);

    yield put(actions.saveIntoAgroInvestError(error));
  }
}

function* getSectionsGreenhouse(action) {
  try {
    const params = {
      isActive: true,
      itemsPerPage: 100,
    };
    const data = yield call(request, COLLECTION_SECTION_NAME, params, 'GET');

    yield put(actions.getSectionsSuccess(data['hydra:member']));
  } catch (err) {
    console.error(err);
    yield put(actions.getSectionsError);
  }
}

function* toggleHandleModalLoad({ payload }) {
  let worker = { ...initState.worker };
  try {
    if (payload !== null) {
      const responseCheckPriorities = yield call(request, COLLECTION_CHECK_PRIORITIES, {
        workerId: payload.id,
        itemsPerPage: 100,
        page: 1,
      });
      worker = {
        ...payload,
        checkPriorities: responseCheckPriorities['hydra:member'],
      };
    }

    yield put(
      actions.toggleModalLoadSuccess({
        worker,
      }),
    );
  } catch (err) {
    console.error(err);
    yield put(actions.toggleModalLoadError(err));
  }
}

export default function* rootSaga() {
  yield all([
    takeEvery(actions.LOAD_FROM_AGROINVEST, loadFromAgroinvest),
    takeEvery(actions.SET_PAGINATION, loadFromAgroinvest),
    takeEvery(actions.LOAD_AUTOCOMPLETE, loadAutocomplete),
    takeEvery(actions.LOAD_1C, load1C),
    takeEvery(actions.LOAD_CAMUNDA_1C, loadCamunda1C),
    takeEvery(actions.SWITCH_NFC, switchNfc),
    takeEvery(actions.TROLLEY_UNBIND, trolleyUnbind),
    takeEvery(actions.SWITCH_NFC_SUCCESS, loadFromAgroinvest),
    takeEvery(actions.ASSIGN_TASKS, assignTasks),
    takeEvery(actions.ASSIGN_TASKS_SUCCESS, loadFromAgroinvest),
    takeEvery(actions.SET_AUTOCOMPLETE, loadAutocomplete),
    takeEvery(actions.SET_FILTERS, loadFromAgroinvest),
    takeEvery(actions.TOGGLE_TIMELINE, loadTimeline),
    takeEvery(actions.SET_TIMELINE_DAY, loadTimeline),
    takeEvery(actions.LOAD_1C_DATA, load1CData),
    takeEvery(actions.LOAD_CACHE, loadCache),
    takeEvery(actions.SAVE_INTO_AGROINVEST, saveIntoAgroinvest),
    takeEvery(actions.MAKE_SCREENSHOT, makeScreenshot),
    takeEvery(actions.UPLOAD_BACKUP, uploadBackup),
    takeEvery(actions.SIGN_OUT, signOut),
    takeEvery(actions.SECTIONS_GREENHOUSE, getSectionsGreenhouse),
    takeEvery(actions.TOGGLE_HANDLE_MODAL_LOAD, toggleHandleModalLoad),
  ]);
}
